@@ -5,15 +5,15 @@
 #ifndef GEMMI_TO_MMCIF_HPP_
 #define GEMMI_TO_MMCIF_HPP_
 
-#include <cassert>
-#include <cmath>  // for isnan
 #include "model.hpp"
 #include "cifdoc.hpp"
 
 namespace gemmi {
 
-void update_cif_block(const Structure& st, cif::Block& block);
+void update_cif_block(const Structure& st, cif::Block& block, bool with_atoms);
 cif::Document make_mmcif_document(const Structure& st);
+cif::Block make_mmcif_headers(const Structure& st);
+void add_minimal_mmcif_data(const Structure& st, cif::Block& block);
 
 // temporarily we use it in crdrst.cpp
 namespace impl {
@@ -24,10 +24,12 @@ void write_struct_conn(const Structure& st, cif::Block& block);
 
 #ifdef GEMMI_WRITE_IMPLEMENTATION
 
+#include <cassert>
+#include <cmath>  // for isnan
 #include <string>
 #include <utility>  // std::pair
 #include "sprintf.hpp"
-#include "entstr.hpp" // for entity_type_to_string, polymer_type_to_string
+#include "entstr.hpp" // for entity_type_to_string, polymer_type_to_qstring
 #include "calculate.hpp"  // for count_atom_sites
 
 namespace gemmi {
@@ -64,6 +66,13 @@ inline std::string string_or_qmark(const std::string& s) {
   return s.empty() ? "?" : cif::quote(s);
 }
 
+// Quote chain name or entity id if necessary. It is necessary
+// only if the chain name is missing, which was OK in the past.
+// Here we use '' rather than . or ?.
+inline std::string qchain(const std::string& s) {
+  return cif::quote(s);
+}
+
 
 inline void add_cif_atoms(const Structure& st, cif::Block& block) {
   // atom list
@@ -92,28 +101,28 @@ inline void add_cif_atoms(const Structure& st, cif::Block& block) {
   for (const Model& model : st.models) {
     for (const Chain& chain : model.chains) {
       for (const Residue& res : chain.residues) {
-        std::string label_seq_id = res.label_seq.str();
+        std::string label_seq_id = res.label_seq.str('.');
         std::string auth_seq_id = res.seqid.num.str();
-        for (const Atom& a : res.atoms) {
+        for (const Atom& atom : res.atoms) {
           vv.emplace_back(std::to_string(++serial));
-          vv.emplace_back(a.element.uname());
-          vv.emplace_back(a.name);
-          vv.emplace_back(1, a.altloc_or('.'));
+          vv.emplace_back(atom.element.uname());
+          vv.emplace_back(cif::quote(atom.name));
+          vv.emplace_back(1, atom.altloc_or('.'));
           vv.emplace_back(res.name);
           vv.emplace_back(subchain_or_dot(res));
           vv.emplace_back(label_seq_id);
           vv.emplace_back(pdbx_icode(res));
-          vv.emplace_back(to_str(a.pos.x));
-          vv.emplace_back(to_str(a.pos.y));
-          vv.emplace_back(to_str(a.pos.z));
-          vv.emplace_back(to_str(a.occ));
-          vv.emplace_back(to_str(a.b_iso));
-          vv.emplace_back(a.charge == 0 ? "?" : std::to_string(a.charge));
+          vv.emplace_back(to_str(atom.pos.x));
+          vv.emplace_back(to_str(atom.pos.y));
+          vv.emplace_back(to_str(atom.pos.z));
+          vv.emplace_back(to_str(atom.occ));
+          vv.emplace_back(to_str(atom.b_iso));
+          vv.emplace_back(atom.charge == 0 ? "?" : std::to_string(atom.charge));
           vv.emplace_back(auth_seq_id);
-          vv.emplace_back(cif::quote(chain.name));
+          vv.emplace_back(impl::qchain(chain.name));
           vv.emplace_back(model.name);
-          if (a.u11 != 0.f)
-            aniso.emplace_back(serial, &a);
+          if (atom.aniso.nonzero())
+            aniso.emplace_back(serial, &atom);
         }
       }
     }
@@ -128,12 +137,118 @@ inline void add_cif_atoms(const Structure& st, cif::Block& block) {
     aniso_val.reserve(aniso_loop.tags.size() * aniso.size());
     for (const auto& a : aniso) {
       aniso_val.emplace_back(std::to_string(a.first));
-      aniso_val.emplace_back(to_str(a.second->u11));
-      aniso_val.emplace_back(to_str(a.second->u22));
-      aniso_val.emplace_back(to_str(a.second->u33));
-      aniso_val.emplace_back(to_str(a.second->u12));
-      aniso_val.emplace_back(to_str(a.second->u13));
-      aniso_val.emplace_back(to_str(a.second->u23));
+      aniso_val.emplace_back(to_str(a.second->aniso.u11));
+      aniso_val.emplace_back(to_str(a.second->aniso.u22));
+      aniso_val.emplace_back(to_str(a.second->aniso.u33));
+      aniso_val.emplace_back(to_str(a.second->aniso.u12));
+      aniso_val.emplace_back(to_str(a.second->aniso.u13));
+      aniso_val.emplace_back(to_str(a.second->aniso.u23));
+    }
+  }
+}
+
+// the names are: monomeric, dimeric, ...meric, 21-meric, 22-meric, ...
+int xmeric_to_number(const std::string& oligomeric) {
+  static const char names[20][10] = {
+    "mono", "di", "tri", "tetra", "penta",
+    "hexa", "hepta", "octa", "nona", "deca",
+    "undeca", "dodeca", "trideca", "tetradeca", "pentadeca",
+    "hexadeca", "heptadeca", "octadeca", "nonadeca", "eicosa"
+  };
+  size_t len = oligomeric.length();
+  const char* p = oligomeric.c_str();
+  for (int i = 0; i != 20; ++i)
+    if (len == std::strlen(names[i]) + 5 && strncmp(p, names[i], len-5) == 0)
+      return i + 1;
+  return no_sign_atoi(p);
+}
+
+void write_assemblies(const Structure& st, cif::Block& block) {
+  block.items.reserve(block.items.size() + 4); // avoid re-allocation
+  cif::Loop& a_loop = block.init_mmcif_loop("_pdbx_struct_assembly.",
+      {"id", "details", "method_details",
+       "oligomeric_details", "oligomeric_count"});
+  cif::Loop& prop_loop = block.init_mmcif_loop("_pdbx_struct_assembly_prop.",
+      {"biol_id", "type", "value"});
+  cif::Loop& gen_loop = block.init_mmcif_loop("_pdbx_struct_assembly_gen.",
+      {"assembly_id", "oper_expression", "asym_id_list"});
+  cif::Loop& oper_loop = block.init_mmcif_loop("_pdbx_struct_oper_list.",
+      {"id", "type",
+       "matrix[1][1]", "matrix[1][2]", "matrix[1][3]", "vector[1]",
+       "matrix[2][1]", "matrix[2][2]", "matrix[2][3]", "vector[2]",
+       "matrix[3][1]", "matrix[3][2]", "matrix[3][3]", "vector[3]"});
+  std::vector<const Assembly::Oper*> distinct_oper;
+  for (const Assembly& as : st.assemblies) {
+    std::string how_defined = "?";
+    if (as.author_determined && as.software_determined)
+      how_defined = "author_and_software_defined_assembly";
+    else if (as.author_determined)
+      how_defined = "author_defined_assembly";
+    else if (as.software_determined)
+      how_defined = "software_defined_assembly";
+    else if (as.special_kind == Assembly::SpecialKind::CompleteIcosahedral)
+      how_defined = "'complete icosahedral assembly'";
+    else if (as.special_kind == Assembly::SpecialKind::RepresentativeHelical)
+      how_defined = "'representative helical assembly'";
+    else if (as.special_kind == Assembly::SpecialKind::CompletePoint)
+      how_defined = "'complete point assembly'";
+    std::string oligomer = to_lower(as.oligomeric_details);
+    int nmer = as.oligomeric_count != 0 ? as.oligomeric_count
+                                        : xmeric_to_number(oligomer);
+    // _pdbx_struct_assembly
+    a_loop.add_row({as.name,
+                    how_defined,
+                    impl::string_or_qmark(as.software_name),
+                    impl::string_or_qmark(oligomer),
+                    nmer == 0 ? "?" : std::to_string(nmer)});
+
+    // _pdbx_struct_assembly_prop
+    if (!std::isnan(as.absa))
+      prop_loop.add_row({as.name, "'ABSA (A^2)'", to_str(as.absa)});
+    if (!std::isnan(as.ssa))
+      prop_loop.add_row({as.name, "'SSA (A^2)'", to_str(as.ssa)});
+    if (!std::isnan(as.more))
+      prop_loop.add_row({as.name, "MORE", to_str(as.more)});
+
+    // _pdbx_struct_assembly_gen and _pdbx_struct_oper_list
+    for (const Assembly::Gen& gen : as.generators) {
+      std::string subchain_str;
+      for (const std::string& name : gen.subchains)
+        string_append_sep(subchain_str, ',', name);
+      if (subchain_str.empty()) // chain names to subchain names
+        for (const Chain& chain : st.models[0].chains)
+          if (in_vector(chain.name, gen.chains))
+            for (const auto& sub : chain.subchains())
+              string_append_sep(subchain_str, ',', sub.front().subchain);
+      std::string oper_str;
+      for (const Assembly::Oper& oper : gen.opers) {
+        size_t k = 0;
+        for (; k != distinct_oper.size(); ++k)
+          if (distinct_oper[k]->transform.approx(oper.transform, 1e-9))
+            break;
+        string_append_sep(oper_str, ',', std::to_string(k+1));
+        if (k != distinct_oper.size())
+          continue;
+        distinct_oper.emplace_back(&oper);
+        oper_loop.values.emplace_back(std::to_string(k+1));
+        if (!oper.type.empty()) {
+          oper_loop.values.emplace_back(cif::quote(oper.type));
+        } else if (oper.transform.is_identity()) {
+          oper_loop.values.emplace_back("'identity operation'");
+        } else if (as.author_determined || as.software_determined) {
+          oper_loop.values.emplace_back("'crystal symmetry operation'");
+        } else {
+          oper_loop.values.emplace_back(".");
+        }
+        for (int i = 0; i < 3; ++i) {
+          for (int j = 0; j < 3; ++j)
+            oper_loop.values.emplace_back(to_str(oper.transform.mat[i][j]));
+          oper_loop.values.emplace_back(to_str(oper.transform.vec.at(i)));
+        }
+      }
+      gen_loop.add_row({as.name,
+                        oper_str.empty() ? "." : oper_str,
+                        subchain_str.empty() ? "?" : subchain_str});
     }
   }
 }
@@ -149,10 +264,10 @@ void write_struct_conn(const Structure& st, cif::Block& block) {
        "ptnr2_auth_asym_id", "ptnr2_label_asym_id", "ptnr2_label_comp_id",
        "ptnr2_label_seq_id", "ptnr2_label_atom_id", "pdbx_ptnr2_label_alt_id",
        "ptnr2_auth_seq_id", "pdbx_ptnr2_PDB_ins_code", "ptnr2_symmetry",
-       "details", "pdbx_dist_value"});
-  for (const Connection& con : st.models.at(0).connections) {
-    const_CRA cra1 = st.models[0].find_cra(con.atom[0]);
-    const_CRA cra2 = st.models[0].find_cra(con.atom[1]);
+       "details", "pdbx_dist_value", "ccp4_link_id"});
+  for (const Connection& con : st.connections) {
+    const_CRA cra1 = st.models[0].find_cra(con.partner1);
+    const_CRA cra2 = st.models[0].find_cra(con.partner2);
     if (!cra1.atom || !cra2.atom)
       continue;
     SymImage im = st.cell.find_nearest_image(cra1.atom->pos,
@@ -160,33 +275,62 @@ void write_struct_conn(const Structure& st, cif::Block& block) {
     conn_loop.add_row({
         con.name,                                  // id
         get_mmcif_connection_type_id(con.type),    // conn_type_id
-        cra1.chain->name,                          // ptnr1_auth_asym_id
+        impl::qchain(cra1.chain->name),            // ptnr1_auth_asym_id
         subchain_or_dot(*cra1.residue),            // ptnr1_label_asym_id
         cra1.residue->name,                        // ptnr1_label_comp_id
-        cra1.residue->label_seq.str(),             // ptnr1_label_seq_id
-        cra1.atom->name,                           // ptnr1_label_atom_id
+        cra1.residue->label_seq.str('.'),          // ptnr1_label_seq_id
+        cif::quote(cra1.atom->name),               // ptnr1_label_atom_id
         std::string(1, cra1.atom->altloc_or('?')), // pdbx_ptnr1_label_alt_id
         cra1.residue->seqid.num.str(),             // ptnr1_auth_seq_id
-        pdbx_icode(con.atom[0].res_id),            // ptnr1_PDB_ins_code
+        pdbx_icode(con.partner1.res_id),           // ptnr1_PDB_ins_code
         "1_555",                                   // ptnr1_symmetry
-        cra2.chain->name,                          // ptnr2_auth_asym_id
+        impl::qchain(cra2.chain->name),            // ptnr2_auth_asym_id
         subchain_or_dot(*cra2.residue),            // ptnr2_label_asym_id
         cra2.residue->name,                        // ptnr2_label_comp_id
-        cra2.residue->label_seq.str(),             // ptnr2_label_seq_id
-        cra2.atom->name,                           // ptnr2_label_atom_id
+        cra2.residue->label_seq.str('.'),          // ptnr2_label_seq_id
+        cif::quote(cra2.atom->name),               // ptnr2_label_atom_id
         std::string(1, cra2.atom->altloc_or('?')), // pdbx_ptnr2_label_alt_id
         cra2.residue->seqid.num.str(),             // ptnr2_auth_seq_id
-        pdbx_icode(con.atom[1].res_id),            // ptnr2_PDB_ins_code
+        pdbx_icode(con.partner2.res_id),           // ptnr2_PDB_ins_code
         im.pdb_symbol(true),                       // ptnr2_symmetry
         "?",                                       // details
-        to_str(im.dist())                          // pdbx_dist_value
+        to_str_prec<4>(im.dist()),                 // pdbx_dist_value
+        impl::string_or_qmark(con.link_id)         // ccp4_link_id
     });
   }
 }
 
+void write_cell_parameters(const UnitCell& cell, cif::Block& block) {
+  block.set_pair("_cell.length_a",    to_str(cell.a));
+  block.set_pair("_cell.length_b",    to_str(cell.b));
+  block.set_pair("_cell.length_c",    to_str(cell.c));
+  block.set_pair("_cell.angle_alpha", to_str(cell.alpha));
+  block.set_pair("_cell.angle_beta",  to_str(cell.beta));
+  block.set_pair("_cell.angle_gamma", to_str(cell.gamma));
+}
+
+void write_ncs_oper(const Structure& st, cif::Block& block) {
+  // _struct_ncs_oper (MTRIX)
+  if (!st.ncs.empty()) {
+    cif::Loop& ncs_oper = block.init_mmcif_loop("_struct_ncs_oper.",
+        {"id", "code",
+         "matrix[1][1]", "matrix[1][2]", "matrix[1][3]", "vector[1]",
+         "matrix[2][1]", "matrix[2][2]", "matrix[2][3]", "vector[2]",
+         "matrix[3][1]", "matrix[3][2]", "matrix[3][3]", "vector[3]"});
+    for (const NcsOp& op : st.ncs) {
+      ncs_oper.values.emplace_back(op.id);
+      ncs_oper.values.emplace_back(op.given ? "given" : "generate");
+      for (int i = 0; i < 3; ++i) {
+        for (int j = 0; j < 3; ++j)
+          ncs_oper.values.emplace_back(to_str(op.tr.mat[i][j]));
+        ncs_oper.values.emplace_back(to_str(op.tr.vec.at(i)));
+      }
+    }
+  }
+}
 } // namespace impl
 
-void update_cif_block(const Structure& st, cif::Block& block) {
+void update_cif_block(const Structure& st, cif::Block& block, bool with_atoms) {
   using std::to_string;
   if (st.models.empty())
     return;
@@ -203,31 +347,70 @@ void update_cif_block(const Structure& st, cif::Block& block) {
 
   // unit cell and symmetry
   block.set_pair("_cell.entry_id", id);
-  block.set_pair("_cell.length_a",    to_str(st.cell.a));
-  block.set_pair("_cell.length_b",    to_str(st.cell.b));
-  block.set_pair("_cell.length_c",    to_str(st.cell.c));
-  block.set_pair("_cell.angle_alpha", to_str(st.cell.alpha));
-  block.set_pair("_cell.angle_beta",  to_str(st.cell.beta));
-  block.set_pair("_cell.angle_gamma", to_str(st.cell.gamma));
+  impl::write_cell_parameters(st.cell, block);
   auto z_pdb = st.info.find("_cell.Z_PDB");
   if (z_pdb != st.info.end())
     block.set_pair(z_pdb->first, z_pdb->second);
   block.set_pair("_symmetry.entry_id", id);
   block.set_pair("_symmetry.space_group_name_H-M",
                  cif::quote(st.spacegroup_hm));
-  if (const SpaceGroup* sg = find_spacegroup_by_name(st.spacegroup_hm))
+  if (const SpaceGroup* sg = st.find_spacegroup())
     block.set_pair("_symmetry.Int_Tables_number", to_string(sg->number));
 
   // _entity
   cif::Loop& entity_loop = block.init_mmcif_loop("_entity.", {"id", "type"});
   for (const Entity& ent : st.entities)
-    entity_loop.add_row({ent.name, entity_type_to_string(ent.entity_type)});
+    entity_loop.add_row({impl::qchain(ent.name),
+                         entity_type_to_string(ent.entity_type)});
 
   // _entity_poly
   cif::Loop& ent_poly_loop = block.init_mmcif_loop("_entity_poly.", {"entity_id", "type"});
   for (const Entity& ent : st.entities)
     if (ent.entity_type == EntityType::Polymer)
-      ent_poly_loop.add_row({ent.name, polymer_type_to_qstring(ent.polymer_type)});
+      ent_poly_loop.add_row({impl::qchain(ent.name),
+                             polymer_type_to_qstring(ent.polymer_type)});
+
+  // _struct_ref, _struct_ref_seq
+  {
+    block.items.reserve(block.items.size() + 2); // avoid re-allocation
+    cif::Loop& ref_loop = block.init_mmcif_loop("_struct_ref.",
+                                  {"id", "entity_id", "db_name", "db_code",
+                                   "pdbx_db_accession", "pdbx_db_isoform"});
+    cif::Loop& seq_loop = block.init_mmcif_loop("_struct_ref_seq.", {
+                                       "align_id", "ref_id", "pdbx_strand_id",
+                                       "seq_align_beg", "seq_align_end",
+                                       "db_align_beg", "db_align_end"});
+    int counter = 0;
+    int counter2 = 0;
+    for (const Entity& ent : st.entities)
+      for (const Entity::DbRef& dbref : ent.dbrefs) {
+        ref_loop.add_row({std::to_string(++counter),
+                          impl::qchain(ent.name),
+                          impl::string_or_dot(dbref.db_name),
+                          impl::string_or_dot(dbref.id_code),
+                          impl::string_or_qmark(dbref.accession_code),
+                          impl::string_or_qmark(dbref.isoform)});
+        for (const std::string& subchain : ent.subchains) {
+          Residue::OptionalNum label_begin = dbref.label_seq_begin;
+          Residue::OptionalNum label_end = dbref.label_seq_end;
+          ConstResidueSpan span = st.models[0].get_subchain(subchain);
+          if (!label_begin) {
+            try {
+              label_begin = span.auth_seq_id_to_label(dbref.seq_begin);
+              label_end = span.auth_seq_id_to_label(dbref.seq_end);
+            } catch (const std::runtime_error&) {}
+          }
+          seq_loop.add_row({std::to_string(++counter2),
+                            std::to_string(counter),
+                            //chain.name,
+                            span.subchain_id(),
+                            std::to_string(*label_begin),
+                            std::to_string(*label_end),
+                            std::to_string(*dbref.db_begin.num),
+                            std::to_string(*dbref.db_end.num)});
+        }
+      }
+  }
 
   // _exptl
   if (!st.meta.experiments.empty()) {
@@ -245,6 +428,14 @@ void update_cif_block(const Structure& st, cif::Block& block) {
     }
   }
 
+  // _exptl_crystal
+  if (!st.meta.crystals.empty()) {
+    cif::Loop& loop = block.init_mmcif_loop("_exptl_crystal.",
+                                            {"id", "description"});
+    for (const CrystalInfo& cryst : st.meta.crystals)
+      loop.add_row({cryst.id, impl::string_or_qmark(cryst.description)});
+  }
+
   // _exptl_crystal_grow
   if (std::any_of(st.meta.crystals.begin(), st.meta.crystals.end(),
             [](const CrystalInfo& c) { return !c.ph_range.empty() || !std::isnan(c.ph); })) {
@@ -328,7 +519,7 @@ void update_cif_block(const Structure& st, cif::Block& block) {
     for (const ExperimentInfo& exper : st.meta.experiments)
       loop.add_row({id,
                     std::to_string(++n),
-                    join_str(exper.diffraction_ids, ","),
+                    impl::string_or_dot(join_str(exper.diffraction_ids, ",")),
                     impl::int_or_qmark(exper.unique_reflections),
                     impl::number_or_qmark(exper.reflections.resolution_high),
                     impl::number_or_qmark(exper.reflections.resolution_low),
@@ -340,6 +531,8 @@ void update_cif_block(const Structure& st, cif::Block& block) {
                     /*impl::number_or_qmark(exper.b_wilson)*/});
     // _reflns_shell
     cif::Loop& shell_loop = block.init_mmcif_loop("_reflns_shell.", {
+        "pdbx_ordinal",
+        "pdbx_diffrn_id",
         "d_res_high",
         "d_res_low",
         "percent_possible_all",
@@ -347,20 +540,26 @@ void update_cif_block(const Structure& st, cif::Block& block) {
         "Rmerge_I_obs",
         "pdbx_Rsym_value",
         "meanI_over_sigI_obs"});
-    for (const ExperimentInfo& exper : st.meta.experiments)
+    n = 0;
+    for (const ExperimentInfo& exper : st.meta.experiments) {
+      std::string diffrn_id =
+        impl::string_or_dot(join_str(exper.diffraction_ids, ","));
       for (const ReflectionsInfo& shell : exper.shells)
-        shell_loop.add_row({impl::number_or_qmark(shell.resolution_high),
+        shell_loop.add_row({std::to_string(++n),
+                            diffrn_id,
+                            impl::number_or_qmark(shell.resolution_high),
                             impl::number_or_qmark(shell.resolution_low),
                             impl::number_or_qmark(shell.completeness),
                             impl::number_or_qmark(shell.redundancy),
                             impl::number_or_qmark(shell.r_merge),
                             impl::number_or_qmark(shell.r_sym),
                             impl::number_or_qmark(shell.mean_I_over_sigma)});
+    }
   }
 
   // _refine
   if (!st.meta.refinement.empty()) {
-    block.items.reserve(block.items.size() + 3);
+    block.items.reserve(block.items.size() + 4);
     cif::Loop& loop = block.init_mmcif_loop("_refine.", {
         "entry_id",
         "pdbx_refine_id",
@@ -372,6 +571,9 @@ void update_cif_block(const Structure& st, cif::Block& block) {
         "entry_id",
         "pdbx_refine_id",
         "Luzzati_coordinate_error_obs"});
+    cif::Loop& restr_loop = block.init_mmcif_loop("_refine_ls_restr.", {
+        "pdbx_refine_id", "type",
+        "number", "weight", "pdbx_restraint_function", "dev_ideal"});
     cif::Loop& shell_loop = block.init_mmcif_loop("_refine_ls_shell.", {
         "pdbx_refine_id",
         "d_res_high",
@@ -385,7 +587,7 @@ void update_cif_block(const Structure& st, cif::Block& block) {
     for (size_t i = 0; i != st.meta.refinement.size(); ++i) {
       const RefinementInfo& ref = st.meta.refinement[i];
       loop.values.push_back(id);
-      loop.values.push_back(ref.id);
+      loop.values.push_back(cif::quote(ref.id));
       loop.values.push_back(impl::number_or_dot(ref.resolution_high));
       loop.values.push_back(impl::number_or_dot(ref.resolution_low));
       loop.values.push_back(impl::number_or_dot(ref.completeness));
@@ -403,8 +605,9 @@ void update_cif_block(const Structure& st, cif::Block& block) {
         add("ls_R_factor_R_work", impl::number_or_qmark(ref.r_work));
       if (st.meta.has(&RefinementInfo::r_free))
         add("ls_R_factor_R_free", impl::number_or_qmark(ref.r_free));
-      add("pdbx_ls_cross_valid_method",
-          impl::string_or_qmark(ref.cross_validation_method));
+      if (st.meta.has(&RefinementInfo::cross_validation_method))
+        add("pdbx_ls_cross_valid_method",
+            impl::string_or_qmark(ref.cross_validation_method));
       if (st.meta.has(&RefinementInfo::rfree_selection_method))
         add("pdbx_R_Free_selection_details",
             impl::string_or_qmark(ref.rfree_selection_method));
@@ -440,10 +643,19 @@ void update_cif_block(const Structure& st, cif::Block& block) {
         add("pdbx_method_to_determine_struct", impl::string_or_qmark(st.meta.solved_by));
       if (!st.meta.starting_model.empty())
         add("pdbx_starting_model", impl::string_or_qmark(st.meta.starting_model));
-      analyze_loop.add_row({id, ref.id,
-                            impl::number_or_qmark(ref.luzzati_error)});
+      if (!std::isnan(ref.luzzati_error))
+        analyze_loop.add_row({id,
+                              cif::quote(ref.id),
+                              impl::number_or_qmark(ref.luzzati_error)});
+      for (const RefinementInfo::Restr& restr : ref.restr_stats)
+        restr_loop.add_row({cif::quote(ref.id),
+                            cif::quote(restr.name),
+                            impl::int_or_qmark(restr.count),
+                            impl::number_or_qmark(restr.weight),
+                            impl::string_or_qmark(restr.function),
+                            impl::number_or_qmark(restr.dev_ideal)});
       for (const BasicRefinementInfo& bin : ref.bins)
-        shell_loop.add_row({ref.id,
+        shell_loop.add_row({cif::quote(ref.id),
                             impl::number_or_dot(bin.resolution_high),
                             impl::number_or_qmark(bin.resolution_low),
                             impl::number_or_qmark(bin.completeness),
@@ -471,23 +683,7 @@ void update_cif_block(const Structure& st, cif::Block& block) {
   if (keywords != st.info.end())
     block.set_pair(keywords->first, cif::quote(keywords->second));
 
-  // _struct_ncs_oper (MTRIX)
-  if (!st.ncs.empty()) {
-    cif::Loop& ncs_oper = block.init_mmcif_loop("_struct_ncs_oper.",
-        {"id", "code",
-         "matrix[1][1]", "matrix[1][2]", "matrix[1][3]", "vector[1]",
-         "matrix[2][1]", "matrix[2][2]", "matrix[2][3]", "vector[2]",
-         "matrix[3][1]", "matrix[3][2]", "matrix[3][3]", "vector[3]"});
-    for (const NcsOp& op : st.ncs) {
-      ncs_oper.values.emplace_back(op.id);
-      ncs_oper.values.emplace_back(op.given ? "given" : "generate");
-      for (int i = 0; i < 3; ++i) {
-        for (int j = 0; j < 3; ++j)
-          ncs_oper.values.emplace_back(to_str(op.tr.mat[i][j]));
-        ncs_oper.values.emplace_back(to_str(op.tr.vec.at(i)));
-      }
-    }
-  }
+  impl::write_ncs_oper(st, block);
 
   // _struct_asym
   cif::Loop& asym_loop = block.init_mmcif_loop("_struct_asym.",
@@ -496,11 +692,12 @@ void update_cif_block(const Structure& st, cif::Block& block) {
     for (ConstResidueSpan& sub : chain.subchains())
       if (!sub.subchain_id().empty()) {
         const Entity* ent = st.get_entity_of(sub);
-        asym_loop.add_row({sub.subchain_id(), (ent ? ent->name : "?")});
+        asym_loop.add_row({sub.subchain_id(),
+                           (ent ? impl::qchain(ent->name) : "?")});
       }
 
   // _database_PDB_matrix (ORIGX)
-  if (st.has_origx) {
+  if (st.has_origx && !st.origx.is_identity()) {
     block.set_pair("_database_PDB_matrix.entry_id", id);
     std::string prefix = "_database_PDB_matrix.origx";
     for (int i = 0; i < 3; ++i) {
@@ -529,18 +726,18 @@ void update_cif_block(const Structure& st, cif::Block& block) {
       struct_conf_loop.add_row({
         "HELX_P",                                     // conf_type_id
         "H" + std::to_string(++count),                // id
-        cra1.chain->name,                             // beg_auth_asym_id
+        impl::qchain(cra1.chain->name),               // beg_auth_asym_id
         impl::subchain_or_dot(*cra1.residue),         // beg_label_asym_id
         cra1.residue->name,                           // beg_label_comp_id
         cra1.residue->label_seq.str(),                // beg_label_seq_id
         cra1.residue->seqid.num.str(),                // beg_auth_seq_id
         impl::pdbx_icode(*cra1.residue),              // beg_PDB_ins_code
-        cra2.chain->name,                             // beg_auth_asym_id
-        impl::subchain_or_dot(*cra2.residue),         // beg_label_asym_id
-        cra2.residue->name,                           // beg_label_comp_id
-        cra2.residue->label_seq.str(),                // beg_label_seq_id
-        cra2.residue->seqid.num.str(),                // beg_auth_seq_id
-        impl::pdbx_icode(*cra2.residue),              // beg_PDB_ins_code
+        impl::qchain(cra2.chain->name),               // end_auth_asym_id
+        impl::subchain_or_dot(*cra2.residue),         // end_label_asym_id
+        cra2.residue->name,                           // end_label_comp_id
+        cra2.residue->label_seq.str(),                // end_label_seq_id
+        cra2.residue->seqid.num.str(),                // end_auth_seq_id
+        impl::pdbx_icode(*cra2.residue),              // end_PDB_ins_code
         std::to_string((int)helix.pdb_helix_class),   // pdbx_PDB_helix_class
         impl::int_or_qmark(helix.length)              // pdbx_PDB_helix_length
       });
@@ -549,6 +746,103 @@ void update_cif_block(const Structure& st, cif::Block& block) {
       block.set_pair("_struct_conf_type.id", "HELX_P");
   }
 
+  // _struct_sheet*
+  if (!st.sheets.empty()) {
+    cif::Loop& sheet_loop = block.init_mmcif_loop("_struct_sheet.",
+                                                  {"id", "number_strands"});
+    for (const Sheet& sheet : st.sheets)
+      sheet_loop.add_row({sheet.name, std::to_string(sheet.strands.size())});
+
+    cif::Loop& order_loop = block.init_mmcif_loop("_struct_sheet_order.",
+                    {"sheet_id", "range_id_1", "range_id_2", "sense"});
+    for (const Sheet& sheet : st.sheets)
+      for (size_t i = 1; i < sheet.strands.size(); ++i) {
+        const Sheet::Strand& strand = sheet.strands[i];
+        if (strand.sense != 0)
+          order_loop.add_row({sheet.name,
+                              std::to_string(i), std::to_string(i+1),
+                              strand.sense > 0 ? "parallel" : "anti-parallel"});
+      }
+
+    cif::Loop& range_loop = block.init_mmcif_loop("_struct_sheet_range.",
+        {"sheet_id", "id",
+         "beg_auth_asym_id", "beg_label_asym_id", "beg_label_comp_id",
+         "beg_label_seq_id", "beg_auth_seq_id", "pdbx_beg_PDB_ins_code",
+         "end_auth_asym_id", "end_label_asym_id", "end_label_comp_id",
+         "end_label_seq_id", "end_auth_seq_id", "pdbx_end_PDB_ins_code"});
+    for (const Sheet& sheet : st.sheets)
+      for (size_t i = 0; i < sheet.strands.size(); ++i) {
+        const Sheet::Strand& strand = sheet.strands[i];
+        const_CRA cra1 = st.models[0].find_cra(strand.start);
+        const_CRA cra2 = st.models[0].find_cra(strand.end);
+        if (!cra1.residue || !cra2.residue)
+          continue;
+        range_loop.add_row({
+          sheet.name,                                 // sheet_id
+          std::to_string(i+1),                        // id
+          impl::qchain(cra1.chain->name),             // beg_auth_asym_id
+          impl::subchain_or_dot(*cra1.residue),       // beg_label_asym_id
+          cra1.residue->name,                         // beg_label_comp_id
+          cra1.residue->label_seq.str(),              // beg_label_seq_id
+          cra1.residue->seqid.num.str(),              // beg_auth_seq_id
+          impl::pdbx_icode(*cra1.residue),            // beg_PDB_ins_code
+          impl::qchain(cra2.chain->name),             // end_auth_asym_id
+          impl::subchain_or_dot(*cra2.residue),       // end_label_asym_id
+          cra2.residue->name,                         // end_label_comp_id
+          cra2.residue->label_seq.str(),              // end_label_seq_id
+          cra2.residue->seqid.num.str(),              // end_auth_seq_id
+          impl::pdbx_icode(*cra2.residue)             // end_PDB_ins_code
+        });
+    }
+
+    cif::Loop& hbond_loop = block.init_mmcif_loop("_pdbx_struct_sheet_hbond.",
+        {"sheet_id", "range_id_1", "range_id_2",
+         "range_1_auth_asym_id", "range_1_label_asym_id",
+         "range_1_label_comp_id", "range_1_label_seq_id", "range_1_auth_seq_id",
+         "range_1_PDB_ins_code", "range_1_label_atom_id",
+         "range_2_auth_asym_id", "range_2_label_asym_id",
+         "range_2_label_comp_id", "range_2_label_seq_id", "range_2_auth_seq_id",
+         "range_2_PDB_ins_code", "range_2_label_atom_id"});
+    for (const Sheet& sheet : st.sheets)
+      for (size_t i = 1; i < sheet.strands.size(); ++i) {
+        const Sheet::Strand& strand = sheet.strands[i];
+        if (strand.hbond_atom2.atom_name.empty())
+          continue;
+        // hbond_atomN is not a full atom "address": altloc is missing
+        const_CRA cra1 = st.models[0].find_cra(strand.hbond_atom1);
+        const_CRA cra2 = st.models[0].find_cra(strand.hbond_atom2);
+        if (!cra1.residue || !cra2.residue)
+          continue;
+        hbond_loop.add_row({
+          sheet.name,                                 // sheet_id
+          std::to_string(i),                          // range_id_1
+          std::to_string(i+1),                        // range_id_2
+          impl::qchain(cra1.chain->name),             // range_1_auth_asym_id
+          impl::subchain_or_dot(*cra1.residue),       // range_1_label_asym_id
+          cra1.residue->name,                         // range_1_label_comp_id
+          cra1.residue->label_seq.str(),              // range_1_label_seq_id
+          cra1.residue->seqid.num.str(),              // range_1_auth_seq_id
+          impl::pdbx_icode(*cra1.residue),            // range_1_PDB_ins_code
+          cif::quote(strand.hbond_atom1.atom_name),   // range_1_label_atom_id
+          impl::qchain(cra2.chain->name),             // range_2_auth_asym_id
+          impl::subchain_or_dot(*cra2.residue),       // range_2_label_asym_id
+          cra2.residue->name,                         // range_2_label_comp_id
+          cra2.residue->label_seq.str(),              // range_2_label_seq_id
+          cra2.residue->seqid.num.str(),              // range_2_auth_seq_id
+          impl::pdbx_icode(*cra2.residue),            // range_2_PDB_ins_code
+          cif::quote(strand.hbond_atom2.atom_name)    // range_2_label_atom_id
+        });
+    }
+  }
+
+  // _pdbx_struct_assembly* and _struct_biol (REMARK 300/350)
+  if (!st.meta.remark_300_detail.empty()) {
+    block.set_pair("_struct_biol.id", "1");
+    block.set_pair("_struct_biol.details", cif::quote(st.meta.remark_300_detail));
+  }
+  if (!st.assemblies.empty())
+    impl::write_assemblies(st, block);
+
   impl::write_struct_conn(st, block);
 
   // _struct_mon_prot_cis
@@ -562,7 +856,7 @@ void update_cif_block(const Structure& st, cif::Block& block) {
         if (res.is_cis)
           prot_cis_loop.add_row({to_string(prot_cis_loop.length()+1),
                                  model.name, impl::subchain_or_dot(res),
-                                 res.label_seq.str(), chain.name,
+                                 res.label_seq.str(), impl::qchain(chain.name),
                                  res.seqid.num.str(), impl::pdbx_icode(res),
                                  res.name, "."});
 
@@ -586,12 +880,20 @@ void update_cif_block(const Structure& st, cif::Block& block) {
                                          {"entity_id", "num", "mon_id"});
   for (const Entity& ent : st.entities)
     if (ent.entity_type == EntityType::Polymer)
-      for (const PolySeqItem& si : ent.poly_seq) {
-        std::string num = si.num >= 0 ? to_string(si.num) : "?";
-        poly_loop.add_row({ent.name, num, si.mon});
+      for (size_t i = 0; i != ent.full_sequence.size(); ++i) {
+        const std::string& mon_ids = ent.full_sequence[i];
+        std::string num = std::to_string(i+1);
+        size_t start = 0, end;
+        while ((end = mon_ids.find(',', start)) != std::string::npos) {
+          poly_loop.add_row({impl::qchain(ent.name), num,
+                             mon_ids.substr(start, end-start)});
+          start = end + 1;
+        }
+        poly_loop.add_row({impl::qchain(ent.name), num, mon_ids.substr(start)});
       }
 
-  impl::add_cif_atoms(st, block);
+  if (with_atoms)
+    impl::add_cif_atoms(st, block);
 
   if (st.meta.has_tls()) {
     cif::Loop& loop = block.init_mmcif_loop("_pdbx_refine_tls.", {
@@ -608,7 +910,7 @@ void update_cif_block(const Structure& st, cif::Block& block) {
         const Mat33& L = tls.L;
         const Mat33& S = tls.S;
         auto q = impl::number_or_qmark;
-        loop.add_row({ref.id, impl::string_or_dot(tls.id),
+        loop.add_row({cif::quote(ref.id), impl::string_or_dot(tls.id),
                       q(T[0][0]), q(T[1][1]), q(T[2][2]),
                       q(T[0][1]), q(T[0][2]), q(T[1][2]),
                       q(L[0][0]), q(L[1][1]), q(L[2][2]),
@@ -636,23 +938,38 @@ void update_cif_block(const Structure& st, cif::Block& block) {
 
   if (!st.meta.software.empty()) {
     cif::Loop& loop = block.init_mmcif_loop("_software.",
-                       {"pdbx_ordinal", "classification", "name", "version"});
+                 {"pdbx_ordinal", "classification", "name", "version", "date"});
     for (const SoftwareItem& item : st.meta.software)
       loop.add_row({
-          to_string(item.pdbx_ordinal),
+          std::to_string(item.pdbx_ordinal),
           cif::quote(software_classification_to_string(item.classification)),
           cif::quote(item.name),
-          item.version.empty() ? "?" : cif::quote(item.version)});
+          impl::string_or_dot(item.version),
+          impl::string_or_qmark(item.date)});
   }
 }
 
 cif::Document make_mmcif_document(const Structure& st) {
   cif::Document doc;
   doc.blocks.resize(1);
-  gemmi::update_cif_block(st, doc.blocks[0]);
+  gemmi::update_cif_block(st, doc.blocks[0], true);
   return doc;
 }
 
+cif::Block make_mmcif_headers(const Structure& st) {
+  cif::Block block;
+  gemmi::update_cif_block(st, block, false);
+  return block;
+}
+
+void add_minimal_mmcif_data(const Structure& st, cif::Block& block) {
+  impl::write_cell_parameters(st.cell, block);
+  block.set_pair("_symmetry.space_group_name_H-M",
+                 cif::quote(st.spacegroup_hm));
+  impl::write_ncs_oper(st, block);
+  impl::add_cif_atoms(st, block);
+}
+
 } // namespace gemmi
 #endif // GEMMI_WRITE_IMPLEMENTATION
 
