@@ -10,8 +10,8 @@ from collections import OrderedDict
 import glob
 from TEMPy.mapprocess import Filter
 import numpy as np
-#set mrcfile import to False (TODO: check mapprocess object compatibility)
-mrcfile_import=False
+
+mrcfile_import=True
 try:
     import mrcfile
 except ImportError:
@@ -23,10 +23,17 @@ pdir = tp.args.pdir
 mdir = tp.args.mdir
 plist = tp.args.plist
 mlist = tp.args.mlist
+smlist = tp.args.smlist
 rlist = tp.args.rlist
 p = tp.args.pdb
 p1 = tp.args.pdb1
 p2 = tp.args.pdb2
+sm = tp.args.sim_map
+#if model maps are provided
+sim_map = None
+if sm is not None:
+    map_file = sm
+    smlist = [sm]
 #map input
 m = tp.args.inp_map
 if m is None: 
@@ -85,11 +92,13 @@ def read_mapfile(map_path):
     if mrcfile_import:
         mrcobj=mrcfile.open(map_path,mode='r')
         emmap = Filter(mrcobj)
+        emmap.fix_origin()
         emmap.set_apix_tempy()
     else:
         mrcobj=MapParser.readMRC(map_path)
         emmap = Filter(mrcobj)
-        emmap.set_apix_tempy()
+        emmap.fix_origin()
+        #emmap.set_apix_tempy()
     return emmap
 
 def write_mapfile(mapobj,map_path):
@@ -116,6 +125,7 @@ def process_global_scores(map_path,
                         map_level,
                         pdb_path_list,
                         directory=None):
+    
     em_map = read_mapfile(map_path)
     if map_level is None:
         map_level = em_map.calculate_map_contour(sigma_factor=1.5)
@@ -124,14 +134,23 @@ def process_global_scores(map_path,
     dict_table = {}
     dict_scores = {}
     list_pdbs = []
+    l = 0
     for pdb_path in pdb_path_list:
         pdb_id = os.path.basename(pdb_path)
+        pName = os.path.splitext(pdb_id)[0]
         list_pdbs.append(pdb_id)
-        pName,modelmap = blur_model(pdb_path, map_resolution, em_map)
-        if modelmap.__class__.__name__ == 'Map':
-             emmap2 = Filter(modelmap)
+        if smlist is not None:
+            if len(list_models) == len(smlist):
+                sim_mapfile = smlist[l]
+                if os.path.isfile(sim_mapfile):
+                    modelmap = read_mapfile(sim_mapfile)
+                    print 'Using second map: ',sim_mapfile
+                else:
+                    pName,modelmap = blur_model(pdb_path, map_resolution, em_map)
+#                     if modelmap.__class__.__name__ == 'Map':
+#                          emmap2 = Filter(modelmap)
         synmap_file = pName+'_syn.mrc'
-        write_mapfile(emmap2,synmap_file)
+        write_mapfile(modelmap,synmap_file)
         if map_resolution > 20.0: t = 2.0
         elif map_resolution > 10.0: t = 1.0
         elif map_resolution > 6.0: t = 0.5
@@ -139,6 +158,7 @@ def process_global_scores(map_path,
         model_level = model_contour(modelmap, t)
         #calculate scores
         get_global_scores(em_map,modelmap,map_level,model_level,dict_scores,pName)
+        l += 1
     sc = ScoringFunctions()
     
     dict_scores['mi_ov'] = sc.scale_median(
@@ -167,7 +187,7 @@ def get_global_scores(emmap1,emmap2,c1,c2,dict_scores_hits,Name2):
         ccc_mask,ovr = sc.CCC_map(emmap1,emmap2,c1,c2,3,meanDist=True)
         print 'Percent overlap:', ovr
         if ovr < 0.0: ovr = 0.0
-    except:
+    except IOError:
         print 'ccc and overlap score calculation failed for', Name2
         ovr = 0.0
     if ovr < 0.02:
@@ -208,7 +228,7 @@ def get_global_scores(emmap1,emmap2,c1,c2,dict_scores_hits,Name2):
             nmi = 0.0
           #MI
         try:
-            mi = sc.MI(emmap1,emmap2,c1,c2,1,None,None,False)
+            mi = sc.MI(emmap1,emmap2,c1,c2,mode=1,weight=False,layers1=20,layers2=20)
             print 'Mutual information score:', mi
             if mi < 0.0: mi = 0.0
         except:
