@@ -61,6 +61,19 @@ elif not p is None:
     list_models = [p]
 
 
+def read_mapfile(map_path):
+    if mrcfile_import:
+        mrcobj=mrcfile.open(map_path,mode='r')
+        emmap = Filter(mrcobj)
+        emmap.fix_origin()
+        emmap.set_apix_tempy()
+    else:
+        mrcobj=MapParser.readMRC(map_path)
+        emmap = Filter(mrcobj)
+        emmap.fix_origin()
+    return emmap
+
+
 m_count = 0
 dict_chains_mapval = {}
 maxlim = 0.0
@@ -68,12 +81,8 @@ for m in list_maps:
     #get details of map
     Name1 = os.path.basename(m).split('.')[0]
     #read map file
-    if mrcfile_import:
-        mrcobj=mrcfile.open(m,mode='r')
-        map_target = Filter(mrcobj)
-        map_target.set_apix_tempy()
-    else:
-        map_target=MapParser.readMRC(m)
+    map_target = read_mapfile(m)
+    
     resolution_densMap = rlist[m_count]
     m_count += 1
     
@@ -89,7 +98,7 @@ for m in list_maps:
         blurrer = StructureBlurrer()
         dict_chain_scores1 = blurrer.get_map_values(
                             structure_instance,map_target,
-                            resolution_densMap,win=1,sigma_thr=0.1)
+                            resolution_densMap,win=1,sigma_thr=0.5)
         list_val = []
         maxval = 100.5
         #include scores as b-factor records
