@@ -20,10 +20,12 @@
 #===============================================================================
 
 
-from numpy import array,  zeros, real,sqrt,exp, mgrid, transpose, mean as npmean
+from numpy import array,  zeros, real,sqrt,exp, mgrid, transpose, mean as npmean, amax as npmax
 from scipy.fftpack import fftn, ifftn
 from scipy.ndimage import fourier_gaussian,gaussian_filter,uniform_filter
 from TEMPy.EMMap import Map
+from collections import OrderedDict
+import gc
 
 class StructureBlurrer:
     """ 
@@ -152,7 +154,7 @@ class StructureBlurrer:
         
         #MODIFIED BY PAP
         if((box_size[2] > x_pos >= 0) and (box_size[1] > y_pos >= 0) and (box_size[0] > z_pos >= 0)):
-            return (x_pos, y_pos, z_pos, atom.mass)
+            return (x_pos, y_pos, z_pos, atom.at_number)
         else:
             return 0
     
@@ -217,11 +219,13 @@ class StructureBlurrer:
                 from scipy.spatial import KDTree
                 gridtree = KDTree(indi)
             except ImportError: return
-        return gridtree
+        return gridtree, indi
 
     #added by aj
     #voxels occupied by the atom density (3 sigma gaussian)
-    def mapGridPositions(self,densMap, atom, gridtree,res_map,sim_sigma_coeff=0.187):
+    def mapGridPositions(self,densMap, atom, 
+                         gridtree,res_map,sim_sigma_coeff=0.187,
+                         sigma_thr=2.0):
         """
         
         Returns the indices of the nearest pixels to an atom as a list.
@@ -245,12 +249,16 @@ class StructureBlurrer:
         y_pos = int(round((atom.y-origin[1])/apix,0))
         z_pos = int(round((atom.z-origin[2])/apix,0))
         #print atom.get_res_no(), x_pos,y_pos,z_pos
-        if((densMap.x_size() > x_pos >= 0) and (densMap.y_size() > y_pos >= 0) and (densMap.z_size() > z_pos >= 0)):
+        if((densMap.x_size() >= x_pos >= 0) and 
+           (densMap.y_size() >= y_pos >= 0) and 
+           (densMap.z_size() >= z_pos >= 0)):
             #search all points withing 1.5sigma
-            list_points = gridtree.query_ball_point([atom.x,atom.y,atom.z], 1.5*max(sim_sigma_coeff*res_map,1.0))
+            list_points = gridtree.query_ball_point(\
+                            [atom.x,atom.y,atom.z], 
+                            max(apix,sigma_thr*max(sim_sigma_coeff*res_map,1.0)))
             return list_points
         else:
-            print "Warning, atom out of map box"
+            print "Warning, atom out of map box", atom.chain, atom.get_res_no(), atom.atom_name
             return []
     #added by aj
     def model_tree(self,list_coord1,distpot=6.0,list_coord2=None):
@@ -291,14 +299,20 @@ class StructureBlurrer:
         currentChain = structure_instance.atomList[0].chain
         for x in structure_instance.atomList:
             if not x.chain == currentChain:
-                try: dict_chain_indices[x.model][currentChain] = dict_res_indices.copy()
-                except KeyError: 
+                if not x.model in dict_chain_indices:
                     dict_chain_indices[x.model] = {}
                     dict_chain_indices[x.model][currentChain] = dict_res_indices.copy()
-                try: dict_chain_CA[x.model][currentChain] = dict_res_CA.copy()
-                except KeyError:
+                else:
+                     try: dict_chain_indices[x.model][currentChain].update(dict_res_indices.copy())
+                     except KeyError: dict_chain_indices[x.model][currentChain] = dict_res_indices.copy()
+                
+                if not x.model in dict_chain_CA:
                     dict_chain_CA[x.model] = {}
                     dict_chain_CA[x.model][currentChain] = dict_res_CA.copy()
+                else:
+                    try: dict_chain_CA[x.model][currentChain].update(dict_res_CA.copy())
+                    except KeyError: dict_chain_CA[x.model][currentChain] = dict_res_CA.copy()
+                    
                 currentChain = x.chain
                 dict_res_indices = {}
                 dict_res_CA = {}
@@ -310,21 +324,28 @@ class StructureBlurrer:
                 dict_res_CA[cur_res] = [x.x, x.y, x.z]
             try: dict_res_indices[cur_res].append([x.x, x.y, x.z])
             except KeyError: dict_res_indices[cur_res] = [[x.x, x.y, x.z]]
-        if not dict_chain_indices.has_key(currentChain):
-            #
-            try: dict_chain_CA[x.model][currentChain] = dict_res_CA.copy()
-            except KeyError:
-                dict_chain_CA[x.model] = {}
-                dict_chain_CA[x.model][currentChain] = dict_res_CA.copy()
-            try: dict_chain_indices[x.model][currentChain] = dict_res_indices.copy()
+        if not dict_chain_indices.has_key(x.model):
+            dict_chain_indices[x.model] = {}
+            dict_chain_indices[x.model][currentChain] = dict_res_indices.copy()
+        else:
+            try: dict_chain_indices[x.model][currentChain].update(dict_res_indices.copy())
             except KeyError: 
-                dict_chain_indices[x.model] = {}
                 dict_chain_indices[x.model][currentChain] = dict_res_indices.copy()
+        if not x.model in dict_chain_CA:
+            dict_chain_CA[x.model] = {}
+            dict_chain_CA[x.model][currentChain] = dict_res_CA.copy()
+        else:
+            try: dict_chain_CA[x.model][currentChain].update(dict_res_CA.copy())
+            except KeyError:
+                dict_chain_CA[x.model][currentChain] = dict_res_CA.copy()
+                
             #dict_chain_indices[currentChain] = dict_res_indices.copy()
         return dict_chain_indices, dict_chain_CA   
     #added by aj
     # get nearest grid indices for a residue atoms
-    def get_indices(self,structure_instance,emmap,res_map,sim_sigma_coeff=0.187):
+    def get_indices(self,structure_instance,emmap,res_map,
+                    sim_sigma_coeff=0.187,sigma_thr=2.0,
+                    atom_sel=None, atom_centre='CA'):
         """
         
         Returns flat indices of the pixels occupied by each residue in a chain.
@@ -341,54 +362,104 @@ class StructureBlurrer:
                Sigma factor used for blurring
         """
 
-        dict_res_indices = {}
-        dict_res_dist = {}
-        dict_chain_res = {}
-        dict_chain_indices = {}
-        gridtree = self.maptree(emmap)
+        dict_res_indices = OrderedDict()
+        dict_res_CA = OrderedDict()
+        dict_chain_res = OrderedDict()
+        dict_chain_CA = OrderedDict()
+        dict_chain_indices = OrderedDict()
+        gridtree = self.maptree(emmap)[0]
         points = []
         #get chain details
         currentChain = structure_instance.atomList[0].chain
+        prev_res = structure_instance.atomList[0].get_res_no()
+        prev_resname = structure_instance.atomList[0].res
+        ca_coord = ['','','']
         for x in structure_instance.atomList:
+            if atom_sel == 'CA' and not x.atom_name == 'CA': continue
+            elif atom_sel == 'AR' and x.res not in ['TYR','PHE','TRP'] \
+                                and not x.atom_name == 'CA': continue
+                
+            cur_res = x.get_res_no()
             if not x.chain == currentChain:
+                #print 'New chain: ', x.chain
                 #uniquify lists
-                for el in dict_res_indices:
-                    tmplist = dict_res_indices[el][:]
+                for e in dict_res_indices:
+                    tmplist = dict_res_indices[e][:]
                     setlist = set(tmplist)
-                    dict_res_indices[el] = list(setlist)
-                dict_chain_indices[currentChain] = dict_res_indices.copy()
+                    dict_res_indices[e] = list(setlist)
+                
+                try: dict_chain_indices[currentChain].update(dict_res_indices.copy())
+                except KeyError: dict_chain_indices[currentChain] = dict_res_indices.copy()
+                
+                try: dict_chain_CA[currentChain].update(dict_res_CA.copy())
+                except KeyError: dict_chain_CA[currentChain] = dict_res_CA.copy()
                 currentChain = x.chain
                 dict_res_indices = {}
-            cur_chain = x.chain
-            cur_res = x.get_res_no()
+            else:
+                 #check dict_CA if atom_centre not found for prev res
+                if cur_res != prev_res:
+                    if not dict_res_CA.has_key(prev_res):
+                        try: dict_res_CA[prev_res] = [prev_resname].extend(ca_coord)
+                        except AttributeError: 
+                            dict_res_CA[prev_res] = [' '].extend(ca_coord)
+            
+            currentChain = x.chain
             #save residue numbers in order
             try: 
-              if not cur_res in dict_chain_res[currentChain]: dict_chain_res[currentChain].append(cur_res)
+              if not cur_res in dict_chain_res[currentChain]: 
+                  dict_chain_res[currentChain].append(cur_res)
             except KeyError: dict_chain_res[currentChain] = [cur_res]
             #return indices covered by gaussian blur
-            if not res_map is None: points = self.mapGridPositions(emmap,x,gridtree,res_map,sim_sigma_coeff)
+            if not res_map is None: 
+                points = self.mapGridPositions( \
+                                            emmap,x,gridtree,res_map,
+                                            sim_sigma_coeff,sigma_thr)
             #return indices covered by vdW radii
             else: points = self.mapGridPositions_vdw(emmap,x,gridtree)
+            
+            #nucleic acids
+            if x.res in ['A','T','C','G','U']:
+                if atom_centre in ['CA','CB']:
+                    atom_centre = "C1'"
+                if x.atom_name in ["P","C3'","C1'"]:
+                    ca_coord = [x.x, x.y, x.z]
+            elif x.atom_name == 'CA':
+                ca_coord = [x.x, x.y, x.z]
+            if x.atom_name == atom_centre: #x.fullid
+                #CA coordinates
+                try: dict_res_CA[cur_res] = [x.res, x.x, x.y, x.z]
+                except AttributeError: 
+                    dict_res_CA[cur_res] = [' ', x.x, x.y, x.z]
+            
+            prev_res = cur_res
+            prev_resname = x.res
+            #continue if no indices are returned
             if len(points) == 0:
                 dict_res_indices[cur_res] = []
                 continue
-            if x.atom_name == 'CA': #x.fullid
-                #CA coordinates
-                dict_res_dist[cur_res] = [x.x, x.y, x.z]
             # get points occupied by the residue
             if dict_res_indices.has_key(cur_res):
                 dict_res_indices[cur_res].extend(points)
             else: dict_res_indices[cur_res] = points
         if not dict_chain_indices.has_key(currentChain):
             #uniquify lists
-            for el in dict_res_indices:
-                tmplist = dict_res_indices[el][:]
+            for e in dict_res_indices:
+                tmplist = dict_res_indices[e][:]
                 setlist = set(tmplist)
-                dict_res_indices[el] = list(setlist)
-            dict_chain_indices[currentChain] = dict_res_indices.copy()
-        return dict_chain_indices, dict_chain_res,dict_res_dist
+                dict_res_indices[e] = list(setlist)
+            try: dict_chain_indices[currentChain].update(dict_res_indices.copy())
+            except KeyError: dict_chain_indices[currentChain] = dict_res_indices.copy()
+            try: dict_chain_CA[currentChain].update(dict_res_CA.copy())
+            except KeyError: dict_chain_CA[currentChain] = dict_res_CA.copy()
+        if not dict_res_CA.has_key(cur_res):
+            try: dict_res_CA[cur_res] = [x.res].extend(ca_coord)
+            except AttributeError: 
+                    dict_res_CA[cur_res] = [' '].extend(ca_coord)
+        return dict_chain_indices, dict_chain_res,dict_chain_CA, gridtree
+    
     #added by aj
-    def _get_map_values(self,structure_instance,emmap,res_map,sim_sigma_coeff=0.187,win=5):
+    def get_map_values(self,structure_instance,emmap,res_map,
+                       sim_sigma_coeff=0.187,win=5,sigma_thr=2.0):
         """
         
         Returns avg map density from voxels occupied by overlapping residue fragments.
@@ -406,7 +477,10 @@ class StructureBlurrer:
            *win*
                Fragment length (odd values)
         """
-        dict_chain_indices, dict_chain_res,dict_res_dist = self.get_indices(structure_instance,emmap,res_map,sim_sigma_coeff)
+        dict_chain_indices, dict_chain_res,dict_chain_CA, gridtree = \
+            self.get_indices(structure_instance,emmap,res_map,
+                             sim_sigma_coeff=sim_sigma_coeff,
+                             sigma_thr=sigma_thr)
         origin = emmap.origin
         apix = emmap.apix
         box_size = emmap.box_size()
@@ -418,7 +492,11 @@ class StructureBlurrer:
         for ch in dict_chain_indices:
             dict_res_scores = {}
             dict_res_indices = dict_chain_indices[ch]    
+            prev_mapval = 0.0
             for res in dict_res_indices:
+                try:
+                    resname = dict_chain_CA[ch][res][0]
+                except KeyError: resname = ' '
                 if not dict_res_scores.has_key(res):
                     indices = dict_res_indices[res][:]
                     #consider residues on both sides. NOTE: wont work for insertion codes!
@@ -449,10 +527,17 @@ class StructureBlurrer:
                             except (IndexError,KeyError): dict_res_scores[res] = 0.0
                         continue
                     '''
+                    res_id = str(res) + '_' + resname
                     array_indices = array(sc_indices)
                     ind_arrxyz = transpose(array_indices)
-                    ind_arrzyx = (ind_arrxyz[2],ind_arrxyz[1],ind_arrxyz[0])
-                dict_res_scores[res] = npmean(emmap.fullMap[ind_arrzyx])
+                    try:
+                        ind_arrzyx = (ind_arrxyz[2],ind_arrxyz[1],ind_arrxyz[0])
+                    except IndexError:
+                        dict_res_scores[res_id] = prev_mapval
+                        continue
+                    mapval = npmean(emmap.fullMap[ind_arrzyx])
+                    dict_res_scores[res_id] = mapval
+                    prev_mapval = mapval
             dict_chain_scores[ch] = dict_res_scores.copy()
         return dict_chain_scores
                     
@@ -478,7 +563,7 @@ class StructureBlurrer:
         dict_res_indices = {}
         dict_res_dist = {}
         dict_chain_indices = {}
-        gridtree = self.maptree(emmap)
+        gridtree = self.maptree(emmap)[0]
         points = []
         #get chain details
         currentChain = structure_instance.atomList[0].chain
@@ -594,7 +679,7 @@ class StructureBlurrer:
         nz,ny,nx = densMap.fullMap.shape
         zg,yg,xg = mgrid[0:nz,0:ny,0:nx]
         indi = zip(xg.ravel(), yg.ravel(), zg.ravel())
-        gridtree = self.maptree(densMap)
+        gridtree = self.maptree(densMap)[0]
         
         currentChain = prot.atomList[0].chain
         for x in prot.atomList:
@@ -827,7 +912,7 @@ class StructureBlurrer:
         return newMap
 
     #add IF
-    def gaussian_blur_real_space(self, prot, resolution, densMap=False, sigma_coeff=0.356, normalise=True,filename="None"):
+    def gaussian_blur_real_space_1(self, prot, resolution, densMap=False, sigma_coeff=0.356, normalise=True,filename="None"):
         
         """
         
@@ -869,21 +954,130 @@ class StructureBlurrer:
             #>
             densMap = self.protMap(prot, max(1.0,min(resolution/4., 3.5)), resolution)
             print "WARNING: Use StructureBlurrer.gaussian_blur_real_space_box() to blured a map with a user defined defined cubic box"
-        x_s = int(densMap.x_size()*densMap.apix)
-        y_s = int(densMap.y_size()*densMap.apix)
-        z_s = int(densMap.z_size()*densMap.apix)
+        #TEMPy class
+        if densMap.__class__.__name__ == 'Map':
+            x_s = int(densMap.x_size()*densMap.apix)
+            y_s = int(densMap.y_size()*densMap.apix)
+            z_s = int(densMap.z_size()*densMap.apix)
+            newMap = Map(zeros((z_s, y_s, x_s)), densMap.origin, densMap.apix, 'mapname')
+            newMap.apix = (densMap.apix*densMap.x_size())/x_s
+            densMap_apix = densMap.apix
+        #MapEdit/Filter class
+        else:
+            if type(densMap.apix) is float: 
+                densMap_apix = densMap.apix
+                x_s = int(densMap.x_size()*densMap_apix)
+                y_s = int(densMap.y_size()*densMap_apix)
+                z_s = int(densMap.z_size()*densMap_apix)
+            else: 
+                densMap_apix = densMap.apix[0]
+                x_s = int(densMap.x_size()*densMap.apix[0])
+                y_s = int(densMap.y_size()*densMap.apix[1])
+                z_s = int(densMap.z_size()*densMap.apix[2])
+            newMap = Map(zeros((z_s, y_s, x_s)), densMap.origin, densMap_apix, 'mapname')
+            newMap.apix = (densMap_apix*densMap.x_size())/x_s
         ##newMap = densMap.resample_by_box_size([z_s, y_s, x_s])
         ##newMap.fullMap *= 0
-        newMap = densMap.copy()
-        newMap.fullMap = zeros((z_s, y_s, x_s))
-        newMap.apix = (densMap.apix*densMap.x_size())/x_s
-        sigma = max(sigma_coeff*resolution,1.0)
+        
+        sigma = sigma_coeff*resolution
         newMap = self.make_atom_overlay_map(newMap, prot)
         gauss_map = gaussian_filter(newMap.fullMap, sigma)
+        del newMap.fullMap
         newMap.fullMap = gauss_map
-        newMap = newMap.resample_by_box_size(densMap.box_size())
+        newMap = newMap.downsample_map(densMap_apix,grid_shape=densMap.fullMap.shape)
+        del gauss_map
         if normalise:
             newMap = newMap.normalise()
+        newMap.update_header()
+        gc.collect()
+        return newMap
+    #edited by AJ
+    def gaussian_blur_real_space(self, prot, resolution, densMap=False, sigma_coeff=0.225, normalise=True,filename="None"):
+        
+        """
+        
+        Returns a Map instance based on a Gaussian blurring of a protein.
+        The convolution of atomic structures is done in real space
+        
+
+        Arguments:
+
+            *prot*
+                the Structure instance to be blurred.
+            *resolution*
+                the resolution, in Angstroms, to blur the protein to.
+            *densMap*
+                False to build a Map with dimensions based on the protein, or a Map instance to be used as a template.
+            *sigma_coeff*
+                the sigma value (multiplied by the resolution) that controls the width of the Gaussian. 
+                Default values is 0.356.
+                
+                Other values used :
+                
+                    0.187R corresponding with the Gaussian width of the Fourier transform falling to half the maximum at 1/resolution, as used in Situs (Wriggers et al, 1999);
+                    
+                    0.225R which makes the Fourier transform of the distribution fall to 1/e of its maximum value at wavenumber 1/resolution, the default in Chimera (Petterson et al, 2004)
+                    
+                    0.356R corresponding to the Gaussian width at 1/e maximum height equaling the resolution, an option in Chimera (Petterson et al, 2004);
+                    
+                    0.425R the fullwidth half maximum being equal to the resolution, as used by FlexEM (Topf et al, 2008);
+                                
+                    0.5R the distance between the two inflection points being the same length as the resolution, an option in Chimera (Petterson et al, 2004);
+                                
+                    1R where the sigma value simply equal to the resolution, as used by NMFF (Tama et al, 2004).
+
+           *filename*
+                output name of the map file.
+                
+        """
+        if not densMap:
+            #>
+            densMap = self.protMap(prot, max(1.0,min(resolution/4., 3.5)), resolution)
+            print "WARNING: Use StructureBlurrer.gaussian_blur_real_space_box() to blured a map with a user defined defined cubic box"
+        #TEMPy class
+        if densMap.__class__.__name__ == 'Map':
+            if densMap.apix > 1.0:
+                x_s = int(densMap.x_size()*densMap.apix)
+                y_s = int(densMap.y_size()*densMap.apix)
+                z_s = int(densMap.z_size()*densMap.apix)
+            else: x_s,y_s,z_s = densMap.x_size(),densMap.y_size(),densMap.z_size()
+            newMap = Map(zeros((z_s, y_s, x_s)), densMap.origin, densMap.apix, 'mapname')
+            newMap.apix = (densMap.apix*densMap.x_size())/x_s
+            densMap_apix = densMap.apix
+        #MapEdit/Filter class
+        else:
+            if type(densMap.apix) is float: 
+                densMap_apix = densMap.apix
+                if densMap.apix > 1.0:
+                    x_s = int(densMap.x_size()*densMap_apix)
+                    y_s = int(densMap.y_size()*densMap_apix)
+                    z_s = int(densMap.z_size()*densMap_apix)
+                else: x_s,y_s,z_s = densMap.x_size(),densMap.y_size(),densMap.z_size()
+            else: 
+                densMap_apix = densMap.apix[0]
+                if densMap.apix[0] > 1.0:
+                    x_s = int(densMap.x_size()*densMap.apix[0])
+                    y_s = int(densMap.y_size()*densMap.apix[1])
+                    z_s = int(densMap.z_size()*densMap.apix[2])
+                else:
+                    x_s,y_s,z_s = densMap.x_size(),densMap.y_size(),densMap.z_size()
+            newMap = Map(zeros((z_s, y_s, x_s)), densMap.origin, densMap_apix, 'mapname')
+            newMap.apix = (densMap_apix*densMap.x_size())/x_s
+        ##newMap = densMap.resample_by_box_size([z_s, y_s, x_s])
+        ##newMap.fullMap *= 0
+        
+        sigma = sigma_coeff*resolution
+        sigma = max(1,round(sigma/newMap.apix,0))
+        newMap = self.make_atom_overlay_map(newMap, prot)
+        gauss_map = gaussian_filter(newMap.fullMap, sigma)
+        del newMap.fullMap
+        newMap.fullMap = gauss_map
+        newMap = newMap.downsample_map(densMap_apix,grid_shape=densMap.fullMap.shape)
+        del gauss_map
+        if normalise:
+            newMap = newMap.normalise()
+        newMap.update_header()
+        gc.collect()
         return newMap
 
     def gaussian_blur_real_space_box(self, prot, resolution, box_size_x, box_size_y, box_size_z, sigma_coeff=0.356, normalise=True,filename="None"):
