@@ -287,9 +287,8 @@ class MapEdit(object):
         Set dimensions (Angstroms) given voxel size
         """
         self.apix = apix
-        self.dim = (self.x_size()*self.apix[0],
-                        self.y_size()*self.apix[1],
-                        self.z_size()*self.apix[2])
+        self.dim = (self.x_size()*self.apix[0],self.y_size()*self.apix[1],
+                    self.z_size()*self.apix[2])
         
     def set_apix_dim(self,dim):
         """
@@ -311,6 +310,40 @@ class MapEdit(object):
                 self.downsample_apix(max(self.apix),inplace=True)
                 self.apix = self.apix[0]
     
+    def fix_origin(self):
+        """
+        Set origin record based on nstart if non-zero
+        """
+        if self.origin[0] == 0. and self.origin[1] == 0. and \
+                            self.origin[2] == 0.:
+            if self.class_name == 'MrcFile':
+                if self.mrc.header.nxstart != 0 or \
+                    self.mrc.header.nystart != 0 or \
+                        self.mrc.header.nzstart != 0:
+                    if isinstance(self.apix, tuple):
+                        #origin
+                        self.origin = (self.mrc.header.nxstart * self.apix[0],
+                                        self.mrc.header.nystart * self.apix[1],
+                                        self.mrc.header.nzstart * self.apix[2])
+                    else:
+                        self.origin = (self.mrc.header.nxstart * self.apix,
+                                          self.mrc.header.nystart * self.apix,
+                                          self.mrc.header.nzstart * self.apix)
+            elif self.class_name == 'Map':
+                nstart_index = self.mrc.header[4:7]
+                if nstart_index[0] != 0 or \
+                    nstart_index[1] != 0 or \
+                        nstart_index[2] != 0:
+                    if isinstance(self.apix, tuple):
+                        #origin
+                        self.origin = (nstart_index[0] * self.apix[0],
+                                          nstart_index[1] * self.apix[1],
+                                          nstart_index[2] * self.apix[2])
+                    else:
+                        self.origin[0] = (nstart_index[0] * self.apix,
+                                            nstart_index[1] * self.apix,
+                                            nstart_index[2] * self.apix)
+                
     def set_apix_as_tuple(self):
         if not isinstance(self.apix, tuple):
             self.apix = (self.apix,self.apix,self.apix)
@@ -498,7 +531,11 @@ class MapEdit(object):
         grid_shape = int(round(self.z_size()*apix_ratio[2])),\
                     int(round(self.y_size()*apix_ratio[1])),\
                     int(round(self.x_size()*apix_ratio[0]))
-        self.interpolate_to_grid(grid_shape,new_spacing,self.origin,
+        if inplace:
+            self.interpolate_to_grid(grid_shape,new_spacing,self.origin,
+                                 inplace=inplace)
+        else:
+            return self.interpolate_to_grid(grid_shape,new_spacing,self.origin,
                                  inplace=inplace)
             
     #PROCESS DENSITY in the same grid
@@ -572,7 +609,7 @@ class MapEdit(object):
         return contour
 
     #divide map density into bins
-    def map_digitize(self,contour=None,nbins=10,right=False,
+    def map_digitize(self,contour=None,nbins=10,left=False,right=True,
                      inplace=False):
         """
         Divide density into bins (store bin indices in grid)
@@ -598,14 +635,15 @@ class MapEdit(object):
         bins = []
         step = (self.fullMap.max()-float(contour))/nbins
         # > contour value
-        if right: ini = float(contour)
+        if left: ini = float(contour) - (0.000001*step)
+        elif right: ini = float(contour)
         else: ini = float(contour) + (0.000001*step)
         bins.append(ini)
         for ii in range(1,nbins+1):
-                bins.append(float(contour) + ii*step)
+            bins.append(float(contour) + ii*step)
         if bins[-1] < self.fullMap.max():
-                bins = bins[:-1]
-                bins.append(self.fullMap.max())
+            bins = bins[:-1]
+            bins.append(self.fullMap.max())
         if not inplace: newmap = self.copy()
         for z in range(len(self.fullMap)):
             for y in range(len(self.fullMap[z])):
