@@ -202,19 +202,27 @@ class MetaDataTable(DataFrame):
 
     def convert_to_resolution_angstrom(self):
         '''
-        Convert from <4SSQ/LL> to Angstrom.  
-            N.B. Refmac occasionally labels <4SSQ/LL>  as 2sin(th)/l
+        Convert from <4SSQ/LL> or 2sin(th)/l to Angstrom.
+
+        1/d = 2sin(th)/l
+        1/d^2 = 4sin^2(th)/l^2 = <4SSQ/LL>
+
+        (Refmac uses <4SSQ/LL> in some places and 2sin(th)/l in others.)
         '''
+        # TODO: why are the resolution values stored as strings, not numbers?
         angstrom_label = (ur'Resolution (\u00c5)').encode('utf-8')
-        res_labels = ['<4SSQ/LL>', '2sin(th)/l']
-        for column_name in res_labels:
+        res_conversions = {
+            '<4SSQ/LL>': lambda x: str(1.0 / math.sqrt(float(x))),
+            '2sin(th)/l': lambda x: str(1.0 / float(x))
+        }
+        for column_name in res_conversions:
             if column_name in self.columns.values:
                 # to prevent division by zero as REFMAC5 sometimes outputs
                 # 0.000 for very low resolution shells (large structures)
                 self[column_name] = self[column_name].apply(
                     lambda x: x if float(x)>0 else 'NaN')
                 self[column_name] = self[column_name].apply(
-                    lambda x: str(1.0 / math.sqrt(float(x))))
+                    res_conversions[column_name])
                 self.rename(columns={column_name: angstrom_label},
                                      inplace=True)
 
