@@ -17,8 +17,9 @@ def plot_density_histogram(arr,plotfile='density_histogram.png'):
     assert len(freq) == len(bin_centre)
     
     try:
+        #import matplotlib
         import matplotlib.pyplot as plt
-    except RuntimeError,ImportError:
+    except (RuntimeError,ImportError) as e:
         plt = None
         return
     import warnings
@@ -309,8 +310,10 @@ def get_raps(arr,keep_shape=False,plotfile="ps.png",map_apix=1.):
         list_freq.append(float(r)/arr.shape[0])
         list_intensities.append(avg_shell_intensity)
     try:
+        import matplotlib
+        matplotlib.use('Agg')
         import matplotlib.pyplot as plt
-    except RuntimeError,ImportError:
+    except (RuntimeError,ImportError) as e:
         plt = None
         return list_freq,list_intensities
     
@@ -327,6 +330,32 @@ def get_raps(arr,keep_shape=False,plotfile="ps.png",map_apix=1.):
     plt.savefig(plotfile)
     plt.close()
 
+def calculate_fscavg(listfreq,listfsc,listnsf,
+                              map_apix,
+                              maxRes=None,minRes=None,
+                              list_weights=None):
+    if maxRes is None:
+        maxfreq_index = len(listfreq)
+    else:
+        maxfreq = 1./maxRes
+        print maxfreq
+        maxfreq_index = np.searchsorted(listfreq,maxfreq,side='right')
+    if minRes is None:
+        minfreq_index = 0
+    else:
+        minfreq = 1./minRes
+        minfreq_index = np.searchsorted(listfreq,minfreq)
+        
+    maxfreq_index = min(maxfreq_index,len(listfreq))
+    weighted_fsc_sum = 0.0
+    sum_nsf = 0.
+    for ind in xrange(minfreq_index,maxfreq_index):
+        print listfreq[ind], listfsc[ind],listnsf[ind]
+        weighted_fsc_sum += listfsc[ind]*listnsf[ind]
+        sum_nsf += listnsf[ind] 
+    FSCavg = weighted_fsc_sum/sum_nsf
+    return FSCavg
+
 def find_background_peak(arr, iter=2):
     lbin = np.amin(arr)
     rbin = np.amax(arr)
@@ -682,32 +711,6 @@ def calculate_shell_correlation(shell1,shell2):
     else: fsc = cov_ps1_ps2/(np.sqrt(var_ps1*var_ps2))
     return fsc
 
-def calculate_fscavg(listfreq,listfsc,listnsf,
-                              map_apix,
-                              maxRes=None,minRes=None,
-                              list_weights=None):
-    if maxRes is None:
-        maxfreq_index = len(listfreq)
-    else:
-        maxfreq = 1./maxRes
-        print maxfreq
-        maxfreq_index = np.searchsorted(listfreq,maxfreq,side='right')
-    if minRes is None:
-        minfreq_index = 0
-    else:
-        minfreq = 1./minRes
-        minfreq_index = np.searchsorted(listfreq,minfreq)
-        
-    maxfreq_index = min(maxfreq_index,len(listfreq))
-    weighted_fsc_sum = 0.0
-    sum_nsf = 0.
-    for ind in xrange(minfreq_index,maxfreq_index):
-        print listfreq[ind], listfsc[ind],listnsf[ind]
-        weighted_fsc_sum += listfsc[ind]*listnsf[ind]
-        sum_nsf += listnsf[ind] 
-    FSCavg = weighted_fsc_sum/sum_nsf
-    return FSCavg
-
 def calculate_fsc(ftarr1,ftarr2,dist1,dist2=None,
                      step=None,
                      maxlevel=None,
