@@ -146,7 +146,7 @@ def get_central_scaled_pixel_vals_after_scaling(emmap, modmap, masked_xyz_locs,
                                                     pyfftwobj=mod_pyfftwobj,
                                                     bytealigned=mod_inputarray)
 
-        scale_factors = compute_scale_factors(em_profile, mod_profile)
+        scale_factors = compute_scale_factors(em_profile, mod_profile,avg=avg)
         #for pyfftw ifft
         if cnt == 1.0:
             #plan ifft
