@@ -11,6 +11,7 @@ import multiprocessing
 import json
 import os
 import time
+from ccpem_core import ccpem_utils
 from ccpem_core import settings
 from ccpem_core import test_data
 from . import process_utils
@@ -26,7 +27,19 @@ class CCPEMPipelineCustomFinish(object):
 
     def on_finish(self, parent_pipeline=None):
         raise NotImplementedError('Subclasses should implement this')
+    
+    def on_part_fail(self,parent_pipeline=None):
+        raise NotImplementedError('Subclasses should implement this')
 
+class CCPEMPipelineCustomRunning(object):
+    '''
+    Class for custom pipeline running.
+    '''
+    def __init__(self):
+        pass
+
+    def on_running(self, parent_pipeline=None):
+        raise NotImplementedError('Subclasses should implement this')
 
 class CCPEMPipeline(object):
     '''
@@ -46,7 +59,7 @@ class CCPEMPipeline(object):
                  taskname=None,
                  title=None,
                  metadata=None,
-                 num_workers=None,
+                 on_running_custom = None,
                  on_finish_custom=None,
                  wait_json=None,
                  job_id=None,
@@ -78,11 +91,11 @@ class CCPEMPipeline(object):
             self.db = None
         self.db_id = None
         self.job_id = job_id
-        self.num_workers = num_workers
         self.wait_json = wait_json
         if metadata is not None:
             assert isinstance(metadata, dict)
         self.metadata = metadata
+        self.on_running_custom = on_running_custom
         self.on_finish_custom = on_finish_custom
         self.json = os.path.join(self.location,
                                  process_utils.task_filename)
@@ -97,8 +110,6 @@ class CCPEMPipeline(object):
             for stage in self.pipeline:
                 for job in stage:
                     job.export_json()
-        self.verbose = verbose
-        self.tp = None
 
     def start(self):
         '''
@@ -132,15 +143,15 @@ class CCPEMPipeline(object):
         self.metadata[key] = value
         self.export_json()
 
-    def get_metadata(self,
-                     key):
-        metadata = json.load(fp=open(self.json, 'r'))['metadata']
-        self.metadata = metadata
-        if self.metadata is not None:
-            if key in self.metadata.keys():
-                return self.metadata[key]
-        else:
-            return None
+    def get_metadata(self, key):
+        with open(self.json, 'r') as json_file:
+            metadata = json.load(fp=json_file)['metadata']
+            self.metadata = metadata
+            if self.metadata is not None:
+                if key in self.metadata.keys():
+                    return self.metadata[key]
+            else:
+                return None
 
     def on_start(self):
         if self.db is not None:
@@ -150,6 +161,7 @@ class CCPEMPipeline(object):
                 job_location=self.location,
                 title=self.title)
         self.set_status_running()
+        self.on_running()
 
     def on_wait(self):
         '''
@@ -161,15 +173,29 @@ class CCPEMPipeline(object):
                 return process_utils.get_process_status(
                     json_filepath=self.wait_json)
         return None
-
+    
+    def on_running(self):
+        # 
+        if self.on_running_custom is not None:
+            self.on_running_custom.on_running(parent_pipeline=self)
+            
     def on_finish(self):
         # Check all jobs finished
         failed = False
+        count_failed = 0
+        count_jobs = 0
         for stage in self.json_pl:
             for job in stage:
                 status = process_utils.get_process_status(json_filepath=job)
                 if status != 'finished':
                     failed = True
+                    count_failed += 1
+                count_jobs += 1
+        #if multiple parallel jobs are run and
+        #custom on_part_fail is provided
+        if count_failed > 0 and float(count_failed)/count_jobs < 0.25:
+            if self.on_finish_custom is not None:
+                self.on_finish_custom.on_part_fail(parent_pipeline=self)
         if failed:
             self.set_status_failed()
         else:
@@ -227,19 +253,19 @@ class CCPEMPipeline(object):
                      'args_path': self.args_path,
                      'database_path': self.database_path,
                      'db_id': self.db_id,
-                     'num_workers': self.num_workers,
                      'metadata': self.metadata,
                      'pipeline': self.json_pl}
-        json.dump(json_dict,
-                  fp=open(self.json, 'w'),
-                  sort_keys=True,
-                  separators=(',', ': '),
-                  indent=4)
+        ccpem_utils.atomic_write_json(json_dict,
+                                      self.json,
+                                      sort_keys=True,
+                                      separators=(',', ': '),
+                                      indent=4)
 
     def import_json(self, jsonfile):
         self.json = jsonfile
-        self.__dict__.update(json.load(fp=open(self.json, 'r')))
-        self.restore_processes()
+        with open(self.json, 'r') as json_file:
+            self.__dict__.update(json.load(fp=json_file))
+            self.restore_processes()
 
     def restore_processes(self):
         pipeline = []
