@@ -80,6 +80,12 @@ def ccpem_settings_parser():
         help='Location of chimera binary',
         type=str,
         default=None)
+    additonal_program_locations.add_argument(
+        '-ChimeraX_bin',
+        '--ChimeraX_bin',
+        help='Location of ChimeraX binary',
+        type=str,
+        default=None)
     additonal_program_locations.add_argument(
         '-ccp4mg_bin',
         '--ccp4mg_bin',
@@ -105,6 +111,13 @@ def ccpem_settings_parser():
         type=str,
         default=None)
     #
+    additonal_program_locations.add_argument(
+        '-haruspex_path',
+        '--haruspex_path',
+        help='Location of root haruspex library path for testing',
+        type=str,
+        default=None)
+    #
     reset_to_defaults = parser.add_argument_group()
     reset_to_defaults.add_argument(
         '-reset',
@@ -229,6 +242,17 @@ def which(program, mode=os.F_OK | os.X_OK, path=None):
             cmd = '/Applications/Chimera.app/Contents/MacOS/chimera'
             if _access_check(cmd, mode=mode):
                 return cmd
+        # ChimeraX has version number in directory name.  Find and return
+        # most recent version from sorted list
+        if program == 'ChimeraX':
+            files = os.listdir('/Applications/')
+            files.sort(reverse=True)
+            for f in files:
+                if 'ChimeraX' in f:
+                    name = f
+                    cmd = '/Applications/' + name + '/Contents/MacOS/ChimeraX'
+                    if _access_check(cmd, mode=mode):
+                        return cmd
         if program == 'coot':
             # Look for ccp4 coot (ccp-em coot would have been found and
             # returned already if it was available).
