@@ -15,7 +15,8 @@ import json
 
 class SetCaBLAMResults():
     def __init__(self,cablam_process,list_chains=[],
-                 dssp_dict=None, cootset_instance=None):
+                 dssp_dict=None, cootset_instance=None,
+                 glob_reportfile=None):
         self.cablam_process = cablam_process
         self.cootset = cootset_instance
         self.list_chains = list_chains
@@ -40,7 +41,7 @@ class SetCaBLAMResults():
             cm = CaBLAM_LogParser(cablam_job.stdout)
             self.dict_logparse[modelid] = cm
             #self.residue_outliers[modelid] = {}
-            self.residue_coordinates[modelid] = {}
+            
             if len(list_chains) == 0:
                 for ch in cm.list_chains:
                     if not ch in self.list_chains:
@@ -49,7 +50,14 @@ class SetCaBLAMResults():
             self.dict_jsonfiles[modelid] = os.path.join(job_location,
                                                         modelid+"_cablam.json")
         
+        self.glob_reportfile = glob_reportfile
+        if self.glob_reportfile is not None:
+            self.glob_reportfile.write('*CaBLAM* (backbone geometry)\n')
+            self.glob_reportfile.write('============================\n')
         self.set_global_results()
+        
+        if self.glob_reportfile is not None: 
+            self.glob_reportfile.write('----------------------------\n\n')
         self.set_local_data()
         self.save_outlier_json()
         self.add_cootdata()
@@ -82,10 +90,16 @@ class SetCaBLAMResults():
                                   list_horz_header_tooltips)
         ct_model = 0
         for mid in self.list_modelids:
+            if self.glob_reportfile is not None:
+                self.glob_reportfile.write('>{}\n'.format(mid))
             dict_summary = self.dict_logparse[mid].dict_summary
             data_array = [[dict_summary[gs.list_vert_headers[l]]] \
                           for l in xrange(len(gs.list_vert_headers))]
             add_data_to_table(table_id,data_array, start_col=ct_model)
+            for l in xrange(len(gs.list_vert_headers)):
+                if self.glob_reportfile is not None:
+                    self.glob_reportfile.write('-{}: {}%\n'.format(gs.list_vert_headers[l],
+                                                         data_array[l][0]))
             ct_model += 1
         pyrvapi.rvapi_flush()
 
@@ -122,12 +136,14 @@ class SetCaBLAMResults():
         
         for measure in list_measures:
             for mid in self.list_modelids:
-                if not mid in self.residue_outliers:
-                    self.residue_outliers[mid] = {}
-                if not chain in self.residue_outliers[mid]:
-                    #save outlier residues
-                    self.residue_outliers[mid][chain] = {}
-                    self.residue_coordinates[mid][chain] = {}
+                if 'Disfavored' not in measure:
+                    if not mid in self.residue_outliers:
+                        self.residue_outliers[mid] = {}
+                        self.residue_coordinates[mid] = {}
+                    if not chain in self.residue_outliers[mid]:
+                        #save outlier residues
+                        self.residue_outliers[mid][chain] = {}
+                        self.residue_coordinates[mid][chain] = {}
                 lp = self.dict_logparse[mid]
                 if not chain in lp.dict_outliers: continue
                 dict_outlier_data = lp.dict_outliers[chain]
@@ -159,7 +175,8 @@ class SetCaBLAMResults():
                     #7 ILE
                     val_split = val.split(' ')
                     #save outlier residue and outlier type
-                    self.residue_outliers[mid][chain][str(val_split[0])] = \
+                    if 'Disfavored' not in measure:
+                        self.residue_outliers[mid][chain][str(val_split[0])] = \
                                     outlier_settings.data_titles[measure] 
                 outlier.append(roundedval)
                 ct_val += 1
@@ -186,8 +203,8 @@ class SetCaBLAMResults():
 
                     CAx,CAy,CAz = [float(val) for val in \
                                     self.dssp_dict[mid][chain][residue_number][-3:] ]
-                    
-                    self.residue_coordinates[mid][chain][str(val_split[0])] = \
+                    if 'Disfavored' not in measure:
+                        self.residue_coordinates[mid][chain][str(val_split[0])] = \
                                     (CAx,CAy,CAz)
                 except KeyError:
                     pass
