@@ -27,7 +27,8 @@ angstrom_label = (ur'(\u00c5)').encode('utf-8')
 class SetMolprobityResults():
     def __init__(self,molprobity_process,
                  list_chains=[],
-                 cootset_instance=None):
+                 cootset_instance=None,
+                 glob_reportfile=None):
         self.molprobity_process = molprobity_process
         self.cootset = cootset_instance
         self.list_chains = list_chains
@@ -43,6 +44,7 @@ class SetMolprobityResults():
         self.dict_outlier_details = OrderedDict()
         #output json files for each model
         self.dict_jsonfiles = OrderedDict()
+        self.dict_json_cacoord = OrderedDict()
         self.list_chains = []
         dict_cootset = {}
         self.add_chain_sections = False
@@ -80,7 +82,17 @@ class SetMolprobityResults():
                         self.list_chains.append(chain)
                 self.add_chain_sections = True
             else: self.list_chains = list_chains[:]
+            
+            ca_coord_jsonfile = os.path.join(molprocess.location,'ca_coord.json')
+            self.dict_json_cacoord[model_id] = ca_coord_jsonfile
+        
+        self.glob_reportfile = glob_reportfile
+        if self.glob_reportfile is not None:
+            self.glob_reportfile.write('*Molprobity* model geometry\n')
+            self.glob_reportfile.write('===========================\n')
         self.set_global_results()
+        if self.glob_reportfile is not None:
+            self.glob_reportfile.write('---------------------------\n\n')
         self.set_local_data()
         self.save_outlier_json()
         self.set_local_results()
@@ -106,12 +118,16 @@ class SetMolprobityResults():
                 hheader = gs.list_horz_headers[h]
                 list_horz_headers.append(hheader+"<br>({})".format(mid))
                 list_horz_header_tooltips.append(gs.list_horz_header_tooltips[h])
+                
         list_horz_headers.append('Expected range')
         list_horz_header_tooltips.append('Expected/acceptable range')
         add_horz_headers_to_table(table_id,list_horz_headers,
                                   list_horz_header_tooltips)
         mnum = 0
         for mid in self.list_modelids:
+            if self.glob_reportfile is not None:
+                self.glob_reportfile.write('>{}\n'.format(mid))
+                
             lp = self.dict_logparse[mid]
             dict_summary = lp.dict_summary
             self.add_global_data(table_id,dict_summary,gs,mnum)
@@ -143,12 +159,15 @@ class SetMolprobityResults():
                     percentile = stat_split[1]
             if isinstance(stat_value,float):
                 stat_value = "{:.2f}".format(round(stat_value,2))
+            if len(self.list_modelids) > 1:#add individual percentiles
+                stat_value = stat_value + '<br>' + percentile
             pyrvapi.rvapi_put_table_string(table_name,stat_value,i,mnum)
             
             if mnum == 0:
                 if gs.dict_referencerange.has_key(statname):
                     if statname in ['Clashscore','MolProbity score']:
-                        pyrvapi.rvapi_put_table_string(table_name,
+                        if len(self.list_modelids) == 1:
+                            pyrvapi.rvapi_put_table_string(table_name,
                                            percentile,
                                            i,len(self.list_modelids))
                     else:
@@ -156,6 +175,26 @@ class SetMolprobityResults():
                                            gs.dict_referencerange[statname],
                                            i,len(self.list_modelids))
             i += 1
+            
+            if self.glob_reportfile is not None:
+                if '<br>' in stat_value:
+                    stat_value = " ".join(stat_value.split('<br>'))
+                        
+                statname_fixed = statname
+                if statname == 'favored':
+                    statname_fixed = 'Ramachandran '+statname
+                if statname in ['Clashscore','MolProbity score'] and \
+                                                len(self.list_modelids) == 1:
+                    self.glob_reportfile.write('-{}: {} [{}] \n'.format(
+                                                        statname_fixed,
+                                                         stat_value,
+                                                         percentile))
+                else:
+                    self.glob_reportfile.write('-{}: {} [expected: {}] \n'.format(
+                                                        statname_fixed,
+                                                         stat_value,
+                                                         gs.dict_referencerange[statname]))
+                    
         pyrvapi.rvapi_flush()
     
     def set_local_data(self):
@@ -196,6 +235,11 @@ class SetMolprobityResults():
             self.residue_coordinates[mid] = {}
         if not chain in self.residue_coordinates[mid]:
             self.residue_coordinates[mid][chain] = {}
+        try:
+            with open(self.dict_json_cacoord[mid],'r') as cj:
+                dict_cacoord = json.load(cj)
+        except KeyError: dict_cacoord = {}
+        
         for measure in dict_outlier_data:
             l = 0
             ct_res = 0
@@ -295,16 +339,44 @@ class SetMolprobityResults():
                         resdet_split = residue_details[0].split('<br>')
                         #save outlier residue and outlier type
                         outlier_title = outlier_settings.data_titles[measure]
+                        #remove'outliers' from measure
+                        if 'outliers' in outlier_title: 
+                            outlier_title = ' '.join(outlier_title.split()[:-1])
+                        outlier_title_ext = outlier_title
+                        flag_pass = 1
+                        res_prev = ''
                         #multiple residues
                         for resdet in resdet_split:
                             #G T   8
                             if measure == 'Backbone torsion suites':
                                 residue_num = str(resdet.split()[2])
-                            else: residue_num = str(resdet.split()[0])
-                            #TODO: disabled adding non cootdata to summary table
-#                             self.add_outlier_title_for_residues(outlier_title, 
-#                                                                 chain, residue_num,
-#                                                                 mnum=mnum)
+                                #fix this
+                                try: atom_name = str(resdet.split()[3])
+                                except IndexError: atom_name = ' '
+                            else: 
+                                residue_num = str(resdet.split()[0])
+                                try: atom_name = str(resdet.split()[2])
+                                except IndexError:
+                                    print atom_name
+                            
+                            try:
+                                if len(res_prev) > 0 and res_prev != residue_num:
+                                    self.add_outlier_title_for_residues(outlier_title_ext, 
+                                                                     chain, res_prev,
+                                                                     mnum=mnum)
+                                    outlier_title_ext = outlier_title
+                                res_coordinate = tuple(dict_cacoord['1'][chain][residue_num][0])
+                                self.residue_coordinates[mid][chain][residue_num] = \
+                                                    res_coordinate
+                                
+                                outlier_title_ext += ':'+atom_name
+                                res_prev = residue_num
+                            except (KeyError,TypeError) as exc: 
+                                flag_pass = 0
+                        if flag_pass == 1:
+                            self.add_outlier_title_for_residues(outlier_title_ext, 
+                                                                     chain, residue_num,
+                                                                     mnum=mnum)
                     if len(residue_details) == 0: continue
                     list_data.append(residue_details)
             if len(list_data) == 0: continue
@@ -529,6 +601,27 @@ class global_summary_settings():
                                 'Twisted Proline',
                                 'Twisted General']
 
+def get_gemmi_object(model_path):
+    structure = gemmi.read_structure(model_path)
+    return structure
+
+def get_coordinates(structure,chainID,resnum):
+    list_coord = []
+    for model in structure:
+        for chain in model:
+            if chain.name != chainID: continue
+            polymer = chain.get_polymer()
+            #skip non polymers
+            if not polymer: continue
+            for residue in chain:
+                if residue.seqid.num != resnum: continue
+                residue_id = str(residue.seqid.num)+'_'+residue.name
+                for atom in residue:
+                    if atom.name == 'CA':
+                        coord = atom.pos #gemmi Position
+                        return (coord.x,coord.y,coord.z)
+                return None
+
 
 class residue_outlier_settings() :
     #NOTE: data_keys has same keywords as in molprobity coot file
