@@ -13,10 +13,14 @@ import json
 
 class SetSMOCResults():
     def __init__(self, smoc_process, list_chains=[],
-                 molprobity_process=None, cootset_instance=None):
-        self.smoc_process = smoc_process
+                 pipeline_location=None, cootset_instance=None,
+                 glob_reportfile=None):
+        self.smoc_process = sorted(smoc_process, 
+                                     key=lambda j: j.name.split(' ')[-1][-1]+
+                                                    j.name.split(' ')[0][-1])
         self.cootset=cootset_instance
         self.smoc_results = []
+        self.smoc_csvs = []#smoc scores per model
         self.smoc_dataframes = OrderedDict()
         #outlier details
         self.dict_outlier_details = OrderedDict()
@@ -28,10 +32,11 @@ class SetSMOCResults():
         self.residue_outliers = OrderedDict()
         self.residue_coordinates = OrderedDict()
         self.dict_cootdata = OrderedDict()
+        self.dict_ca_coord = OrderedDict()
         for smoc_job in smoc_process:
-            smoc_result = os.path.join(smoc_job.location,'smoc_score.txt')
+            smoc_result = os.path.join(smoc_job.location,'smoc_score.csv')
             self.smoc_results.append(smoc_result)
-
+            
             if os.path.isfile(smoc_result):
                 modelid = smoc_job.name.split(' ')[-1]
                 self.smoc_dataframes[modelid] = pd.read_csv(smoc_result,
@@ -40,9 +45,15 @@ class SetSMOCResults():
                 self.list_modelids.append(modelid)
                 self.dict_jsonfiles[modelid] = os.path.join(smoc_job.location,
                                                         modelid+"_smoc.json")
+                self.smoc_csvs.append(
+                        os.path.join(smoc_job.location,modelid+'_smoc.csv'))
                 pdbid = modelid.split(':')[-1]
                 if not pdbid in self.list_pdbids: 
                     self.list_pdbids.append(pdbid)
+                if pipeline_location is not None:
+                    ca_coordfile = os.path.join(pipeline_location,pdbid,'ca_coord.json')
+                    if os.path.isfile(ca_coordfile):
+                        self.dict_ca_coord[pdbid] = ca_coordfile
                 mapid = modelid.split(':')[0]
                 if not mapid in self.list_mapids: 
                     self.list_mapids.append(mapid)
@@ -92,10 +103,12 @@ class SetSMOCResults():
 #                 self.add_subsection_to_chain(chain,smoc_subsection)
         self.dict_pdbdata = OrderedDict()
         self.dict_mapdata = OrderedDict()
+        ct_model = 0
         for mid in self.list_modelids:
             if mid in self.smoc_dataframes:
                 df = self.smoc_dataframes[mid]
-                self.set_results_data(df, mid)
+                self.set_results_data(df, mid,self.smoc_csvs[ct_model])
+                ct_model += 1
     
     def save_outlier_json(self):
         for mid in self.dict_outlier_details:
@@ -117,9 +130,8 @@ class SetSMOCResults():
         pyrvapi.rvapi_add_section1(chain+'/'+smoc_subsection, 
                                'SMOC (Segment Manders\' Overlap Coefficient)', 
                                0, 0, 1, 1, False)
-        
                         
-    def set_results_data(self,df,mid):
+    def set_results_data(self,df,mid,mid_csv=None):
         mapid = mid.split(':')[0]
         pdbid = mid.split(':')[-1]
         list_chains = []
@@ -134,22 +146,35 @@ class SetSMOCResults():
         if not pdbid in self.residue_outliers:
             self.residue_outliers[pdbid] = {}
             self.residue_coordinates[pdbid] = {}
+        if mid_csv is not None:
+            scoreout = mid_csv
+            sco = open(scoreout,'w')
+            sco.write('#chain,residue,score,z_local,z_global\n')
         # Get data
         for pdb in df:
             if not '_' in pdb: continue
             #add plots for each chain
             chain = pdb.split('_')[-1]
+            
             ct_data_index = 0
             if '.' in chain: 
                 chain = chain.split('.')[0]
                 pdbname = '.'.join(pdb.split('.')[:-1])
             if chain == '': chain = ' '
             added_smoc_text = False
+            #check if ca coordinates are available
+            flag_coord_exists = False
+            if pdbid in self.dict_ca_coord:
+                ca_coord_json = self.dict_ca_coord[pdbid]
+                with open(ca_coord_json,'r') as j:
+                    dict_pdb_ca_coord = json.load(j)
+                flag_coord_exists = True
+            else:
+                dict_pdb_ca_coord = {}
+            
             #add chain sections
             if self.add_chain_sections:
-                if len(self.list_chains) == 0:
-                     self.list_chains = [chain]
-                elif chain not in self.list_chains:
+                if not chain in self.list_chains:
                     self.list_chains.append(chain)
                 chain_section = chain
                 chain_name = 'chain '+chain
@@ -170,32 +195,53 @@ class SetSMOCResults():
                 if pdb_df[0] == 'resnum': 
                     dict_results['residue'] = []
                     for val in pdb_df[1:]:
-                        dict_results['residue'].append(int(float(val)))
+                        residue_num = int(float(val))
+                        dict_results['residue'].append(residue_num)
+                        try:
+                            ca_coord = dict_pdb_ca_coord["1"][chain][str(residue_num)][0]
+                            res_name = dict_pdb_ca_coord["1"][chain][str(residue_num)][1]
+                            try:
+                                dict_results['CAx'].append(ca_coord[0])
+                                dict_results['CAy'].append(ca_coord[1])
+                                dict_results['CAz'].append(ca_coord[2])
+                                dict_results['resname'].append(res_name)
+                            except KeyError:
+                                dict_results['CAx'] = [ca_coord[0]]
+                                dict_results['CAy'] = [ca_coord[1]]
+                                dict_results['CAz'] = [ca_coord[2]]
+                                dict_results['resname'] = [res_name]
+                        except: 
+                            flag_coord_exists = False
                 
-                if pdb_df[0] == 'resname':
-                    dict_results['resname'] = []
-                    for val in pdb_df[1:]:
-                        dict_results['resname'].append(val)
+                if not flag_coord_exists:
+                    if 'CAx' in dict_results: del dict_results['CAx']
+                    if 'CAy' in dict_results: del dict_results['CAy']
+                    if 'CAz' in dict_results: del dict_results['CAz']
+                
+#                 if pdb_df[0] == 'resname':
+#                     dict_results['resname'] = []
+#                     for val in pdb_df[1:]:
+#                         dict_results['resname'].append(val)
                 
                 if pdb_df[0] == 'smoc':
                     dict_results['smoc'] = []
                     for val in pdb_df[1:]:
                         dict_results['smoc'].append(float(val))
                 
-                if pdb_df[0] == 'CAx':
-                    dict_results['CAx'] = []
-                    for val in pdb_df[1:]:
-                        dict_results['CAx'].append(float(val))
-                
-                if pdb_df[0] == 'CAy':
-                    dict_results['CAy'] = []
-                    for val in pdb_df[1:]:
-                        dict_results['CAy'].append(float(val))
-                
-                if pdb_df[0] == 'CAz':
-                    dict_results['CAz'] = []
-                    for val in pdb_df[1:]:
-                        dict_results['CAz'].append(float(val))
+#                 if pdb_df[0] == 'CAx':
+#                     dict_results['CAx'] = []
+#                     for val in pdb_df[1:]:
+#                         dict_results['CAx'].append(float(val))
+#                 
+#                 if pdb_df[0] == 'CAy':
+#                     dict_results['CAy'] = []
+#                     for val in pdb_df[1:]:
+#                         dict_results['CAy'].append(float(val))
+#                 
+#                 if pdb_df[0] == 'CAz':
+#                     dict_results['CAz'] = []
+#                     for val in pdb_df[1:]:
+#                         dict_results['CAz'].append(float(val))
                 
                 if 'residue' in dict_results and 'smoc' in dict_results:
                     try:
@@ -219,18 +265,19 @@ class SetSMOCResults():
                     self.dict_pdbdata[chain][pdbid][mapid] = [
                                                 dict_results['residue'][:],
                                                 dict_results['smoc'][:]]
-                    #save outliers in dict
-                    for l in xrange(len(dict_results['residue'])):
-                        try:
-                            self.dict_outlier_details[mid][chain].append([
-                                                        str(dict_results['residue'][l]) +' '+
-                                                        dict_results['resname'][l],
-                                                        dict_results['smoc'][l]])
-                        except KeyError:
-                            self.dict_outlier_details[mid][chain] = [[
-                                                        str(dict_results['residue'][l]) +' '+
-                                                        dict_results['resname'][l], 
-                                                        dict_results['smoc'][l]]]
+                    if 'resname' in dict_results:
+                        #save outliers in dict
+                        for l in xrange(len(dict_results['residue'])):
+                            try:
+                                self.dict_outlier_details[mid][chain].append([
+                                                            str(dict_results['residue'][l]) +' '+
+                                                            dict_results['resname'][l],
+                                                            dict_results['smoc'][l]])
+                            except KeyError:
+                                self.dict_outlier_details[mid][chain] = [[
+                                                            str(dict_results['residue'][l]) +' '+
+                                                            dict_results['resname'][l], 
+                                                            dict_results['smoc'][l]]]
                     if not chain in self.dict_mapdata: 
                         self.dict_mapdata[chain] = OrderedDict()
                     if not mapid in self.dict_mapdata[chain]:
@@ -244,10 +291,22 @@ class SetSMOCResults():
                     
                     if 'resname' in dict_results and 'CAx' in dict_results and \
                         'CAy' in dict_results and 'CAz' in dict_results:
-                        smoc_z = self.select_outliers_by_z(dict_results, -1.5)
+                        
+                        smoc_z, list_z_local, list_z_global = self.select_outliers_by_z(dict_results, -2.0)
+                        #print len(dict_results['smoc']),len(list_z_local)
                         for l in xrange(len(dict_results['residue'])):
+                            #write z scores
+                            residue_scores = ','.join([
+                                chain,str(dict_results['resname'][l])+
+                                '_'+str(dict_results['residue'][l]),
+                                "{0:.2f}".format(float(dict_results['smoc'][l])),
+                                "{0:.2f}".format(list_z_local[l]),
+                                "{0:.2f}".format(list_z_global[l])
+                                               ])
+                            sco.write(residue_scores+"\n")
+                            
                             if smoc_z[l]:
-                                self.residue_outliers[pdbid][chain][str(dict_results['residue'][l])] = 'outlier'
+                                self.residue_outliers[pdbid][chain][str(dict_results['residue'][l])] = 'Outlier'
                                 self.residue_coordinates[pdbid][chain][str(dict_results['residue'][l])] = \
                                                                                 (dict_results['CAx'][l],
                                                                                  dict_results['CAy'][l],
@@ -278,6 +337,7 @@ class SetSMOCResults():
     #         pyrvapi.rvapi_set_plot_ymin('plot1', 'graphWidget1', 0.0)
         
         pyrvapi.rvapi_flush()
+        sco.close()
         #print self.dict_outlier_details
         
     def check_chain_outliers(self,chain):
@@ -403,9 +463,11 @@ class SetSMOCResults():
                     pyrvapi.rvapi_flush()
                 ct_plot += 1
 
-    def select_outliers_by_z(self,dict_results, z_cutoff = -1.5):
+    def select_outliers_by_z(self,dict_results, z_cutoff = -2.0):
         list_smoc = dict_results['smoc']
         array_smoc = np.array(list_smoc)
+        list_z_global = self.calc_z_median(list_smoc)
+        #print len(list_smoc),len(list_z_global)
         #if coords are available, set z score locally
         if 'CAx' in dict_results and 'CAy' in dict_results and \
             'CAz' in dict_results:
@@ -419,33 +481,67 @@ class SetSMOCResults():
                     gridtree = KDTree(indi)
                 except ImportError: 
                     gridtree = None
+            list_z_local = []
             if gridtree is not None:
                 list_z = []
                 for l in xrange(len(list_smoc)):
                     ca_coord = indi[l]
-                    list_neigh = self.get_indices_sphere(gridtree,ca_coord,dist=15.0)
-                    #print l, list_neigh
+                    val = list_smoc[l]
+                    list_neigh = self.get_indices_sphere(gridtree,ca_coord,dist=8.0)
                     list_smoc_local = array_smoc[list_neigh]
-                    z = self.calc_z(list_smoc_local)
-                    res_index = list_neigh.index(l)
-                    list_z.append(z[res_index] < z_cutoff)
-                    #print res_index
-                return list_z
+                    z_local = self.calc_z_median(list_smoc_local,val)
+                    #res_index = list_neigh.index(l)
+                    list_z_local.append(z_local)
+                    list_z.append(z_local < z_cutoff)
+                
         else:
-            z = self.calc_z(array_smoc)
-            return z < z_cutoff
+            list_z_local = [0.]*len(list_z_global) #when no local z is calculated
+            z = self.calc_z_median(array_smoc)
+            list_z = z < z_cutoff
+        #print len(list_z),len(list_z_local),len(list_z_global)
+        return list_z, list_z_local, list_z_global
     
-    def calc_z(self,list_vals):
-        sum_smoc = np.sum(list_vals)
-        mean_smoc = sum_smoc/len(list_vals)
-        median_smoc = np.median(list_vals)
-        std_smoc = np.std(list_vals)
-        if std_smoc == 0.:
-            z = np.zeros(len(list_vals))
-            return z
-        z = (list_vals - mean_smoc)/std_smoc
+    def calc_z(self,list_vals,val=None):
+        if val is not None:
+            try:
+                list_vals.remove(val) #remove val from list
+            except: pass
+        sum_mapval = np.sum(list_vals)
+        mean_mapval = sum_mapval/len(list_vals)
+        #median_mapval = np.median(list_vals)
+        std_mapval = np.std(list_vals)
+        if val is None:
+            if std_mapval == 0.:
+                z = [0.0]*len(list_vals)
+            else:
+                z = np.round((np.array(list_vals) - mean_mapval)/std_mapval,2)
+        else:
+            if std_mapval == 0.:
+                z = 0.0
+            else:
+                z = (val - mean_mapval)/std_mapval
         return z
-    
+     
+    def calc_z_median(self,list_vals,val=None):
+        if val is not None:
+            try:
+                list_vals.remove(val)
+            except: pass
+        sum_smoc = np.sum(list_vals)
+        median_smoc = np.median(list_vals)
+        mad_smoc = np.median(np.absolute(np.array(list_vals)-median_smoc))
+        if val is None:
+            if mad_smoc == 0.:
+                z = [0.0]*len(list_vals)
+            else:
+                z = np.around((np.array(list_vals) - median_smoc)/mad_smoc,2)
+        else:
+            if mad_smoc == 0.:
+                z = 0.0
+            else:
+                z = round(((val - median_smoc)/mad_smoc),2)
+        return z    
+
     def get_indices_sphere(self,gridtree,coord,dist=5.0):
         list_points = gridtree.query_ball_point(\
                     [coord[0],coord[1],coord[2]], 
