@@ -26,8 +26,10 @@ from smoc.set_smoc_results import SetSMOCResults, SetSCCCResults
 from molprobity.set_molprobity_results import SetMolprobityResults
 from cablam.set_cablam_results import SetCaBLAMResults
 from refmac.set_refmac_results import SetRefmacResults
+from global_scores.set_scores_results import SetScoresResults
 from dssp.dssp_log_parser import DSSP_LogParser
 from jpred.set_jpred_results import SetJpredResults
+from bfactor_analysis.analyse_bfactors_results import SetBfactResults
 from cootscript_edit import SetCootScript
 import json
 import gc
@@ -76,9 +78,11 @@ class PipelineResultsViewer(object):
                                     None, None, None, None)
         pyrvapi.rvapi_flush()
         #lists of job processes
+        self.bfactor_process = []
         self.molprobity_process = []
         self.cablam_process = []
         self.refmac_process = []
+        self.scores_process = []
         self.smoc_process = []
         self.sccc_process = []
         self.dssp_process = []
@@ -97,7 +101,9 @@ class PipelineResultsViewer(object):
         self.residue_outliers = OrderedDict()
         #dictionary with residue coordinates for each model
         self.residue_coordinates = {}
-         
+        #output report for global results
+        gr = open('global_report.txt','w')
+            
         self.list_failed_jobs = []
         self.list_finished_jobs = []
         self.list_complete = []
@@ -131,28 +137,45 @@ class PipelineResultsViewer(object):
                     else: continue
                     time.sleep(2)
                     #print job.name
-                    
+                    if 'Bfactor' in job.name and \
+                        'Bfactor' not in self.list_complete:
+                        self.bfactor_process.append(job)
+                        # all jobs of this type are completed?
+                        flag_complete = self.check_job_set_complete('Bfactor',
+                                                    dict_num_jobs['Bfactor'])
+                        if flag_complete == 1:
+                            if flag_add_tabs:
+                                #add tabs
+                                self.add_global_local_tabs()
+                                flag_add_tabs = False
+                            bfa = SetBfactResults(self.bfactor_process,
+                                                      self.list_chains,
+                                                      cootset_instance=cs,
+                                                      glob_reportfile=gr)
+                            self.update_list_chains(bfa.list_chains)
+                            gc.collect()
                     if 'Molprobity' in job.name and \
                             'Molprobity' not in self.list_complete:
-                            self.molprobity_process.append(job)
-                            # all jobs of this type are completed?
-                            flag_complete = self.check_job_set_complete('Molprobity',
+                        self.molprobity_process.append(job)
+                        # all jobs of this type are completed?
+                        flag_complete = self.check_job_set_complete('Molprobity',
                                                         dict_num_jobs['Molprobity'])
-                            if flag_complete == 1:
-                                if flag_add_tabs:
-                                    #add tabs
-                                    self.add_global_local_tabs()
-                                    flag_add_tabs = False
-                                mp = SetMolprobityResults(self.molprobity_process,
-                                                          self.list_chains,
-                                                          cootset_instance=cs)
-                                self.update_list_chains(mp.list_chains)
-                                if len(mp.residue_outliers) != 0: 
-                                    self.list_summary_methods.append('molprobity')
-                                    self.residue_outliers['molprobity'] = mp.residue_outliers
-                                    self.update_coordinates(mp.residue_coordinates)
-                                #collect non-ref/deleted items from the object
-                                gc.collect()
+                        if flag_complete == 1:
+                            if flag_add_tabs:
+                                #add tabs
+                                self.add_global_local_tabs()
+                                flag_add_tabs = False
+                            mp = SetMolprobityResults(self.molprobity_process,
+                                                      self.list_chains,
+                                                      cootset_instance=cs,
+                                                      glob_reportfile=gr)
+                            self.update_list_chains(mp.list_chains)
+                            if len(mp.residue_outliers) != 0: 
+                                self.list_summary_methods.append('molprobity')
+                                self.residue_outliers['molprobity'] = mp.residue_outliers
+                                self.update_coordinates(mp.residue_coordinates)
+                            #collect non-ref/deleted items from the object
+                            gc.collect()
              
                     if 'Refmac' in job.name and \
                         'Refmac' not in self.list_complete:
@@ -164,9 +187,27 @@ class PipelineResultsViewer(object):
                                 #add tabs
                                 self.add_global_local_tabs()
                                 flag_add_tabs = False
-                            SetRefmacResults(self.refmac_process)
+                            SetRefmacResults(self.refmac_process,
+                                             glob_reportfile=gr)
     #                 if 'Refmac refine (local)' in job.name:
     #                     self.refmac_process.append(job)
+                    if 'TEMPy scores' in job.name and \
+                        'TEMPy scores' not in self.list_complete:
+                        #print job.name, status
+                        self.scores_process.append(job)
+                        flag_complete = self.check_job_set_complete('TEMPy scores',
+                                                        dict_num_jobs['TEMPy scores'])
+                        if flag_complete == 1:
+                            if flag_add_tabs:
+                                #add tabs
+                                self.add_global_local_tabs()
+                                flag_add_tabs = False
+                            tsc = SetScoresResults(self.scores_process, 
+                                                self.list_chains,
+                                                glob_reportfile=gr)
+                                 
+                            gc.collect()
+                    
                     if 'SMOC' in job.name and \
                         'SMOC' not in self.list_complete:
                         #print job.name, status
@@ -179,8 +220,10 @@ class PipelineResultsViewer(object):
                                 self.add_global_local_tabs()
                                 flag_add_tabs = False
                             sm = SetSMOCResults(self.smoc_process, 
-                                                self.list_chains, 
-                                                cootset_instance=cs)
+                                                self.list_chains,
+                                                pipeline_location=self.pipeline.location,
+                                                cootset_instance=cs,
+                                                glob_reportfile=gr)
                             if len(sm.residue_outliers) != 0: 
                                 self.list_summary_methods.append('smoc')
                                 self.residue_outliers['smoc'] = sm.residue_outliers
@@ -209,7 +252,8 @@ class PipelineResultsViewer(object):
                             cm = SetCaBLAMResults(self.cablam_process,
                                                   list_chains=self.list_chains,
                                                   dssp_dict=dssp_dict,
-                                                    cootset_instance=cs)
+                                                    cootset_instance=cs,
+                                                    glob_reportfile=gr)
              
                             self.update_list_chains(cm.list_chains)
                              
@@ -250,7 +294,9 @@ class PipelineResultsViewer(object):
                                 self.update_coordinates(jp.residue_coordinates)
                             gc.collect()
                     pyrvapi.rvapi_flush()
-                    
+        gr.flush()
+        gr.close()
+        
         if len(self.list_finished_jobs) == 0 or len(self.list_complete) == 0:
             pyrvapi.rvapi_add_tab( global_tab, "", True)
             pyrvapi.rvapi_add_section("results_section","",global_tab,0,0,1,1,True )
@@ -293,10 +339,18 @@ class PipelineResultsViewer(object):
                     try: dict_num_jobs['Molprobity'] += 1
                     except KeyError:
                         dict_num_jobs['Molprobity'] = 1
+                elif 'Bfactor' in job.name:
+                    try: dict_num_jobs['Bfactor'] += 1
+                    except KeyError:
+                        dict_num_jobs['Bfactor'] = 1
                 elif 'Refmac' in job.name:
                     try: dict_num_jobs['Refmac'] += 1
                     except KeyError:
                         dict_num_jobs['Refmac'] = 1
+                elif 'TEMPy scores' in job.name:
+                    try: dict_num_jobs['TEMPy scores'] += 1
+                    except KeyError:
+                        dict_num_jobs['TEMPy scores'] = 1
                 elif 'SMOC' in job.name:
                     try: dict_num_jobs['SMOC'] += 1
                     except KeyError:
@@ -383,7 +437,7 @@ class PipelineResultsViewer(object):
                     if not pdbid in dict_model_outlier: 
                         dict_model_outlier[pdbid] = {}
                     if not pdbid in self.dict_cluster_number:
-                        self.dict_cluster_number[pdbid] = 1
+                        self.dict_cluster_number[pdbid] = [1,1]
                     #add outlier set to dict_model_outlier
                     self.add_new_residue_outlier_set(self.residue_outliers[met][pdbid][chain],
                                                  dict_model_outlier[pdbid],pdbid,
@@ -394,6 +448,7 @@ class PipelineResultsViewer(object):
                         
             if chain_table_flag:
                 self.set_summary_tables(dict_model_outlier,chain)
+                self.dict_cluster_number[pdbid][1] = 1
                 self.set_coot_dict_clusters(dict_model_outlier, chain)
         
     
@@ -487,9 +542,49 @@ class PipelineResultsViewer(object):
             #add headers and data
             add_horz_headers_to_table(table_id,list_horz_headers,list_horz_headers_tips)
             add_data_to_table(table_id,data_array)
-                
+            textfile = os.path.join(self.pipeline.location,
+                                pdbid+'_'+chain+'_cluster_outliers.csv')
+            self.add_data_to_textfile(textfile,list_horz_headers,data_array)
             pyrvapi.rvapi_flush()
-            
+    
+    def add_data_to_textfile(self,textfile,list_horz_headers,data_array):
+        with open(textfile,'w') as c:
+            c.write('#Molprobity: geometry outliers reported by molprobity.\n'
+                    '#CaBLAM: Backbone geometry outliers reported by molprobity.\n'
+                    '#SMOC: local fit to map outliers reported by TEMPy SMOC.\n'
+                    '#Jpred: Secondary structure prediction from sequence inconsistent with model.\n'
+                    )
+            list_header = []
+            ct_str = 0
+            for header in list_horz_headers:
+                formatted_header = ' '.join(header.split('<br>'))
+                formatted_header = '_'.join(formatted_header.split())
+                if ct_str < 2:
+                    list_header.append('{}'.format(formatted_header))
+                else:
+                    list_header.append('{}'.format(formatted_header))
+                ct_str += 1
+            c.write(','.join(list_header)+'\n')
+            len_prev = 0
+            for row in data_array:
+                list_outl = []
+                ct_str = 0
+                if len_prev != 0 and len_prev - len(row) == 1:#cluster number missing
+                    list_outl.append('{}'.format(prev_row[0]))
+                    ct_str += 1
+                
+                for outl in row:
+                    formatted_outl = '*'.join(outl.split('<br>'))
+                    if ct_str < 2:
+                        list_outl.append('{}'.format(formatted_outl))
+                    else:
+                        list_outl.append('{}'.format(formatted_outl))
+                    ct_str += 1
+                c.write(','.join(list_outl)+'\n')
+                if len_prev <= len(row):
+                    len_prev = len(row)
+                    prev_row = row
+                
     def set_df_csv(self,dict_data,data_id):
         df = pd.DataFrame.from_dict(dict_data, orient='index')
         #df1 = df.replace(np.nan,' ')
@@ -519,13 +614,16 @@ class PipelineResultsViewer(object):
                 self.residue_coordinates[pdbid] = dict_coordinates[pdbid]
             else:
                 for chain in dict_coordinates[pdbid]:
-                    if not chain in dict_coordinates[pdbid]:
+                    if not chain in self.residue_coordinates[pdbid]:
                         self.residue_coordinates[pdbid][chain] = dict_coordinates[pdbid][chain]
                     else:
                         for resnum in dict_coordinates[pdbid][chain]:
                             res_coordinate = dict_coordinates[pdbid][chain][resnum]
                             if len(res_coordinate) != 3: continue
-                            self.residue_coordinates[pdbid][chain][resnum] = res_coordinate
+                            if not resnum in self.residue_coordinates[pdbid][chain]:
+                                self.residue_coordinates[pdbid][chain][resnum] = res_coordinate
+                            elif len(self.residue_coordinates[pdbid][chain][resnum]) != 3:
+                                self.residue_coordinates[pdbid][chain][resnum] = res_coordinate
     
     def cluster_outliers(self):
         '''
@@ -563,8 +661,9 @@ class PipelineResultsViewer(object):
                     if str(res) in list_res: continue
                     list_coord.append(self.residue_coordinates[pdbid][chain][res])
                     list_res.append(str(res)) #resid chain
-                list_res_clusters = self.cl_out.get_cluster_outliers(list_res, list_coord)
-                list_res_clusters.sort(key=len,reverse=True)
+                if len(list_coord) > 0:
+                    list_res_clusters = self.cl_out.get_cluster_outliers(list_res, list_coord)
+                    list_res_clusters.sort(key=len,reverse=True)
                 
                 self.dict_residue_clusters[pdbid][chain] = copy.deepcopy(list_res_clusters)
     
@@ -581,7 +680,7 @@ class PipelineResultsViewer(object):
             try: list_clusters = self.dict_residue_clusters[pdbid][chain]
             except KeyError: continue
 #             try:
-#                 cl_number = self.dict_cluster_number[pdbid]
+#                 cl_number = self.dict_cluster_number[pdbid][0]
 #             except KeyError: continue
             for l in xrange(len(list_clusters)):
                 if len(list_clusters[l]) == 0: continue
@@ -627,11 +726,13 @@ class PipelineResultsViewer(object):
                         flag_cluster = True
                         try:
                             self.dict_coot_outlier_clusters[pdbid]['clusters'].append(
-                            (chain,res,self.dict_cluster_number[pdbid],outlier_types_string,residue_coordinate))
+                            (chain,res,self.dict_cluster_number[pdbid][1],outlier_types_string,residue_coordinate))
                         except KeyError:
                             self.dict_coot_outlier_clusters[pdbid]['clusters'] = \
-                            [(chain,res,self.dict_cluster_number[pdbid],outlier_types_string,residue_coordinate)]
-                if flag_cluster: self.dict_cluster_number[pdbid] += 1
+                            [(chain,res,self.dict_cluster_number[pdbid][1],outlier_types_string,residue_coordinate)]
+                if flag_cluster: 
+                    self.dict_cluster_number[pdbid][0] += 1 #model
+                    self.dict_cluster_number[pdbid][1] += 1 #chain
 
     def add_cootdata(self):
         '''
@@ -702,7 +803,7 @@ class PipelineResultsViewer(object):
         '''
         pyrvapi.rvapi_add_section1(chain+'/'+summary_table_subsection, 
                                'Summary table', 
-                               0, 0, 1, 1, False)
+                               0, 0, 1, 1, True)
         
     def get_gemmi_model(self,structure_filename):
         '''
