@@ -10,7 +10,8 @@
 
 import os
 import json
-import math
+import math, glob
+import numpy as np
 import mrcfile
 try:
     import modeller  # @UnusedImport
@@ -63,7 +64,15 @@ class FlexEM(task_utils.CCPEMTask):
             30:B 40:B 60:B 70:B<br>
             indicates two rigid bodies, one formed of residues 10 
             to 20 from chain A, and another formed of residues 30 to 40 
-            and 60 to 70 of chain B.
+            and 60 to 70 of chain B.<br><br>
+            
+            Keywords can be used to add specific restraints:<br>
+            To add helix restraint between residue 10 to 20 of chain A
+                use keyword: <br>
+                helix 10:A 20:A<br><br>
+            To add a distance restraint (say 5 Angstroms) between residue 35 and 40
+                use keyword: <br>
+                distance 35:A 40:A 5.0<br>
             '''),
         short_description=(
             'Fitting and refinement of atomic structures guided by cryoEM density'),
@@ -162,10 +171,11 @@ class FlexEM(task_utils.CCPEMTask):
         iterations.add_argument(
             '-iterations',
             '--iterations',
-            help='Number of iterations for MD or number runs for CG',
+            help='''Number of Flex-EM iterations 
+                (dependant on the extent of conformational difference)''',
             metavar='Iterations',
             type=int,
-            default=10)
+            default=3)
         #
         ribfind_cutoff = parser.add_argument_group()
         ribfind_cutoff.add_argument(
@@ -182,8 +192,9 @@ class FlexEM(task_utils.CCPEMTask):
         create_rigid_body.add_argument(
             '-create_rigid_body',
             '--create_rigid_body',
-            help=('Generate rigid bodies based on '
-                  'secondary structure elements'),
+            help=('''Generate rigid bodies based on resolution 
+                    (e.g. individual secondary structures for 
+                    resolutions better than 7.5)'''),
             metavar='Auto rigid body',
             type=bool,
             default=True)
@@ -202,21 +213,54 @@ class FlexEM(task_utils.CCPEMTask):
             '-max_atom_disp',
             '--max_atom_disp',
             help=('Maximum atom displacement cutoff recommended to use 0.39 for '
-                  'large rigid bodies, 0.15 for SSE refinement, 0.10 for '
+                  'large rigid bodies, 0.2 for SSE refinement, 0.10 for '
                   'all-atom refinement'),
             metavar='Atom disp',
             type=float,
-            default=0.39)
+            default=0.20)
         #
         phi_psi_restraints = parser.add_argument_group()
         phi_psi_restraints.add_argument(
             '-phi_psi_restraints',
             '--phi_psi_restraints',
             help=('Restrain to starting model phi-psi (see homology '
-                  'derived restraints in Modeller'),
-            metavar='Self restrain',
+                  'derived restraints in Modeller)'),
+            metavar='Template phi/psi restraint',
             type=bool,
             default=True)
+        distance_restraints = parser.add_argument_group()
+        distance_restraints.add_argument(
+            '-distance_restraints',
+            '--distance_restraints',
+            help=('Locally restrain to starting model distances (see homology '
+                  'derived restraints in Modeller)'),
+            metavar='Template distance restraint',
+            type=bool,
+            default=False)
+        
+        distance_restraints.add_argument(
+            '-max_distance',
+            '--max_distance',
+            help=('Maximum distance to apply template restraints (see homology '
+                  'derived restraints in Modeller)'),
+            metavar='maximum distance for restraint',
+            type=float,
+            default=5.0)
+        
+        parser.add_argument(
+            '-density_weight',
+            '--density_weight',
+            help=('Factor to scale density with [0.0-1.0]'),
+            metavar='density weight',
+            type=float,
+            default=1.0)
+        #
+        parser.add_argument(
+            '-keywords',
+            '--keywords',
+            help='Keywords for advanced options',
+            type=str,
+            default='')
         #
         test_mode = parser.add_argument_group()
         test_mode.add_argument(
@@ -301,16 +345,47 @@ class FlexEM(task_utils.CCPEMTask):
             print warnings
         return args_correct
 
+    def get_keywords(self):
+        self.dict_keywords = {}
+        keyword_text = self.args.keywords.value
+        if len(keyword_text) > 0:
+            keyword_lines = keyword_text.split('\n')
+            for line in keyword_lines:
+                line_split = line.split()
+                if len(line_split) < 3: continue
+                if line_split[0] == 'helix':
+                    try:
+                        self.dict_keywords['helix'].append([line_split[1],line_split[2]])
+                    except KeyError: 
+                        self.dict_keywords['helix'] = [[line_split[1],line_split[2]]]
+                elif line_split[0] == 'strand':
+                    try:
+                        self.dict_keywords['strand'].append([line_split[1],line_split[2]])
+                    except KeyError: 
+                        self.dict_keywords['strand'] = [[line_split[1],line_split[2]]]
+                elif line_split[0] == 'distance':
+                    try:
+                        res_distance = float(line_split[3])
+                        try:
+                            self.dict_keywords['distance'].append([line_split[1],line_split[2],res_distance])
+                        except KeyError: 
+                            self.dict_keywords['distance'] = [[line_split[1],line_split[2],res_distance]]
+                    except ValueError: pass
+        
+        with open(self.keyword_jsonfile,'w') as kj:
+            json.dump(self.dict_keywords,kj)
+        
+        
     def flexem_job(self):
         #
         flex_em_json_args = {
             'optimization': self.args.mode.value,
             'input_pdb_file': self.args.input_pdb.value,
-            'code': 'xxxx',
+            'code': 'fit',
             'em_map_file': self.args.input_map.value,
-            'map_format': 'mrc',
+            'map_format': 'MRC',
             'apix': self.map_data.voxel_size.z.item(),#voxx_voxy_voxz[2],  # Voxel size
-            'resolution': 10,  # Self.args.map_resolution.value,
+            'resolution': self.args.map_resolution.value,
             'x': self.map_data.header.origin.x.item(),
             'y': self.map_data.header.origin.y.item(),
             'z': self.map_data.header.origin.z.item(),
@@ -321,6 +396,10 @@ class FlexEM(task_utils.CCPEMTask):
             'job_location': self.job_location,
             'max_atom_disp': self.args.max_atom_disp.value,
             'phi_psi_restraints': self.args.phi_psi_restraints.value,
+            'distance_restraints':  self.args.distance_restraints.value,
+            'max_distance': self.args.max_distance.value,
+            'keyword_json': self.keyword_jsonfile,
+            'density_weight': self.args.density_weight.value,
             'test_mode': self.args.test_mode.value}
         jsonpath = os.path.join(self.job_data, 'flex_em_args.json')
         json.dump(flex_em_json_args,
@@ -392,19 +471,58 @@ class FlexEM(task_utils.CCPEMTask):
             pl = [[self.ribfind_process]]
 
             # Set rigid body path
-            # Cutoff to nearest 10
-            cutoff = int(math.ceil(self.args.ribfind_cutoff() / 10.0)) * 10
+            pdb = self.args.input_pdb.value
+            pdb_outdir = '.'.join(os.path.basename(pdb).split('.')[:-1])
+            rigid_body_path = \
+                os.path.join(self.ribfind_process.location,
+                             pdb_outdir,'protein')
+                
+            if self.args.ribfind_cutoff() is None:
+                if self.args.map_resolution.value < 7.5:
+                    cutoff = 100
+                elif self.args.map_resolution.value < 15.0:
+                    cutoff = 60
+                elif self.args.map_resolution.value < 100.0:
+                    cutoff = 30
+            else:
+                cutoff = int(math.ceil(self.args.ribfind_cutoff() / 10.0)) * 10
+            
             self.args.ribfind_cutoff.value = cutoff
-            self.args.input_rigid.value = os.path.join(
-                self.ribfind_process.location,
-                'rigid_body_{0:0>3}.txt'.format(
-                    self.args.ribfind_cutoff.value))
+            list_cutoffs = []
+            rigid_body_files = []
+            for rigidclust in xrange(0,101,10):
+                rigid_body_files.append(os.path.join(rigid_body_path,
+                    pdb_outdir+'_denclust_{}.txt'.format(rigidclust)))
+            
+            for rf in rigid_body_files:
+                basename = '.'.join(os.path.basename(rf).split('.')[:-1])
+                basename = basename.split('_')[-1]
+                try:
+                    list_cutoffs.append(int(basename))
+                except TypeError: pass
+            list_cutoffs.sort()
+            cutoff_array = np.array(list_cutoffs)
+            sel_indices = np.searchsorted(cutoff_array,cutoff) 
+            if isinstance(sel_indices,int):
+                sel_cutoff = ''
+                if sel_indices >= len(cutoff_array):
+                    sel_indices = -1
+                sel_cutoff = cutoff_array[sel_indices]
+            else: 
+                try: sel_cutoff = cutoff_array[sel_indices[0]]
+                except IndexError: sel_cutoff = cutoff_array[-1]
+            self.args.input_rigid.value = \
+                os.path.join(rigid_body_path,
+                             pdb_outdir+'_denclust_{}.txt'.format(sel_cutoff))
 
         # Directory to store intermediate data
         self.job_data = os.path.join(self.job_location, 'job_data')
         ccpem_utils.check_directory_and_make(self.job_data)
-
+        
         # FlexEM
+        self.keyword_jsonfile = os.path.join(self.job_data,
+                                         'keywords.json')
+        self.get_keywords()
         self.flexem_job()
         if pl is None:
             pl = [[self.flexem_process]]
