@@ -19,6 +19,10 @@ from ccpem_core.tasks.refmac import refmac_task
 from ccpem_core import settings
 from ccpem_core.tasks.bin_wrappers import mapmask
 from ccpem_core.tasks.bin_wrappers import fft
+from ccpem_core.model_tools.gemmi_utils import shift_coordinates, remove_atomic_charges
+from ccpem_core.map_tools.TEMPy import map_preprocess
+from ccpem_core.tasks.model2map.model2map_task import mapprocess_wrapper, \
+                sf2mapWrapper, PDBSetCell
 
 
 mrc_axis = {1: 'x',
@@ -44,7 +48,10 @@ class LocScale(task_utils.CCPEMTask):
         'loc_scale_python': ['ccpem-python',
                              os.path.realpath(np_locscale_fft.__file__)],
         'refmac': settings.which(program='refmac5'),
-        'pdbset':  settings.which(program='pdbset')}
+        'pdbset':  settings.which(program='pdbset'),
+        'ccpem-gemmi':settings.which(program='gemmi'),
+        'ccpem-mapprocess':
+        ['ccpem-python', os.path.realpath(map_preprocess.__file__)]}
 
     def __init__(self,
                  database_path=None,
@@ -177,8 +184,13 @@ class LocScale(task_utils.CCPEMTask):
             default=2)
         #
         return parser
-
+    
+    def fix_model_for_refmac(self,pdb_path,pdb_edit_path,trans_vector,
+                             remove_charges=False):
+        shift_coordinates(pdb_path,pdb_edit_path,trans_vector,remove_charges)
+        
     def run_pipeline(self, job_id=None, db_inject=None):
+        fix_map_model = True
         # Calculate pixel size
         with mrcfile.open(self.args.target_map(), 'r') as mrc:
             map_nx = mrc.header.nx
@@ -187,11 +199,21 @@ class LocScale(task_utils.CCPEMTask):
             map_mapc = mrc.header.mapc
             map_mapr = mrc.header.mapr
             map_maps = mrc.header.maps
+            map_nxstart = mrc.header.nxstart
+            map_nystart = mrc.header.nystart
+            map_nzstart = mrc.header.nzstart
+            ori_x = mrc.header.origin.x
+            ori_y = mrc.header.origin.y
+            ori_z = mrc.header.origin.z
+            cella = (mrc.header.cella.x,mrc.header.cella.y,
+                     mrc.header.cella.z)
+            apix = mrc.voxel_size.item()
 
             if self.args.pixel_size() is None:
                 self.args.pixel_size.value = (mrc.voxel_size.x + 
                                               mrc.voxel_size.y + 
                                               mrc.voxel_size.z) / 3.0
+        self.map_edit_path = self.args.target_map.value
         
 #         # Run Refmac to refine reference structure B-factors
 #         if self.args.refine_bfactors():
@@ -211,19 +233,55 @@ class LocScale(task_utils.CCPEMTask):
 #             loc_scale_structure = ''
 #         else:
 #             loc_scale_structure = self.args.reference_model
+
+        
         pl = []
         if self.args.reference_map() is None:
             self.reference_map = os.path.join(
                 self.job_location,
                 'model_reference.mrc')
-
+            #fix map and model when nstart is non-zero
+            if ori_x == 0. and ori_y == 0. and ori_z == 0.:
+                if map_nxstart != 0 or map_nystart != 0 or map_nzstart != 0:
+                    map_edit = os.path.splitext(
+                        os.path.basename(self.args.target_map.value))[0]+'_fix.mrc'
+                    self.map_edit_path = os.path.join(
+                                    self.job_location,map_edit)
+                    # fix origin to agree with nstart
+                    shutil.copyfile(self.args.target_map.value,self.map_edit_path)
+                    assert os.path.isfile(self.map_edit_path)
+                    with mrcfile.open(self.map_edit_path,'r+') as mrc:
+                        mrc.header.origin.x = map_nxstart*apix[0]
+                        mrc.header.origin.y = map_nystart*apix[1]
+                        mrc.header.origin.z = map_nzstart*apix[2]
+                    ori_x = map_nxstart*apix[0]
+                    ori_y = map_nystart*apix[1]
+                    ori_z = map_nzstart*apix[2]
+                
+            ##self.pdb_edit_path = self.args.pdb_path.value
+            self.pdb_edit_path = os.path.join(self.job_location,
+                                    os.path.splitext(os.path.basename(
+                                        self.args.reference_model.value))[0]+'_fix.pdb')
+            #translate model to origin 0 if nstart is 0
+            if fix_map_model and not all(o == 0. for o in (ori_x,ori_y,ori_z)):
+                trans_vector = [-ori_x,
+                                -ori_y,
+                                -ori_z]
+                self.fix_model_for_refmac(self.args.reference_model.value,
+                                          self.pdb_edit_path,trans_vector,
+                                          remove_charges=True)
+            else:
+                #remove charges
+                remove_atomic_charges(self.args.reference_model.value,
+                                          self.pdb_edit_path)
+                
             # Set PDB cryst from mtz
-            self.process_pdb_set = refmac_task.PDBSetCell(
+            self.process_pdb_set = PDBSetCell(
                 command=self.commands['pdbset'],
                 job_location=self.job_location,
                 name='Set PDB cell',
-                pdb_path=self.args.reference_model(),
-                map_path=self.args.target_map())
+                pdb_path=self.pdb_edit_path,
+                map_path=self.map_edit_path)
             pl = [[self.process_pdb_set.process]]
 
             self.pdb_set_path = os.path.join(
@@ -233,6 +291,7 @@ class LocScale(task_utils.CCPEMTask):
             # Calculate mtz from pdb 
             self.refmac_sfcalc_crd_process = refmac_task.RefmacSfcalcCrd(
                 job_location=self.job_location,
+                name='SF calc from coord',
                 pdb_path=self.pdb_set_path,
                 lib_in=self.args.lib_in(),
                 resolution=self.args.resolution())
@@ -241,23 +300,68 @@ class LocScale(task_utils.CCPEMTask):
                 'sfcalc_from_crd.mtz')
             pl.append([self.refmac_sfcalc_crd_process.process])
 
-            # Convert calc mtz to map
-            self.fft_process = fft.FFT(
+            #gemmi sf2map
+            self.sf2map_process = sf2mapWrapper(
                 job_location=self.job_location,
+                name='SF to map',
+                command=self.commands['ccpem-gemmi'],
                 map_nx=map_nx,
                 map_ny=map_ny,
                 map_nz=map_nz,
                 # For now only use x,y,z
-#                 fast=mrc_axis[int(map_mapc)],
-#                 medium=mrc_axis[int(map_mapr)],
-#                 slow=mrc_axis[int(map_maps)],
+    #                 fast=mrc_axis[int(map_mapc)],
+    #                 medium=mrc_axis[int(map_mapr)],
+    #                 slow=mrc_axis[int(map_maps)],
                 fast='X',
                 medium='Y',
                 slow='Z',
                 mtz_path=self.refmac_sfcalc_mtz,
                 map_path=self.reference_map)
-            pl.append([self.fft_process.process])
-
+            pl.append([self.sf2map_process.process])
+#             # Convert calc mtz to map
+#             self.fft_process = fft.FFT(
+#                 job_location=self.job_location,
+#                 map_nx=map_nx,
+#                 map_ny=map_ny,
+#                 map_nz=map_nz,
+#                 # For now only use x,y,z
+# #                 fast=mrc_axis[int(map_mapc)],
+# #                 medium=mrc_axis[int(map_mapr)],
+# #                 slow=mrc_axis[int(map_maps)],
+#                 fast='X',
+#                 medium='Y',
+#                 slow='Z',
+#                 mtz_path=self.refmac_sfcalc_mtz,
+#                 map_path=self.reference_map)
+#             pl.append([self.fft_process.process])
+            #set origin and remove background
+            self.reference_map_processed = os.path.join(
+                self.job_location,
+                'model_reference_syn.mrc')
+            if not all(o == 0. for o in (ori_x,ori_y,ori_z)):
+                self.mapprocess_wrapper = mapprocess_wrapper(
+                                job_location=self.job_location,
+                                command=self.commands['ccpem-mapprocess'],
+                                map_path=self.reference_map,
+                                list_process=['shift_origin','threshold'],
+                                map_origin=(ori_x,ori_y,ori_z),
+                                map_contour=0.02,
+                                out_map=self.reference_map_processed
+                                )
+            else:
+                self.mapprocess_wrapper = mapprocess_wrapper(
+                                job_location=self.job_location,
+                                command=self.commands['ccpem-mapprocess'],
+                                map_path=self.reference_map,
+                                list_process=['threshold'],
+                                map_origin=(ori_x,ori_y,ori_z),
+                                map_contour=0.02,
+                                out_map=self.reference_map_processed
+                                )
+            pl.append([self.mapprocess_wrapper.process])
+            
+            self.reference_map = self.reference_map_processed
+        
         else:
             self.reference_map = self.args.reference_map()
             with mrcfile.open(self.reference_map, 'r') as mrc:
