@@ -15,6 +15,7 @@ from ccpem_core.tasks.refmac import refmac_task
 from ccpem_core.tasks.buccaneer import buccaneer_task
 from ccpem_core.tasks.nautilus import nautilus_results
 
+
 class Nautilus(task_utils.CCPEMTask):
     '''
     CCPEM Nautilus task.
@@ -35,11 +36,10 @@ class Nautilus(task_utils.CCPEMTask):
         references=None)
 
     commands = {'refmac': which(program='refmac5'),
-                'cnautilus':which('cnautilus'),
-                'sftools':which('sftools'),
-                'cad':which('cad'),
-                'freerflag':which('freerflag'),
-                'libg':which('libg')
+                'cnautilus': which('cnautilus'),
+                'sftools': which('sftools'),
+                'freerflag': which('freerflag'),
+                'libg': which('libg')
                 }
 
     def __init__(self,
@@ -52,9 +52,9 @@ class Nautilus(task_utils.CCPEMTask):
                  parent=None):
         # No longer need to set atomsf since refmac job defaults to use
         # "source EM MB" instead.
-#        self.atomsf = os.path.join(
-#            os.environ['CLIBD'],
-#            'atomsf_electron.lib')
+        #        self.atomsf = os.path.join(
+        #            os.environ['CLIBD'],
+        #            'atomsf_electron.lib')
 
         super(Nautilus, self).__init__(
             database_path=database_path,
@@ -147,6 +147,29 @@ class Nautilus(task_utils.CCPEMTask):
             type=float,
             default=0)
         #
+        parser.add_argument(
+            '-lib_in',
+            '--lib_in',
+            help='Library dictionary file for ligands (lib/cif format)',
+            metavar='Ligand dictionary',
+            type=str,
+            default=None)
+        # added
+        parser.add_argument(
+            '-local_refinement_on',
+            '--local_refinement_on',
+            help='Refine around radius of supplied molecule only',
+            metavar='Local refine',
+            type=bool,
+            default=False)
+        #
+        parser.add_argument(
+            '-mask_radius',
+            '--mask_radius',
+            help='Distance around molecule the map should be cut.',
+            type=float,
+            default=3.0)
+        #
         parser.add_argument(
             '-keywords',
             '--keywords',
@@ -177,12 +200,14 @@ class Nautilus(task_utils.CCPEMTask):
             if bfactor == 0:
                 label_out_suffix = '0'
             else:
-                label_out_suffix = 'Blur_{0:.2f}'.format(self.args.map_sharpen.value)
+                label_out_suffix = 'Blur_{0:.2f}'.format(
+                    self.args.map_sharpen.value)
         else:
             sharp_array = [-1 * bfactor]
             blur_array = None
-            label_out_suffix = 'Sharp_{0:.2f}'.format(self.args.map_sharpen.value)
-        
+            label_out_suffix = 'Sharp_{0:.2f}'.format(
+                -1 * self.args.map_sharpen.value)
+
         self.process_maptomtz = refmac_task.RefmacMapToMtz(
             command=self.commands['refmac'],
             resolution=self.args.resolution.value,
@@ -203,17 +228,6 @@ class Nautilus(task_utils.CCPEMTask):
             name='Set MTZ SigF and SG')
         pl.append([self.process_sftools_buccanneer.process])
 
-        # I find this redundant so can skip
-        # Set column labels with Cad
-        #self.process_cad_labels = buccaneer_task.CADBuccaneerLabels(
-        #    command=self.commands['cad'],
-        #    job_location=self.job_location,
-        #    resolution=self.args.resolution.value,
-        #    hklin=self.process_sftools_buccanneer.hklout,
-        #    sharpen_bfactor=self.args.map_sharpen.value,
-        #    name='Set MTZ labels')
-        #pl.append([self.process_cad_labels.process])
-
         # Create R free flags
         hklout = os.path.join(self.job_location,
                               'nautilus.mtz')
@@ -235,112 +249,102 @@ class Nautilus(task_utils.CCPEMTask):
             f.close()
             self.args.input_seq.value = f.name
 
-        # Run Nautilus - new pipeline = Nautilus->Refmac refine pipeline->repeat...
-        refine_keywords=add_refmac_keywords(self.args.refmac_keywords.value, 
-                                            label_out_suffix)
+        # Add optional refmac keywords
+        refine_keywords = ''
+        # make newligand noexit to prevent refmac from exiting if no ligand dictionary found,
+        # very rare unless is a new ligand
+        if self.args.lib_in.value:
+            refine_keywords += 'MAKE NEWLigand Noexit\n'
+        refine_keywords += add_refmac_keywords(self.args.refmac_keywords.value)
 
-        for i in range(1, (self.args.ncycle.value + 1)) :
+        # Run Nautilus pipeline
+        # Nautilus->LIBG->{1}->Refmac->repeat...{1}=maptomtz local
+        for i in range(1, (self.args.ncycle.value + 1)):
             # 1st cycle Nautilus
-            if i == 1 :
-                self.process_nautilus_pipeline = NautilusPipeline(
-                            command=self.commands['cnautilus'],
-                            job_location=self.job_location,
-                            hklin=self.process_free_r_flags.hklout,
-                            seqin=self.args.input_seq.value,
-                            label_out_suffix=label_out_suffix,
-                            ncycle=i,
-                            internal_cycle=self.args.ncycle_nau1st.value,
-                            resolution=self.args.resolution.value,
-                            pdbin=self.args.extend_pdb.value,
-                            pdbout=None,
-                            name='Nautilus build ' + str(i),
-                            job_title=self.args.job_title.value,
-                            keywords=self.args.keywords.value)
-                pl.append([self.process_nautilus_pipeline.process])
+            if i == 1:
+                naut_int_cycle = self.args.ncycle_nau1st.value
+                pdbin_path = self.args.extend_pdb.value
+            else:
+                naut_int_cycle = self.args.ncycle_naunth.value
+                pdbin_path = self.process_refine.pdbout_path
 
-                # Run libg for DNA/RNA restraints
-                self.process_libg = refmac_task.LibgRestraints(
-                    command=self.commands['libg'],
-                    job_location=self.job_location,
-                    name='DNA or RNA basepair restraints ' + str(i),
-                    pdb_path=self.process_nautilus_pipeline.pdbout)
-                pl.append([self.process_libg.process])
+            self.process_nautilus_pipeline = NautilusPipeline(
+                command=self.commands['cnautilus'],
+                job_location=self.job_location,
+                hklin=self.process_free_r_flags.hklout,
+                seqin=self.args.input_seq.value,
+                label_out_suffix=label_out_suffix,
+                ncycle=i,
+                internal_cycle=naut_int_cycle,
+                resolution=self.args.resolution.value,
+                pdbin=pdbin_path,
+                pdbout=None,
+                name='Nautilus build {0}'.format(str(i)),
+                job_title=self.args.job_title.value,
+                keywords=self.args.keywords.value)
+            pl.append([self.process_nautilus_pipeline.process])
 
-                # Run RefmacRefine (global)
-                # Invert sharpening value for refmac
-                if self.args.map_sharpen() is None:
-                    sharp = 0
-                else:
-                    sharp = -1.0*self.args.map_sharpen()
-                self.process_refine = refmac_task.RefmacRefine(
+            # Run libg for DNA/RNA restraints
+            self.process_libg = refmac_task.LibgRestraints(
+                command=self.commands['libg'],
+                job_location=self.job_location,
+                name='DNA or RNA basepair restraints {0}'.format(str(i)),
+                pdb_path=self.process_nautilus_pipeline.pdbout)
+            pl.append([self.process_libg.process])
+
+            # Invert sharpening value for refmac
+            if self.args.map_sharpen() is None:
+                sharp = 0
+            else:
+                sharp = -1.0 * self.args.map_sharpen()
+            # Run RefmacRefine (global/local)
+            if self.args.local_refinement_on():
+                self.process_maptomtz_local = refmac_task.RefmacMapToMtz(
                     command=self.commands['refmac'],
-                    job_location=self.job_location,
-                    pdb_path=self.process_nautilus_pipeline.pdbout,
-                    mtz_path=self.process_free_r_flags.hklout,
+                    name='Map to MTZ (local) {0}'.format(str(i)),
                     resolution=self.args.resolution.value,
-                    pdbout_path=os.path.join(self.job_location, 'refined' + str(i) + '.pdb'),
-                    mode='Global',
-                    name='Refmac refine (global) ' + str(i),
-                    sharp=sharp,
-                    ncycle=self.args.ncycle_refmac.value,
-                    output_hkl=True,
-                    symmetry_auto=False,
-                    keywords=refine_keywords,
-                    libg_restraints=True,
-                    libg_restraints_path=[self.process_libg.libg_restraints_path])
-                pl.append([self.process_refine.process])
-            else :
-                #nth cycle Nautilus
-                self.process_nautilus_pipeline = NautilusPipeline(
-                            command=self.commands['cnautilus'],
-                            job_location=self.job_location,
-                            hklin=self.process_free_r_flags.hklout,
-                            seqin=self.args.input_seq.value,
-                            label_out_suffix=label_out_suffix,
-                            ncycle=i,
-                            internal_cycle=self.args.ncycle_naunth.value,
-                            resolution=self.args.resolution.value,
-                            pdbin=self.process_refine.pdbout_path,
-                            pdbout=None,
-                            name='Nautilus build ' + str(i),
-                            job_title=self.args.job_title.value,
-                            keywords=self.args.keywords.value)
-                pl.append([self.process_nautilus_pipeline.process])
-
-                # Run libg for DNA/RNA restraints
-                self.process_libg = refmac_task.LibgRestraints(
-                    command=self.commands['libg'],
-                    job_location=self.job_location,
-                    name='DNA or RNA basepair restraints ' + str(i),
-                    pdb_path=self.process_nautilus_pipeline.pdbout)
-                pl.append([self.process_libg.process])
-
-                # Run RefmacRefine (global)
-                # Invert sharpening value for refmac
-                if self.args.map_sharpen() is None:
-                    sharp = 0
-                else:
-                    sharp = -1.0*self.args.map_sharpen()
-                self.process_refine = refmac_task.RefmacRefine(
-                    command=self.commands['refmac'],
+                    mode='Local',
                     job_location=self.job_location,
+                    lib_path=self.args.lib_in.value,
+                    map_path=self.args.input_map.value,
                     pdb_path=self.process_nautilus_pipeline.pdbout,
-                    mtz_path=self.process_free_r_flags.hklout,
-                    resolution=self.args.resolution.value,
-                    pdbout_path=os.path.join(self.job_location, 'refined' + str(i) + '.pdb'),
-                    mode='Global',
-                    name='Refmac refine (global) ' + str(i),
-                    sharp=sharp,
-                    ncycle=self.args.ncycle_refmac.value,
-                    output_hkl=True,
-                    symmetry_auto=False,
-                    keywords=refine_keywords,
-                    libg_restraints=True,
-                    libg_restraints_path=[self.process_libg.libg_restraints_path])
-                pl.append([self.process_refine.process])
+                    blur_array=blur_array,
+                    sharp_array=sharp_array,
+                    mrad=self.args.mask_radius.value)
+                pl.append([self.process_maptomtz_local.process])
+                # Refinement parameters: Local
+                mode = 'Local'
+                name = 'Refmac refine (local) {0}'.format(str(i))
+                pdb_path_a = self.process_maptomtz_local.pdbout_path
+                mtz_path_a = 'masked_fs.mtz'
+            else:
+                # Refinement parameters: Global
+                mode = 'Global'
+                name = 'Refmac refine (global) {0}'.format(str(i))
+                pdb_path_a = self.process_nautilus_pipeline.pdbout
+                mtz_path_a = self.process_free_r_flags.hklout
+
+            self.process_refine = refmac_task.RefmacRefine(
+                command=self.commands['refmac'],
+                job_location=self.job_location,
+                pdb_path=pdb_path_a,
+                mtz_path=mtz_path_a,
+                resolution=self.args.resolution.value,
+                pdbout_path=os.path.join(
+                    self.job_location, 'refined{0}.pdb'.format(str(i))),
+                mode=mode,
+                name=name,
+                sharp=sharp,
+                ncycle=self.args.ncycle_refmac.value,
+                output_hkl=True,
+                symmetry_auto=False,
+                keywords=refine_keywords,
+                libg_restraints=True,
+                libg_restraints_path=[self.process_libg.libg_restraints_path])
+            pl.append([self.process_refine.process])
 
         custom_finish = NautilusResultsOnFinish(
-            pipeline_path=self.job_location+'/task.ccpem',
+            pipeline_path=self.job_location + '/task.ccpem',
             refine_process=self.process_refine)
 
         if run:
@@ -358,29 +362,30 @@ class Nautilus(task_utils.CCPEMTask):
                 on_finish_custom=custom_finish)
             self.pipeline.start()
 
-
     def validate_args(self):
         # Now do at gui level
         return True
 
+
 class NautilusPipeline(object):
     '''
     Real time nucleic acid chain tracing (N.B. runs cnautilus)
     '''
+
     def __init__(self,
-            command,
-            job_location,
-            hklin,
-            seqin,
-            label_out_suffix,
-            ncycle=1,
-            internal_cycle=5,
-            resolution=2.0,
-            pdbin=None,
-            pdbout=None,
-            name=None,
-            job_title=None,
-            keywords=None):
+                 command,
+                 job_location,
+                 hklin,
+                 seqin,
+                 label_out_suffix,
+                 ncycle=1,
+                 internal_cycle=5,
+                 resolution=2.0,
+                 pdbin=None,
+                 pdbout=None,
+                 name=None,
+                 job_title=None,
+                 keywords=None):
         # counter for overall NautilusPipeline cycles, start with 1
         assert command is not None
         self.job_location = job_location
@@ -390,11 +395,12 @@ class NautilusPipeline(object):
         self.label_out_suffix = label_out_suffix
         self.pdbout = pdbout
         self.ncycle = ncycle
-        self.internal_cycle=internal_cycle
+        self.internal_cycle = internal_cycle
         self.resolution = resolution
         self.pdbin = pdbin
         if self.pdbout is None:
-            self.pdbout = os.path.join(job_location, 'build' + str(ncycle) + '.pdb')
+            self.pdbout = os.path.join(
+                job_location, 'build' + str(ncycle) + '.pdb')
         self.name = name
         if self.name is None:
             self.name = self.__class__.__name__
@@ -407,24 +413,24 @@ class NautilusPipeline(object):
         self.set_stdin()
         #
         self.process = process_manager.CCPEMProcess(
-                name=self.name,
-                command=command,
-                args=self.args,
-                location=self.job_location,
-                stdin=self.stdin)
+            name=self.name,
+            command=command,
+            args=self.args,
+            location=self.job_location,
+            stdin=self.stdin)
 
     def set_args(self):
         self.args = ['-stdin']
 
     def set_stdin(self):
         pdbin_ref = os.path.join(os.environ['CLIBD'],
-            'nautilus_lib.pdb')
-        #if self.ncycle == 1:
+                                 'nautilus_lib.pdb')
+        # if self.ncycle == 1:
         #    self.stdin_extra = '''colin-phifom PHIem, FOMem
         #    '''
-        #else:
+        # else:
         #    self.stdin_extra = '''colin-hl HLACOMB, HLBCOMB, HLCCOMB, HLDCOMB
-#colin-fc FWT, PHWT
+# colin-fc FWT, PHWT
 #'''
 
         self.stdin = '''
@@ -440,16 +446,15 @@ cycles {6}
 resolution {7}
 anisotropy-correction
 xmlout program{8}.xml
-'''.format(
-    self.job_title,
-    pdbin_ref,
-    self.seqin,
-    self.hklin,
-    self.label_out_suffix,
-    self.pdbout,
-    self.internal_cycle,
-    self.resolution,
-    self.ncycle)
+'''.format(self.job_title,
+           pdbin_ref,
+           self.seqin,
+           self.hklin,
+           self.label_out_suffix,
+           self.pdbout,
+           self.internal_cycle,
+           self.resolution,
+           self.ncycle)
 
         #self.stdin = self.stdin + self.stdin_extra
         # Add optional nautilus keywords
@@ -462,10 +467,12 @@ xmlout program{8}.xml
         if self.pdbin is not None:
             self.stdin += 'pdbin {0}\n'.format(self.pdbin)
 
+
 class NautilusResultsOnFinish(process_manager.CCPEMPipelineCustomFinish):
     '''
     Generate RVAPI results on finish.
     '''
+
     def __init__(self,
                  pipeline_path,
                  refine_process):
@@ -474,18 +481,19 @@ class NautilusResultsOnFinish(process_manager.CCPEMPipelineCustomFinish):
         self.refine_process = refine_process
 
     def on_finish(self, parent_pipeline=None):
-        #generate RVAPI report
+        # generate RVAPI report
         nautilus_results.PipelineResultsViewer(
-                  pipeline_path=self.pipeline_path)
+            pipeline_path=self.pipeline_path)
 
-def add_refmac_keywords(refkeywords, labelout_suffix):
+
+def add_refmac_keywords(refkeywords):
     '''
     Add default and additional refmac keywords
     '''
-    refine_keywords=''
-#PHOUT
-#PNAME nautilus
-#DNAME nautilus
+    refine_keywords = ''
+# PHOUT
+# PNAME nautilus
+# DNAME nautilus
 #'''.format(labelout_suffix)
 
     if isinstance(refkeywords, str):
@@ -496,4 +504,3 @@ def add_refmac_keywords(refkeywords, labelout_suffix):
                 refine_keywords += line + '\n'
 
     return refine_keywords
-
