@@ -65,7 +65,7 @@ class Privateer(task_utils.CCPEMTask):
         parser.add_argument(
             '-input_pdb',
             '--input_pdb',
-            help='''Input coordinate file (pdb format)#CCPEMTypeFilePDB#''',
+            help='''Input coordinate file (pdb format)''',
             metavar='Input PDB',
             type=str,
             default=None)
