@@ -49,11 +49,13 @@ class Refmac(task_utils.CCPEMTask):
         documentation_link='https://www2.mrc-lmb.cam.ac.uk/groups/murshudov/content/refmac/refmac_keywords.html',
         references=None)
 
-    commands = {'refmac': settings.which(program='refmac5'),
-                'sfcheck': settings.which(program='sfcheck'),
-                'pdbset':  settings.which(program='pdbset'),
-                'libg': settings.which(program='libg')
-                }
+    commands = {
+        'refmac': settings.which(program='refmac5'),
+        'sfcheck': settings.which(program='sfcheck'),
+        'pdbset':  settings.which(program='pdbset'),
+        'libg': settings.which(program='libg'),
+        'ccpem-python':  settings.which(program='ccpem-python')
+    }
 
 
     def __init__(self,
@@ -141,14 +143,6 @@ class Refmac(task_utils.CCPEMTask):
             nargs='*',
             default=None)
 
-        parser.add_argument(
-            '-sfcalc_only',
-            '--sfcalc_only',
-            help='''Calculate structure factors''',
-            metavar='Sfcalc only',
-            type=bool,
-            default=False)
-
         parser.add_argument(
             '-use_cluster',
             '--use_cluster',
@@ -203,7 +197,7 @@ class Refmac(task_utils.CCPEMTask):
             help=('Half map 2 of input map, required for cross validation '
                   'post refinement (mrc format)'),
             type=str,
-            metavar='Half map 1',
+            metavar='Half map 2',
             default=None)
         #
         parser.add_argument(
@@ -216,7 +210,7 @@ class Refmac(task_utils.CCPEMTask):
         parser.add_argument(
             '-start_pdb',
             '--start_pdb',
-            help='Input coordinate file (pdb format)',
+            help='Input coordinate file (pdb/cif format)',
             type=str,
             nargs='*',
             default=None)
@@ -515,7 +509,8 @@ class Refmac(task_utils.CCPEMTask):
                 else:
                     sharp = -1.0*self.args.map_sharpen()
                 self.process_DaC = DivideAndConquer(
-                  command=['ccpem-python', '-m', 'ccpem_core.tasks.refmac.dummy_dac'],
+                  command=[self.commands['ccpem-python'],
+                           '-m', 'ccpem_core.tasks.refmac.dummy_dac'],
                   name='Refine chain by chain',
                   job_location=self.job_location,
                   option_l = option_l_FileName,
@@ -593,212 +588,210 @@ class Refmac(task_utils.CCPEMTask):
             self.libg_restraints_path = [self.process_libg.libg_restraints_path]
             pl.append([self.process_libg.process])
 
-        if not self.args.sfcalc_only.value:
-            # Set cell parameters and scale of PDB
-            self.process_pdb_set = PDBSetCell(
-                command=self.commands['pdbset'],
-                job_location=self.job_location,
-                name='Set PDB cell',
-                pdb_path=self.args.start_pdb.value,
-                map_path=self.args.input_map.value)
-            pl.append([self.process_pdb_set.process])
+        # Set cell parameters and scale of model
+        self.process_set_unit_cell = SetModelCell(
+            job_location=self.job_location,
+            name='Set model unit cell',
+            pdb_path=self.args.start_pdb.value,
+            map_path=self.args.input_map.value)
+        pl.append([self.process_set_unit_cell.process])
 
-            # Find PDB in map
-            if self.args.find_in_map.value:
-                self.process_find_in_map = MolRepFindInMap(
-                     job_location=self.job_location,
-                     mtz_path=self.process_maptomtz.hklout_path,
-                     pdb_path=os.path.join(self.job_location,
-                                           'pdbset.pdb'),
-                     name='Find PDB in map',
-                     resolution=self.args.resolution.value,
-                     nmon=self.args.n_monomers.value,
-                     f_labin='Fout0')
-                pl.append([self.process_find_in_map.process])
-                self.start_refmac_pdb_path = \
-                    self.process_find_in_map.pdbout_path
-            else:
-                self.start_refmac_pdb_path = self.process_pdb_set.pdbout_path
+        # Find PDB in map
+        if self.args.find_in_map.value:
+            self.process_find_in_map = MolRepFindInMap(
+                 job_location=self.job_location,
+                 mtz_path=self.process_maptomtz.hklout_path,
+                 pdb_path=os.path.join(self.job_location,
+                                       'starting_model.pdb'),
+                 name='Find PDB in map',
+                 resolution=self.args.resolution.value,
+                 nmon=self.args.n_monomers.value,
+                 f_labin='Fout0')
+            pl.append([self.process_find_in_map.process])
+            self.start_refmac_pdb_path = \
+                self.process_find_in_map.pdbout_path
+        else:
+            self.start_refmac_pdb_path = self.process_set_unit_cell.cifout_path
+
+        if self.args.local_refinement_on():
+            # Local refine
+            # Get local structure factors.  N.B. this must come after find in
+            # map
+            self.process_maptomtz_local = RefmacMapToMtz(
+                command=self.commands['refmac'],
+                name='Map to MTZ (local)',
+                resolution=self.args.resolution.value,
+                mode='Local',
+                job_location=self.job_location,
+                lib_path=self.args.lib_in.value,
+                map_path=self.args.input_map.value,
+                pdb_path=self.start_refmac_pdb_path,
+                blur_array=[self.args.map_sharpen.value],
+                sharp_array=None,
+                atomsf_path=self.atomsf)
+            pl.append([self.process_maptomtz_local.process])
+
+            # Refinement parameters: Local
+            mode = 'Local'
+            name='Refmac refine (local)'
+            pdb_path=self.process_maptomtz_local.pdbout_path
+            mtz_path = 'masked_fs.mtz'
+        else:
+            # Refinement parameters: Local
+            mode = 'Global'
+            name = 'Refmac refine (global)'
+            pdb_path = self.start_refmac_pdb_path
+            mtz_path = 'starting_map.mtz'
+
+        #  Refine
+        # Invert sharpening value for refmac
+        if self.args.map_sharpen() is None:
+            sharp = 0
+        else:
+            sharp = -1.0*self.args.map_sharpen()
+        self.process_refine = RefmacRefine(
+            command=self.commands['refmac'],
+            job_location=self.job_location,
+            pdb_path=pdb_path,
+            resolution=self.args.resolution.value,
+            mtz_path=mtz_path,
+            mode=mode,
+            name=name,
+            weight=self.args.weight.value,
+            weight_auto=self.args.weight_auto.value,
+            symmetry_auto=self.args.symmetry_auto.value,
+            sharp=sharp,
+            set_bfactor=self.args.set_bfactor_on.value,
+            atom_bfactor=self.args.set_bfactor.value,
+            atomsf_path=self.atomsf,
+            ncycle=self.args.ncycle.value,
+            external_restraints_on=self.args.external_restraints_on.value,
+            external_restraints_path=self.args.restraints_files(),
+            external_weight_scale=self.args.external_weight_scales(),
+            external_weight_gmwt=self.args.external_weight_gmwts(),
+            external_weight_dmax=self.args.external_weight_dmx.value,
+            external_main_chain=self.args.external_main_only.value,
+            lib_path=self.args.lib_in.value,
+            jelly_body_on=self.args.jelly_body.value,
+            hydrogens=self.args.hydrogens.value,
+            keywords=self.args.keywords.value,
+            libg_restraints=self.libg_restraints,
+            libg_restraints_path=self.libg_restraints_path)
+            #
+        pl.append([self.process_refine.process])
+
+        # Validation
+        if self.args.validate_on.value:
+            # Shake structure
+            self.process_shake = PDBSetShake(
+                name='Shake refined structure',
+                job_location=self.job_location,
+                pdb_path=self.process_refine.pdbout_path)
+            pl.append([self.process_shake.process])
+
+            # Convert half map 1 to mtz; stage
+            self.process_maptomtz_hm1 = RefmacMapToMtz(
+                command=self.commands['refmac'],
+                resolution=self.args.resolution.value,
+                refmap_path = self.args.reference_map.value,
+                mode=mode,
+                lib_path=self.args.lib_in.value,
+                name='Map to MTZ (HM1)',
+                job_location=self.job_location,
+                pdb_path=self.process_shake.pdbout_path,
+                pdbout_path='shifted_local_hm1.pdb',
+                map_path=self.args.half_map_1.value,
+                hklout_path=os.path.join(self.job_location,
+                                         'starting_map_hm1.mtz'),
+                blur_array=[self.args.map_sharpen.value],
+                sharp_array=None,
+                atomsf_path=self.atomsf)
+            pl.append([self.process_maptomtz_hm1.process])
+
+            # Refine shaken structure vs half map 1
+            self.process_refine_hm1 = copy.deepcopy(
+                self.process_refine)
 
             if self.args.local_refinement_on():
-                # Local refine
-                # Get local structure factors.  N.B. this must come after find in
-                # map
-                self.process_maptomtz_local = RefmacMapToMtz(
-                    command=self.commands['refmac'],
-                    name='Map to MTZ (local)',
-                    resolution=self.args.resolution.value,
-                    mode='Local',
-                    job_location=self.job_location,
-                    lib_path=self.args.lib_in.value,
-                    map_path=self.args.input_map.value,
-                    pdb_path=self.start_refmac_pdb_path,
-                    blur_array=[self.args.map_sharpen.value],
-                    sharp_array=None,
-                    atomsf_path=self.atomsf)
-                pl.append([self.process_maptomtz_local.process])
-
-                # Refinement parameters: Local
-                mode = 'Local'
-                name='Refmac refine (local)'
-                pdb_path=self.process_maptomtz_local.pdbout_path
-                mtz_path = 'masked_fs.mtz'
+                self.process_refine_hm1.pdb_path = \
+                    self.process_maptomtz_hm1.pdbout_path
+                self.process_refine_hm1.mtz_path = 'masked_fs.mtz'
             else:
-                # Refinement parameters: Local
-                mode = 'Global'
-                name = 'Refmac refine (global)'
-                pdb_path = self.start_refmac_pdb_path
-                mtz_path = 'starting_map.mtz'
+                self.process_refine_hm1.pdb_path = \
+                    self.process_shake.pdbout_path
+                self.process_refine_hm1.mtz_path = \
+                    self.process_maptomtz_hm1.hklout_path
 
-            #  Refine
-            # Invert sharpening value for refmac
-            if self.args.map_sharpen() is None:
-                sharp = 0
-            else:
-                sharp = -1.0*self.args.map_sharpen()
-            self.process_refine = RefmacRefine(
+            self.process_refine_hm1.pdbout_path = os.path.join(
+                self.job_location,
+                'refined_hm1.pdb')
+            self.process_refine_hm1.name = 'Refmac refine (HM1)'
+            self.process_refine_hm1.set_process()
+            pl.append([self.process_refine_hm1.process])
+
+            # Convert half map 2 to mtz
+            self.process_maptomtz_hm2 = RefmacMapToMtz(
                 command=self.commands['refmac'],
-                job_location=self.job_location,
-                pdb_path=pdb_path,
                 resolution=self.args.resolution.value,
-                mtz_path=mtz_path,
-                mode=mode,
-                name=name,
-                weight=self.args.weight.value,
-                weight_auto=self.args.weight_auto.value,
-                symmetry_auto=self.args.symmetry_auto.value,
-                sharp=sharp,
-                set_bfactor=self.args.set_bfactor_on.value,
-                atom_bfactor=self.args.set_bfactor.value,
-                atomsf_path=self.atomsf,
-                ncycle=self.args.ncycle.value,
-                external_restraints_on=self.args.external_restraints_on.value,
-                external_restraints_path=self.args.restraints_files(),
-                external_weight_scale=self.args.external_weight_scales(),
-                external_weight_gmwt=self.args.external_weight_gmwts(),
-                external_weight_dmax=self.args.external_weight_dmx.value,
-                external_main_chain=self.args.external_main_only.value,
+                refmap_path = self.args.reference_map.value,
                 lib_path=self.args.lib_in.value,
-                jelly_body_on=self.args.jelly_body.value,
-                hydrogens=self.args.hydrogens.value,
-                keywords=self.args.keywords.value,
-                libg_restraints=self.libg_restraints,
-                libg_restraints_path=self.libg_restraints_path)
-                #
-            pl.append([self.process_refine.process])
+                mode=mode,
+                name='Map to MTZ (HM2)',
+                job_location=self.job_location,
+                map_path=self.args.half_map_2.value,
+                pdb_path=self.process_refine_hm1.pdbout_path,
+                pdbout_path='shifted_local_hm2.pdb',
+                hklout_path=os.path.join(self.job_location,
+                                         'starting_map_hm2.mtz'),
+                blur_array=[self.args.map_sharpen.value],
+                sharp_array=None,
+                atomsf_path=self.atomsf)
+            pl.append([self.process_maptomtz_hm2.process])
 
-            # Validation
-            if self.args.validate_on.value:
-                # Shake structure
-                self.process_shake = PDBSetShake(
-                    name='Shake refined structure',
-                    job_location=self.job_location,
-                    pdb_path=self.process_refine.pdbout_path)
-                pl.append([self.process_shake.process])
+            # Calculate FSC of model refined against half map 1
+            # vs half map 2
+            self.process_fsc_hm2 = copy.deepcopy(
+                self.process_refine_hm1)
 
-                # Convert half map 1 to mtz; stage
-                self.process_maptomtz_hm1 = RefmacMapToMtz(
-                    command=self.commands['refmac'],
-                    resolution=self.args.resolution.value,
-                    refmap_path = self.args.reference_map.value,
-                    mode=mode,
-                    lib_path=self.args.lib_in.value,
-                    name='Map to MTZ (HM1)',
-                    job_location=self.job_location,
-                    pdb_path=self.process_shake.pdbout_path,
-                    pdbout_path='shifted_local_hm1.pdb',
-                    map_path=self.args.half_map_1.value,
-                    hklout_path=os.path.join(self.job_location,
-                                             'starting_map_hm1.mtz'),
-                    blur_array=[self.args.map_sharpen.value],
-                    sharp_array=None,
-                    atomsf_path=self.atomsf)
-                pl.append([self.process_maptomtz_hm1.process])
+            # Set to 0 cycles to only calculate stats and do not reset
+            # bfactors
+            self.process_fsc_hm2.ncycle=0
+            self.process_fsc_hm2.set_bfactor=False
 
-                # Refine shaken structure vs half map 1
-                self.process_refine_hm1 = copy.deepcopy(
-                    self.process_refine)
+            if self.args.local_refinement_on():
+                self.process_fsc_hm2.pdb_path = \
+                    self.process_maptomtz_hm2.pdbout_path
+                self.process_fsc_hm2.mtz_path = 'masked_fs.mtz'
+            else:
+                self.process_fsc_hm2.pdb_path = \
+                    self.process_refine_hm1.pdbout_path
+                self.process_fsc_hm2.mtz_path = \
+                    self.process_maptomtz_hm2.hklout_path
 
-                if self.args.local_refinement_on():
-                    self.process_refine_hm1.pdb_path = \
-                        self.process_maptomtz_hm1.pdbout_path
-                    self.process_refine_hm1.mtz_path = 'masked_fs.mtz'
-                else:
-                    self.process_refine_hm1.pdb_path = \
-                        self.process_shake.pdbout_path
-                    self.process_refine_hm1.mtz_path = \
-                        self.process_maptomtz_hm1.hklout_path
+            self.process_fsc_hm2.pdbout_path = os.path.join(
+                self.job_location,
+                'refined_hm2.pdb')
+            self.process_fsc_hm2.name = 'Refmac stats (HM2)'
+            self.process_fsc_hm2.set_process()
+            pl.append([self.process_fsc_hm2.process])
 
-                self.process_refine_hm1.pdbout_path = os.path.join(
-                    self.job_location,
-                    'refined_hm1.pdb')
-                self.process_refine_hm1.name = 'Refmac refine (HM1)'
-                self.process_refine_hm1.set_process()
-                pl.append([self.process_refine_hm1.process])
-
-                # Convert half map 2 to mtz
-                self.process_maptomtz_hm2 = RefmacMapToMtz(
-                    command=self.commands['refmac'],
-                    resolution=self.args.resolution.value,
-                    refmap_path = self.args.reference_map.value,
-                    lib_path=self.args.lib_in.value,
-                    mode=mode,
-                    name='Map to MTZ (HM2)',
-                    job_location=self.job_location,
-                    map_path=self.args.half_map_2.value,
-                    pdb_path=self.process_refine_hm1.pdbout_path,
-                    pdbout_path='shifted_local_hm2.pdb',
-                    hklout_path=os.path.join(self.job_location,
-                                             'starting_map_hm2.mtz'),
-                    blur_array=[self.args.map_sharpen.value],
-                    sharp_array=None,
-                    atomsf_path=self.atomsf)
-                pl.append([self.process_maptomtz_hm2.process])
-
-                # Calculate FSC of model refined against half map 1 
-                # vs half map 2
-                self.process_fsc_hm2 = copy.deepcopy(
-                    self.process_refine_hm1)
-
-                # Set to 0 cycles to only calculate stats and do not reset
-                # bfactors
-                self.process_fsc_hm2.ncycle=0
-                self.process_fsc_hm2.set_bfactor=False
-
-                if self.args.local_refinement_on():
-                    self.process_fsc_hm2.pdb_path = \
-                        self.process_maptomtz_hm2.pdbout_path
-                    self.process_fsc_hm2.mtz_path = 'masked_fs.mtz'
-                else:
-                    self.process_fsc_hm2.pdb_path = \
-                        self.process_refine_hm1.pdbout_path
-                    self.process_fsc_hm2.mtz_path = \
-                        self.process_maptomtz_hm2.hklout_path
-
-                self.process_fsc_hm2.pdbout_path = os.path.join(
-                    self.job_location,
-                    'refined_hm2.pdb')
-                self.process_fsc_hm2.name = 'Refmac stats (HM2)'
-                self.process_fsc_hm2.set_process()
-                pl.append([self.process_fsc_hm2.process])
-
-                # Calculate FSC of two half maps
-                self.process_fsc_hm1_vs_hm2 = RefmacFSC(
-                    command=self.commands['refmac'],
-                    name='Refmac FSC (HM1 vs HM2)',
-                    job_location=self.job_location,
-                    map1_path=self.args.half_map_1.value,
-                    map2_path=self.args.half_map_2.value,
-                    resolution=self.args.resolution.value,
-                    refmap_path=self.args.reference_map.value,
-                    atomsf_path=self.atomsf)
-                pl.append([self.process_fsc_hm1_vs_hm2.process])
+            # Calculate FSC of two half maps
+            self.process_fsc_hm1_vs_hm2 = RefmacFSC(
+                command=self.commands['refmac'],
+                name='Refmac FSC (HM1 vs HM2)',
+                job_location=self.job_location,
+                map1_path=self.args.half_map_1.value,
+                map2_path=self.args.half_map_2.value,
+                resolution=self.args.resolution.value,
+                refmap_path=self.args.reference_map.value,
+                atomsf_path=self.atomsf)
+            pl.append([self.process_fsc_hm1_vs_hm2.process])
 
         # Generate results
         refmac_results_path = os.path.realpath(refmac_results.__file__)
         refmac_results_process = process_manager.CCPEMProcess(
             name='Refmac results',
-            command='ccpem-python',
+            command=self.commands['ccpem-python'],
             args=[refmac_results_path, self.job_location, mode],
             location=self.job_location)
         pl.append([refmac_results_process])
@@ -817,12 +810,11 @@ class Refmac(task_utils.CCPEMTask):
             self.pipeline.start()
 
 
-class PDBSetCell(object):
+class SetModelCell(object):
     '''
     Set input PDB cell and scale cards to input map.
     '''
     def __init__(self,
-                 command,
                  job_location,
                  pdb_path,
                  map_path,
@@ -830,44 +822,27 @@ class PDBSetCell(object):
         self.job_location = ccpem_utils.get_path_abs(job_location)
         self.pdb_path = ccpem_utils.get_path_abs(pdb_path)
         self.map_path = ccpem_utils.get_path_abs(map_path)
-        self.pdbout_path = os.path.join(self.job_location,
-                                        'pdbset.pdb')
-        self.cell_info = None
-        self.stdin = None
+        self.pdbout_path = os.path.join(self.job_location, 'starting_model.pdb')
+        self.cifout_path = os.path.join(self.job_location, 'starting_model.cif')
         #
-        self.get_cell_parameters()
-        self.set_stdin()
-        self.set_args()
         self.name = name
         if self.name is None:
             self.name = self.__class__.__name__
+        #
+        command = which('ccpem-python')
+        self.args = [
+            '-m', 'ccpem_core.model_tools.command_line',
+            '--model', self.pdb_path,
+            '--set_cell_from_map', self.map_path,
+            '--output_pdb',
+            '--output_cif',
+            '--output_name', os.path.splitext(self.cifout_path)[0]
+        ]
         self.process = process_manager.CCPEMProcess(
             name=self.name,
             command=command,
             args=self.args,
-            location=self.job_location,
-            stdin=self.stdin)
-
-    def get_cell_parameters(self):
-        if os.path.exists(path=self.map_path):
-            with mrcfile.open(self.map_path, header_only=True) as map:
-                self.cell_info = [map.header.cella.x,
-                                  map.header.cella.y,
-                                  map.header.cella.z,
-                                  map.header.cellb.alpha,
-                                  map.header.cellb.beta,
-                                  map.header.cellb.gamma]
-
-    def set_args(self):
-        self.args = ['xyzin', self.pdb_path]
-        self.args += ['xyzout', self.pdbout_path]
-
-    def set_stdin(self):
-        if self.cell_info is not None:
-            self.stdin = 'CELL '
-            for p in self.cell_info:
-                self.stdin += ' {0}'.format(p)
-            self.stdin += '\nend'
+            location=self.job_location)
 
 
 class LibgRestraints(object):
@@ -1143,6 +1118,7 @@ class RefmacRefine(object):
         self.mtz_path = mtz_path
         self.pdbout_path = os.path.join(self.job_location,
                                         pdbout_path)
+        self.cifout_path = os.path.splitext(self.pdbout_path)[0] + '.mmcif'
         self.name = name
         if self.name is None:
             self.name = self.__class__.__name__ + self.mode
@@ -1284,13 +1260,19 @@ class RefmacSfcalcCrd(object):
                  job_location,
                  pdb_path,
                  lib_in=None,
-                 resolution=None):
+                 resolution=None,
+                 name=None,
+                 command = None):
         self.job_location = job_location
         self.pdb_path = pdb_path
         self.lib_in = lib_in
         self.resolution = resolution
-        self.name = self.__class__.__name__
-
+        if name is None:
+            self.name = self.__class__.__name__
+        else:
+            self.name = name
+        if command is not None:
+            self.command = command
         self.set_args()
         self.set_stdin()
 
@@ -1322,7 +1304,8 @@ class PDBSetShake(object):
                  job_location,
                  pdb_path,
                  pdbout_path=None,
-                 name=None):
+                 name=None,
+                 shift=0.5):
         command = which('pdbset')
         assert command is not None
         self.job_location = job_location
@@ -1337,7 +1320,7 @@ class PDBSetShake(object):
         self.stdin = None
         #
         self.set_args()
-        self.set_stdin()
+        self.set_stdin(shift)
         #
         self.process = process_manager.CCPEMProcess(
             name=self.name,
@@ -1350,9 +1333,8 @@ class PDBSetShake(object):
         self.args = ['xyzin', self.pdb_path]
         self.args += ['xyzout', self.pdbout_path]
 
-    def set_stdin(self):
-        self.stdin = '\nnoise 0.5'
-        self.stdin += '\nend'
+    def set_stdin(self, shift):
+        self.stdin = '\nnoise {}\nend'.format(shift)
 
 
 class RefmacFSC(object):
