@@ -9,7 +9,7 @@
 
 import os
 from ccpem_core.ccpem_utils import ccpem_argparser
-from ccpem_progs.ribfind import run_ribfind
+from ccpem_progs.ribfind import ribfind_c_SM
 from ccpem_core import settings
 from ccpem_core import process_manager
 from ccpem_core.tasks import task_utils
@@ -31,7 +31,7 @@ class Ribfind(task_utils.CCPEMTask):
         references=None)
 
     commands = {'ribfind': ['ccpem-python',
-                            os.path.realpath(run_ribfind.__file__)],
+                            os.path.realpath(ribfind_c_SM.__file__)],
                 'dssp': settings.which('mkdssp')}
 
     def __init__(self,
@@ -111,7 +111,9 @@ class Ribfind(task_utils.CCPEMTask):
 
     def job_ribfind(self):
         assert self.args.jsonfile is not None
-        args = [self.args.jsonfile]
+        #args = [self.args.jsonfile]
+        args = [self.args.input_pdb.value, self.args.output_dssp.value, 
+                self.args.contact_distance.value]
         self.ribfind_process = process_manager.CCPEMProcess(
             name='Ribfind',
             command=self.commands['ribfind'],
@@ -155,7 +157,7 @@ class Ribfind(task_utils.CCPEMTask):
         self.args.output_dssp.value = os.path.join(
             self.job_location,
             os.path.basename(self.args.output_dssp.value))
-        if self.args.contact_distance.value <= 0:
+        if self.args.contact_distance.value < 0:
             warnings += \
                 '\nContact distance not greater than zero : {0}'.format(
                     self.args.contact_distance.value)
