@@ -71,7 +71,7 @@ class Shake(task_utils.CCPEMTask):
         parser.add_argument(
             '-shift',
             '--shift',
-            help='''Maximum shift (Angstrom)''',
+            help='''Maximum shift (Angstrom, 0.0 - 0.5)''',
             metavar='Max shift',
             type=float,
             default=0.5)
@@ -79,7 +79,7 @@ class Shake(task_utils.CCPEMTask):
         parser.add_argument(
             '-keywords',
             '--keywords',
-            help='Keywords for advanced options.  Select file or define text',
+            help='Keywords for advanced options',
             type=str,
             metavar='Keywords',
             default='')
@@ -95,7 +95,8 @@ class Shake(task_utils.CCPEMTask):
             name='Shake refined structure',
             job_location=self.job_location,
             pdb_path=self.args.input_pdb(),
-            pdbout_path=self.pdbout_path)
+            pdbout_path=self.pdbout_path,
+            shift=self.args.shift())
         pl.append([self.process_shake.process])
         self.pipeline = process_manager.CCPEMPipeline(
             pipeline=pl,
