@@ -4,13 +4,14 @@
 import sys
 from TEMPy.StructureParser import PDBParser
 from TEMPy.StructureBlurrer import StructureBlurrer
-import os
+import os, subprocess
 from ccpem_core.tasks.tempy.difference_map.diffmap_argparser import DiffMapParser
 from ccpem_core.map_tools.loc_scale.set_locscale_diffmap import \
         launch_amplitude_scaling
 from TEMPy.ShowPlot import Plot
 from TEMPy.mapprocess import mapcompare
 from TEMPy.mapprocess import Filter
+from TEMPy.mapprocess import array_utils
 from copy import deepcopy
 import gc
 import numpy as np
@@ -23,6 +24,9 @@ except ImportError:
 #from memory_profiler import profile
 import datetime
 import warnings
+from ccpem_core.map_tools.loc_scale import np_locscale_avg
+loc_scale_python= ['ccpem-python',os.path.realpath(np_locscale_avg.__file__)]
+
 
 #calculate model contour
 def model_contour(p,res=4.0,emmap=False,t=-1.):
@@ -31,14 +35,14 @@ def model_contour(p,res=4.0,emmap=False,t=-1.):
     if t != -1.0:
         contour = t*modelmap.std()#0.0
     return modelmap,contour,modelinstance
-def blur_model(p,res=4.0,emmap=False):
+def blur_model(p,res=4.0,emmap=None):
     print 'Reading the model'
     structure_instance=PDBParser.read_PDB_file('pdbfile',p,hetatm=True,water=False)
     print 'Generating map from the model'
     blurrer = StructureBlurrer()
     if res is None: sys.exit('Map resolution required..')
     modelmap = blurrer.gaussian_blur_real_space(structure_instance, 
-                                              res,sigma_coeff=0.187,
+                                              res,sigma_coeff=0.225,
                                               densMap=emmap,normalise=True) 
     return modelmap, structure_instance    
 
@@ -48,10 +52,12 @@ def read_mapfile(map_path):
         if np.any(np.isnan(mrcobj.data)):
             sys.exit('Map has NaN values: {}'.format(map_path))
         emmap = Filter(mrcobj)
+        emmap.fix_origin()
     else:
         mrcobj=MapParser.readMRC(map_path)
         emmap = Filter(mrcobj)
         emmap.set_apix_as_tuple()
+        emmap.fix_origin()
     return emmap
 
 def write_mapfile(mapobj,map_path):
@@ -100,6 +106,16 @@ def set_args_local_scale(args,outfile,mpi=False,
     if args.mask is None:
         args.mask = mask
 
+def lowpass_filter(mrcmap,resolution,inplace=False):
+    lowpass_freq = mrcmap.apix[0]/resolution
+    ftfilter = array_utils.tanh_lowpass(mrcmap.fullMap.shape,
+                                        cutoff=lowpass_freq, 
+                                        fall=0.2)
+    filtmap = mrcmap.fourier_filter(ftfilter=ftfilter,inplace=inplace)
+    if inplace: return mrcmap
+    return filtmap
+
+
 def main():
     #print help
     Usage = \
@@ -194,6 +210,7 @@ def main():
         emmap2,c2,p2inst = model_contour(p2,res=r,emmap=False,t=0.1)
         flag_filt = False
         flag_scale = False
+        output_synthetic_map = True
     #map and model
     elif None in [m1,m2]:
         # for one map and model, m = map, c1 = map contour, c2 = model contour
@@ -203,7 +220,7 @@ def main():
         emmap1 = read_mapfile(m)
         #set contour
         if c1 is None and c is None: 
-            c1 = emmap1.calculate_map_contour(sigma_factor=1.5)
+            c1 = emmap1.calculate_map_contour(sigma_factor=2.0)
             c = c1
             
         if r1 is None and r is None: 
@@ -220,11 +237,12 @@ def main():
         elif r1 > 6.0: mt = 0.5
         else: mt = 0.1
         if c2 is None:
-            Name2 = os.path.basename(p).split('.')[0]
+            Name2 = os.path.basename(p).split('.')[0]+'_syn'
             emmap2,c2,p2inst = model_contour(p,res=r1,emmap=emmap1,t=mt)
         #scale based on the model amplitudes
         if refsc is None:
-            refsc = tp.args.refscale = True   
+            refsc = tp.args.refscale = True
+            #flag_filt = False
         output_synthetic_map = True
     else: 
         # For 2 input maps
@@ -235,12 +253,12 @@ def main():
         Name1 = os.path.basename(m1).split('.')[0]
         emmap1 = read_mapfile(m1)
         if c1 is None:
-            c1 = emmap1.calculate_map_contour(sigma_factor=1.5)
+            c1 = emmap1.calculate_map_contour(sigma_factor=2.0)
         print 'Reading map2' 
         Name2 = os.path.basename(m2).split('.')[0]
         emmap2 = read_mapfile(m2)
         if c2 is None:
-            c2 = emmap2.calculate_map_contour(sigma_factor=1.5)
+            c2 = emmap2.calculate_map_contour(sigma_factor=2.0)
             
     gc.collect()
     #MAIN CALCULATION
@@ -272,59 +290,157 @@ def main():
     
     #SCALING, scaled maps (diff1,diff2)
     if flag_scale:
+            
         if refsc: print 'Using second model/map amplitudes as reference'
         #local scaling
         if scaling_mode != 'global':
+            #make a shallow copy
+            emmap_1 = emmap1.copy(deep=False)
+            emmap_2 = emmap2.copy(deep=False)
+            #low pass filter (new maps generated)
+            if flag_filt:
+                print 'Low-pass filtering'
+                try:
+                    emmap_1 = lowpass_filter(emmap1,r1,inplace=False)
+                    emmap_2 = lowpass_filter(emmap2,r2,inplace=False)
+                    gc.collect()
+                except:
+                    flag_filt = False # to check failure
+                    print 'Low-pass filtering failed'
+            else:
+                print 'Low-pass filter disabled'
+                    
             print 'Using local scaling mode'
             outfile = Name1+'_'+Name2+'.mrc'
+            #set loc scale arguments
+            if tp.args.mpi:
+                list_args = ['ccpem-mpirun',
+                       '-np',
+                       str(tp.args.n_mpi)]+loc_scale_python+['-mpi']
+            else:
+                list_args = loc_scale_python
             #interpolate to common grid
             if not samegrid:
-                spacing = max(max(emmap1.apix),max(emmap2.apix))
-                grid_shape, new_ori = mapcompare.alignment_box(emmap1,emmap2,spacing)
+                spacing = max(max(emmap_1.apix),max(emmap_2.apix))
+                grid_shape, new_ori = mapcompare.alignment_box(emmap_1,emmap_2,spacing)
                 #     if not apix is None: spacing = apix
-                #interpolate to common grid
-                diff1 = emmap1.interpolate_to_grid(grid_shape,spacing,new_ori,inplace=False)
-                diff2 = emmap2.interpolate_to_grid(grid_shape,spacing,new_ori,inplace=False)
-                
+                #interpolate to common grid (new maps)
+                diff1 = emmap_1.interpolate_to_grid(grid_shape,spacing,new_ori,inplace=False)
+                diff2 = emmap_2.interpolate_to_grid(grid_shape,spacing,new_ori,inplace=False)
                 #save maps
                 write_mapfile(diff1, 'map1.mrc')
                 write_mapfile(diff2, 'map2.mrc')
-                
+                #set mask for local scaling
                 if tp.args.mask is None:
-                    maskmap = diff1.copy(deep=True)
-                    maskmap.fullMap[:] = diff1.fullMap > c1
+                    if output_synthetic_map:#model input
+                        maskmap = diff2.copy(deep=True)
+                        maskmap.fullMap[:] = diff2.fullMap > 0.1
+                    else:
+                        maskmap = diff1.copy(deep=True)
+                        #union of both map contours
+                        maskmap.fullMap[:] = (diff1.fullMap > c1) | (diff2.fullMap > c2)
+                        #maskmap.fullMap[:] = maskmap.fullMap > 0
                     write_mapfile(maskmap, 'mask.mrc')
                     del maskmap
+                #delete interpolated maps
                 if refsc: del diff1
                 else: del diff1, diff2
+                #delete lowpass filtered maps
+                if flag_filt:
+                    del emmap_2
+                #set local scaling arguments
+                list_args += ['-em','map1.mrc','-mm','map2.mrc',
+                                 '-p',str(round(spacing,3)),
+                                 '-o',outfile]
                 set_args_local_scale(tp.args, outfile, map1='map1.mrc', map2='map2.mrc',
-                                     apix=spacing,mask='mask.mrc')
+                                     apix=round(spacing,3),mask='mask.mrc')
                 gc.collect()
             else:
                 if tp.args.mask is None:
-                    maskmap = emmap1.copy(deep=True)
-                    maskmap.fullMap[:] = emmap1.fullMap > c1
+                    #create mask
+                    if output_synthetic_map:#input model
+                            maskmap = emmap2.copy(deep=True)
+                            #TODO: check mask with both model and map contours
+                            maskmap.fullMap[:] = emmap2.fullMap > 0.1
+                    else:
+                        try:
+                            mapcompare.compare_grid(emmap1,emmap_1)#if lowpass filtered map has same grid
+                            maskmap = emmap1.copy(deep=True)
+                            #union of two masks
+                            maskmap.fullMap[:] = (emmap1.fullMap > c1) | (emmap2.fullMap > c2)
+                        except AssertionError:
+                            maskmap = emmap_1.copy(deep=True)
+                            maskmap.fullMap[:] = (emmap_1.fullMap > c1) | (emmap_2.fullMap > c2)
+                        #maskmap.fullMap[:] = maskmap.fullMap > 0
                     write_mapfile(maskmap, 'mask.mrc')
                     del maskmap
+                #if lowpass filtered
+                if flag_filt:
+                    write_mapfile(emmap_1, 'map1.mrc')
+                    write_mapfile(emmap_2, 'map2.mrc')
+                    list_args += ['-em','map1.mrc','-mm','map2.mrc',
+                                 '-p',str(round(max(emmap_1.apix),3)),
+                                 '-o',outfile]
+                    set_args_local_scale(tp.args, outfile,map1='map1.mrc',map2='map2.mrc',
+                                         apix=max(emmap_1.apix),mask='mask.mrc')
                 #for model input
-                if tp.args.inp_map2 is None: 
-                    write_mapfile(emmap2, 'map2.mrc')
+                elif tp.args.inp_map2 is None: 
+                    write_mapfile(emmap_2, 'map2.mrc')
+                    list_args += ['-em',tp.args.inp_map1,'-mm','map2.mrc',
+                                 '-p',str(round(max(emmap_1.apix),3)),
+                                 '-o',outfile]
                     set_args_local_scale(tp.args, outfile,map2='map2.mrc',
-                                         apix=max(emmap1.apix),mask='mask.mrc')
-                else: set_args_local_scale(tp.args, outfile,apix=max(emmap1.apix),
+                                         apix=max(emmap_1.apix),mask='mask.mrc')
+                else:
+                    list_args += ['-em',tp.args.inp_map1,'-mm',tp.args.inp_map2,
+                                 '-p',str(round(max(emmap_1.apix),3)),
+                                 '-o',outfile]
+                    set_args_local_scale(tp.args, outfile,apix=max(emmap_1.apix),
                                            mask='mask.mrc')
-                if refsc: diff2 = emmap2.copy(deep=True)
-            
+                #store diff2, delete lowpass filtered map
+                if refsc: 
+                    if not flag_filt: 
+                        diff2 = emmap_2.copy(deep=True)
+                        del emmap_2
+                    else: diff2 = emmap_2.copy(deep=False)
+                gc.collect()
+            #delete low-pass filtered map
+            if flag_filt:
+                del emmap_1
+            if tp.args.mask is None:
+                list_args += ['-ma','mask.mrc']
+            else:
+                list_args += ['-ma',tp.args.mask]
+            if tp.args.window != None:
+                list_args += ['-w',tp.args.window]
+            list_args += ['-v','True']
+            print ' '.join(list_args)
             if refsc:
-                emmap_1 = launch_amplitude_scaling(tp.args)
-                diff1 = Filter(emmap_1)
+                emmap1_locscaled = outfile
+                subprocess.call(list_args)
+                assert os.path.isfile(emmap1_locscaled)
+                
+#                 emmap1_locscaled = launch_amplitude_scaling(tp.args)
+                diff1 = read_mapfile(emmap1_locscaled)
                 
             else:
-                emmap_1,emmap_2 = launch_amplitude_scaling(tp.args)
-                diff1 = Filter(emmap_1)
-                diff2 = Filter(emmap_2)
+                list_args += ['-avg']
+                emmap1_locscaled = '.'.join(outfile.split('.')[:-1])+'_map1.mrc'
+                emmap2_locscaled = '.'.join(outfile.split('.')[:-1])+'_map2.mrc'
+                subprocess.call(list_args)
+                assert os.path.isfile(emmap1_locscaled)
+                assert os.path.isfile(emmap2_locscaled)
                 
+#                 emmap1_locscaled,emmap2_locscaled = launch_amplitude_scaling(tp.args)
+                diff1 = read_mapfile(emmap1_locscaled)
+                diff2 = read_mapfile(emmap2_locscaled)
+            
+            gc.collect()
+        #global scaling
         else:
+            if not flag_filt:
+                print 'Low-pass filter disabled'
+                
             print 'Using global scaling mode'
             if not samegrid:
                 spacing = max(max(emmap1.apix),max(emmap2.apix))
@@ -334,14 +450,15 @@ def main():
                 #interpolate to common grids
                 emmap_1 = emmap1.interpolate_to_grid(grid_shape,spacing,new_ori,inplace=False)
                 emmap_2 = emmap2.interpolate_to_grid(grid_shape,spacing,new_ori,inplace=False)
-                # amplitude scaling independant of the grid
+                # amplitude scaling 
                 diff1,diff2, dict_plot = mapcompare.amplitude_match(
                                     emmap_1,emmap_2,reso=max(r1,r2),lpfiltb=flag_filt,
                                     ref=refsc)
+                #delete interpolated maps
                 del emmap_1, emmap_2
                 gc.collect()
             else:
-                # amplitude scaling independant of the grid
+                # amplitude scaling 
                 diff1,diff2, dict_plot = mapcompare.amplitude_match(
                                     emmap1,emmap2,reso=max(r1,r2),lpfiltb=flag_filt,
                                     ref=refsc)
@@ -350,6 +467,8 @@ def main():
                 pl = Plot()
                 pl.lineplot(dict_plot,'spectra.png')
     else:
+        print 'Scaling disabled'
+        #make copies of input maps
         diff1 = emmap1.copy(deep=True)
         diff2 = emmap2.copy(deep=True)
 
@@ -365,13 +484,14 @@ def main():
         diff2.shift_density(-min_scaled_maps,inplace=True)
     print 'Calculating difference'
     
-    # find difference map and apply contour mask
+    # store scaled map 1
     scaledmap1 = diff1.copy(deep=True)
+    #calculate difference map 1
     diff1.fullMap = (diff1.fullMap - diff2.fullMap)
-    
+    #store scaled map 2
     if not tp.args.nofracmap or flag_dust or not tp.args.thr_frac is None:
         scaledmap2 = diff2.copy(deep=True)
-        
+    #calculate difference map 2
     diff2.fullMap = (diff2.fullMap - scaledmap1.fullMap)
     
 #     if contour_mask:
@@ -404,11 +524,11 @@ def main():
         
         if tp.args.thr_frac is None: 
             if tp.args.mode == 'global': 
-                print 'Using threshold {} for fractional difference'.format(0.35)
-                mask1.fullMap[:] = mask1.fullMap > 0.35
+                print 'Using threshold {} for fractional difference'.format(0.3)
+                mask1.fullMap[:] = mask1.fullMap > 0.3
             else: 
-                print 'Using threshold {} for fractional difference'.format(0.35)
-                mask1.fullMap[:] = mask1.fullMap > 0.35
+                print 'Using threshold {} for fractional difference'.format(0.3)
+                mask1.fullMap[:] = mask1.fullMap > 0.3
         else: 
             print 'Using threshold {} for fractional difference'.format(tp.args.thr_frac)
             mask1.fullMap[:] = mask1.fullMap > tp.args.thr_frac
@@ -419,8 +539,8 @@ def main():
                             prob=randsize,inplace=True)
         
         if tp.args.thr_frac is None: 
-            if tp.args.mode == 'global': mask2.fullMap[:] = mask2.fullMap > 0.35
-            else: mask2.fullMap[:] = mask2.fullMap > 0.35
+            if tp.args.mode == 'global': mask2.fullMap[:] = mask2.fullMap > 0.3
+            else: mask2.fullMap[:] = mask2.fullMap > 0.3
         else: mask2.fullMap[:] = mask2.fullMap > tp.args.thr_frac
         diff2.apply_mask(mask2.fullMap,inplace=True)
         
@@ -456,7 +576,7 @@ def main():
     # If PDB given write out synthetic map
     if output_synthetic_map:
         print 'Output synthetic map from : ', Name2
-        synmap_file = Name2+'_syn.mrc'
+        synmap_file = Name2+'.mrc'
         write_mapfile(emmap2,synmap_file)
 
 if __name__ == '__main__':
