@@ -25,9 +25,13 @@ from ccpem_core import ccpem_utils
 from ccpem_core.tasks.tempy.scores import scores_results
 from collections import OrderedDict
 from TEMPy.mapprocess import Filter
-import mrcfile
+mrcfile_import=True
+try:
+    import mrcfile
+except ImportError:
+    mrcfile_import = False
 
-def main(json_path, verbose=True):
+def main(json_path, verbose=True,set_results=True):
     # Process arguments from json parameter file
     args = json.load(open(json_path, 'r'))
     map_path = args['map_path']
@@ -37,7 +41,18 @@ def main(json_path, verbose=True):
     map_level = None
     if not args['auto_contour_level']:
         map_level = args['map_contour_level']
-
+    syn_map_list = []
+    if args['use_refmac']:
+        if 'sim_maps' in args:
+            simmap = True
+            for smap in args['sim_maps']:
+                if not os.path.exists(smap):
+                    simmap = False
+                    break
+            if simmap: syn_map_list = args['sim_maps']
+        if 'map_path_edit' in args:
+            if os.path.exists(args['map_path_edit']):
+                map_path = args['map_path_edit']
     if job_location is None:
         job_location = os.getcwd() 
 
@@ -52,11 +67,12 @@ def main(json_path, verbose=True):
         map_resolution=map_resolution,
         map_level = map_level,
         pdb_path_list=pdb_path_list,
+        syn_map_list=syn_map_list,
         directory=job_location)
         
     # Save scores as csv
     csv_path = os.path.join(job_location,
-                            'global_scores.txt')
+                            'global_scores.csv')
     if verbose:
         ccpem_utils.print_sub_header('Save raw scores')
         ccpem_utils.print_sub_sub_header(csv_path)
@@ -64,19 +80,19 @@ def main(json_path, verbose=True):
         scores_df.to_csv(path,sep=';')
             
     # Set jsrview
-    scores_results.GlobScoreResultsViewer(
-        scores_df,
-        directory=job_location)
+    if set_results and not set_results == 'False':
+        scores_results.GlobScoreResultsViewer(
+            scores_df,
+            directory=job_location)
 
 
 def process_global_scores(map_path,
                         map_resolution,
                         map_level,
                         pdb_path_list,
+                        syn_map_list,
                         directory):
-    mrcobj=mrcfile.open(map_path,mode='r')
-    em_map = Filter(mrcobj)
-    em_map.set_apix_tempy()
+    em_map = read_mapfile(map_path)
     #em_map = MapParser.readMRC(map_path)
     if map_level is None:
         map_level = map_contour(em_map, t=1.5)
@@ -85,27 +101,37 @@ def process_global_scores(map_path,
     dict_table = {}
     dict_scores = {}
     list_pdbs = []
+    l = 0
+    if map_resolution > 20.0: t = 2.0
+    elif map_resolution > 10.0: t = 1.0
+    elif map_resolution > 6.0: t = 0.5
+    else: t = 0.1
     for pdb_path in pdb_path_list:
         pdb_id = os.path.basename(pdb_path)
         list_pdbs.append(pdb_id)
-        pName,modelmap = blur_model(pdb_path, map_resolution, em_map)
-        if map_resolution > 20.0: t = 2.0
-        elif map_resolution > 10.0: t = 1.0
-        elif map_resolution > 6.0: t = 0.5
-        else: t = 0.1
-        model_level = model_contour(modelmap, t)
+        if len(syn_map_list) == 0:
+            pName,modelmap = blur_model(pdb_path, map_resolution, em_map)
+            model_level = model_contour(modelmap, t)
+        else:
+            synmapfile = syn_map_list[l]
+            modelmap = read_mapfile(synmapfile)
+            pName = os.path.splitext(pdb_id)[0]
+            #level for Refmac map
+            model_level = 0.009
         #calculate scores
         get_global_scores(em_map,modelmap,map_level,model_level,dict_scores,pName)
+        l += 1
+        
     sc = ScoringFunctions()
     
     dict_scores['mi_ov'] = sc.scale_median(
-                            dict_scores['overlap'],
+                            dict_scores['overlap_model'],
                             dict_scores['local_mi'])
     dict_scores['ccc_ov'] = sc.scale_median(
-                            dict_scores['overlap'],
+                            dict_scores['overlap_model'],
                             dict_scores['local_correlation'])
 
-    order = ['overlap','correlation','mi','local_correlation',
+    order = ['overlap_map','overlap_model','correlation','mi','local_correlation',
              'local_mi','ccc_ov','mi_ov']
     dict_scores_ordered = OrderedDict()
     for k in order:
@@ -113,11 +139,23 @@ def process_global_scores(map_path,
     df = pd.DataFrame(dict_scores_ordered,index = list_pdbs)
     return df
 
+def read_mapfile(map_path):
+    if mrcfile_import:
+        mrcobj=mrcfile.open(map_path,mode='r',permissive=True)
+        emmap = Filter(mrcobj)
+        emmap.fix_origin()
+        emmap.set_apix_tempy()
+    else:
+        mrcobj=MapParser.readMRC(map_path)
+        emmap = Filter(mrcobj)
+        emmap.fix_origin()
+    return emmap
+
 #calculate map contour
 def map_contour(emmap,t=-1.):
     c1 = 0.0
     if t != -1.0:
-        print 'calculating map contour'
+        #print 'calculating map contour'
         zeropeak,ave,sigma1 = emmap.peak_density()
         if not zeropeak is None: c1 = zeropeak+(t*sigma1)
     return c1
@@ -126,6 +164,7 @@ def model_contour(modelmap,t=-1.):
     c2 = 0.0
     if t != -1.0:
         c2 = t*modelmap.std()#0.0
+    #print 'Calculated model contour: ', c2
     return c2
 
 def blur_model(p,res=4.0,emmap=False):
@@ -141,18 +180,22 @@ def get_global_scores(emmap1,emmap2,c1,c2,dict_scores_hits,Name2):
     sc = ScoringFunctions()
     print Name2
     #OVR
-    try:
-        ccc_mask,ovr = sc.CCC_map(emmap1,emmap2,c1,c2,3,meanDist=True)
-        print 'Percent overlap:', ovr
-        if ovr < 0.0: ovr = 0.0
-    except:
-        print 'ccc and overlap score calculation failed for', Name2
-        ovr = 0.0
-    if ovr < 0.02:
+    overlap_map, overlap_model = sc.calculate_overlap_scores(emmap1,emmap2,c1,c2)
+    print 'Fraction of map overlapping with model: ', overlap_map
+    print 'Fraction of model overlapping with map: ', overlap_model
+    if overlap_model < 0.02:
         print "Maps do not overlap: ", Name2
         ccc_mask = ccc = -1.0
         mi_mask = mi = 0.0
     else:
+        #CCC
+        try:
+            ccc_mask,ovr = sc.CCC_map(emmap1,emmap2,c1,c2,3,meanDist=True)
+            
+            if ovr < 0.0: ovr = 0.0
+        except:
+            print 'ccc and overlap score calculation failed for', Name2
+            ovr = 0.0
         #SCCC
         print 'Local correlation score: ', ccc_mask
         if ccc_mask < -1.0 or ccc_mask > 1.0:
@@ -189,8 +232,10 @@ def get_global_scores(emmap1,emmap2,c1,c2,dict_scores_hits,Name2):
     except KeyError: dict_scores_hits['local_correlation'] = [ccc_mask]
     try: dict_scores_hits['local_mi'].append(mi_mask)
     except KeyError: dict_scores_hits['local_mi'] = [mi_mask] 
-    try: dict_scores_hits['overlap'].append(ovr)
-    except KeyError: dict_scores_hits['overlap'] = [ovr]
+    try: dict_scores_hits['overlap_map'].append(overlap_map)
+    except KeyError: dict_scores_hits['overlap_map'] = [overlap_map]
+    try: dict_scores_hits['overlap_model'].append(overlap_model)
+    except KeyError: dict_scores_hits['overlap_model'] = [overlap_model]
     try: dict_scores_hits['correlation'].append(ccc)
     except KeyError: dict_scores_hits['correlation'] = [ccc]
     try: dict_scores_hits['mi'].append(mi)
@@ -205,5 +250,7 @@ if __name__ == '__main__':
     if len(sys.argv) <= 1:
         main(json_path='./test_data/unittest_args.json')
 #         print 'Please supply json parameter file'
+    elif len(sys.argv) > 2:
+        main(json_path=sys.argv[1], set_results=sys.argv[2])
     else:
         main(json_path=sys.argv[1])
