@@ -119,7 +119,7 @@ class CCPEMMainWindow(QtGui.QMainWindow):
         self.version = ccpem_utils.CCPEMVersion()
 
         # Get tasks
-        self.tasks = ccpem_tasks.CCPEMTasks()
+        self.tasks = ccpem_tasks.CCPEMTasks(alpha=self.args.alpha())
 
         # Set main layout
         widget = QtGui.QWidget()
@@ -332,15 +332,21 @@ class CCPEMMainWindow(QtGui.QMainWindow):
         self.task_buttons_layout.addWidget(ccpem_widgets.CCPEMMenuSeparator())
 
         # Tasks
+        self.add_task_button('AceDRG', self.tasks.acedrg)
         self.add_task_button('Buccaneer', self.tasks.buccaneer)
         self.add_task_button('Choyce', self.tasks.choyce)
         self.add_task_button('Confidence Maps', self.tasks.confidence_maps)
         self.add_button('Coot', 'Open Coot in the project directory', lambda: gui_process.run_coot(None))
+        self.add_task_button('cryoEF', self.tasks.cryoEF)
         self.add_task_button('DockEM', self.tasks.dock_em)
         self.add_task_button('Flex-EM', self.tasks.flex_em)
+        if self.args.alpha():
+            self.add_task_button('Haruspex', self.tasks.haruspex)
         self.add_task_button('LAFTER', self.tasks.lafter)
         self.add_task_button('LocScale', self.tasks.loc_scale)
         self.add_task_button('Map Process', self.tasks.map_process)
+        if self.args.alpha():
+            self.add_task_button('Model2Map', self.tasks.model2map)
         self.add_task_button('Molrep', self.tasks.molrep)
         self.add_task_button('Model tools', self.tasks.model_tools)
         self.add_task_button('MRC to MTZ', self.tasks.mrc_to_mtz)
