@@ -9,6 +9,7 @@
 
 import json
 import os
+from ccpem_core import ccpem_utils
 from ccpem_core import settings
 from PyQt4 import QtGui
 
@@ -108,11 +109,11 @@ class CCPEMProjectContainer(object):
         project_manager['projects'] = projects
         project_manager['active_project_id'] = self.active_project_id
         if self.filename != '':
-            json.dump(project_manager,
-                      fp=open(self.filename, 'w'),
-                      sort_keys=True,
-                      separators=(',', ': '),
-                      indent=4)
+            ccpem_utils.atomic_write_json(project_manager,
+                                          self.filename,
+                                          sort_keys=True,
+                                          separators=(',', ': '),
+                                          indent=4)
 
     def load(self):
         project_manager = json.load(fp=open(self.filename, 'r'))
