@@ -7,19 +7,17 @@
 #     CCP-EM Secretary, RAL Laboratory, Harwell, OX11 0FA, UK.
 #
 
-import inspect
 import shutil
 import subprocess
 import time
 
 import pyrvapi
 from PyQt4 import QtCore, QtGui, QtWebKit
-
+from ccpem_gui.utils import command_line_launch
 from ccpem_gui.utils import window_utils, gui_process
 from ccpem_core.tasks.atomic_model_validation import validate_task
 from ccpem_core.settings import which
 from ccpem_core.process_manager import process_utils
-from ccpem_gui.utils import command_line_launch
 from ccpem_gui.utils.pdb_utils import *
 from ccpem_core.tasks.atomic_model_validation import validate_results
 from ccpem_core import ccpem_utils
@@ -80,6 +78,22 @@ class ValidateWindow(window_utils.CCPEMTaskWindow):
             required=False,
             args=self.args)
         self.args_widget.args_layout.addWidget(self.map_resolution)
+        # -> Use Refmac for model-map conversion
+        self.use_refmac = window_utils.CheckArgInput(
+            parent=self,
+            arg_name='use_refmac',
+            args=self.args,
+            label_width=245)
+        self.args_widget.args_layout.addWidget(self.use_refmac)
+        self.use_refmac.value_line.stateChanged.connect(self.set_modelmap)
+        # Ligand library input for Refmac5
+        self.lib_input = window_utils.FileArgInput(
+            parent=self,
+            arg_name='lib_in',
+            args=self.args,
+            file_types=ccpem_file_types.lib_ext,
+            required=False)
+        self.args_widget.args_layout.addWidget(self.lib_input)
 
         self.half_map_1_input = window_utils.FileArgInput(
             parent=self,
@@ -100,15 +114,24 @@ class ValidateWindow(window_utils.CCPEMTaskWindow):
         self.args_widget.args_layout.addLayout(method_options_frame)
         self.set_methods_button()
         
+        # Map contour level
+        self.map_contour_level = window_utils.NumberArgInput(
+            parent=self,
+            arg_name='map_contour_level',
+            required=False,
+            args=self.args)
+        self.args_widget.args_layout.addWidget(self.map_contour_level)
         
         self.check_input_for_methods()
         #add to layout
         method_options_frame.add_extension_widget(self.run_molprobity)
         method_options_frame.add_extension_widget(self.run_cablam)
+        method_options_frame.add_extension_widget(self.run_scores)
         method_options_frame.add_extension_widget(self.run_smoc)
         method_options_frame.add_extension_widget(self.run_refmac)
         method_options_frame.add_extension_widget(self.run_jpred)
         
+        self.run_scores.value_line.stateChanged.connect(self.set_run_methods)
         self.run_smoc.value_line.stateChanged.connect(self.set_run_methods)
 #         self.run_jpred.value_line.stateChanged.connect(self.set_run_methods)
         self.map_input.value_line.editingFinished.connect(
@@ -132,6 +155,7 @@ class ValidateWindow(window_utils.CCPEMTaskWindow):
         self.args_widget.args_layout.addWidget(self.results_parallel)
         
         self.handle_run_results_in_parallel()
+        self.set_modelmap()
         
     def set_methods_button(self):
         self.run_molprobity = window_utils.CheckArgInput(
@@ -142,6 +166,10 @@ class ValidateWindow(window_utils.CCPEMTaskWindow):
             parent=self,
             arg_name='run_cablam',
             args=self.args)
+        self.run_scores = window_utils.CheckArgInput(
+            parent=self,
+            arg_name='run_scores',
+            args=self.args)
         self.run_smoc = window_utils.CheckArgInput(
             parent=self,
             arg_name='run_smoc',
@@ -162,6 +190,8 @@ class ValidateWindow(window_utils.CCPEMTaskWindow):
                                 "QCheckBox::indicator { height: 22px;}")       
         self.run_cablam.value_line.setStyleSheet(
                                 "QCheckBox::indicator { height: 22px;}")
+        self.run_scores.value_line.setStyleSheet(
+                                "QCheckBox::indicator { height: 22px;}")
         self.run_smoc.value_line.setStyleSheet(
                                 "QCheckBox::indicator { height: 22px;}")
         self.run_refmac.value_line.setStyleSheet(
@@ -169,19 +199,47 @@ class ValidateWindow(window_utils.CCPEMTaskWindow):
         self.run_jpred.value_line.setStyleSheet(
                                 "QCheckBox::indicator { height: 22px;}")
 
+    def set_modelmap(self):
+        if self.use_refmac.value_line.isChecked():
+            self.task.args.use_refmac.value = True
+            self.lib_input.show()
+        else:
+            self.task.args.use_refmac.value = False
+            self.lib_input.hide()
+
     def check_input_for_methods(self):
         if self.task.args.map_path.value is None or \
             self.task.args.map_resolution.value is None:
+            self.run_scores.setEnabled(False)
             self.run_smoc.setEnabled(False)
             self.run_refmac.setEnabled(False)
+            self.use_refmac.setEnabled(False)
+            self.task.args.use_refmac.value = False
+            self.task.args.map_contour_level.value = None
+            self.map_contour_level.hide()
         else:
+            self.run_scores.setEnabled(True)
             self.run_smoc.setEnabled(True)
             self.run_refmac.setEnabled(True)
+            self.use_refmac.setEnabled(True)
+            if self.use_refmac.value_line.isChecked():
+                self.task.args.use_refmac.value = True
+            if self.run_scores.value_line.isChecked():
+                self.map_contour_level.show()
         
     def set_run_methods(self):
         if self.run_smoc.value_line.isChecked():
             self.handle_run_tempy_smoc()
         else: self.task.args.run_smoc.value = False
+        if self.run_scores.value_line.isChecked():
+            self.handle_run_tempy_scores()
+            self.map_contour_level.show()
+            
+        else: 
+            self.task.args.run_scores.value = False
+            self.task.args.map_contour_level.value = None
+            self.map_contour_level.hide()
+            
         if self.run_jpred.value_line.isChecked():
             self.handle_run_jpred()
         else: self.task.args.run_jpred.value = False
@@ -198,7 +256,9 @@ class ValidateWindow(window_utils.CCPEMTaskWindow):
 
     def handle_run_tempy_smoc(self):
         self.task.args.run_smoc.value = True
-    
+    def handle_run_tempy_scores(self):
+        self.task.args.run_scores.value = True
+        
     def handle_run_results_in_parallel(self):
         if self.results_parallel.value_line.isChecked():
             self.task.args.results_parallel.value = True
@@ -218,18 +278,19 @@ class ValidateWindow(window_utils.CCPEMTaskWindow):
 #         self.smoc_dialog.setMinimumSize(600, 500)
 #         self.smoc_dialog.show()
 
-    def handle_toolbar_run(self):
+    def validate_input(self):
         '''
-        Over ride from baseclass to allow validation of multiple datasets
+        Override from baseclass to allow validation of multiple datasets
         '''
-        if self.check_input():
-            # Valdited multiple datasets
+        ready = super(ValidateWindow, self).validate_input()
+        if ready:
+            # Validate inputs for multiple datasets
             if not self.input_pdbs.validate_datasets():
-                return
-            else :
+                ready = False
+            else:
                 self.input_pdbs.set_args()
-        self.run_detached_process()
-        
+        return ready
+
 #     def set_on_job_running_custom(self):
 #         self.launcher.add_file(
 #             arg_name='pdb_path',
@@ -301,15 +362,19 @@ class ValidateWindow(window_utils.CCPEMTaskWindow):
     #check if only a few jobs failed
     def set_on_job_failed_custom(self):
         count_failed = 0
+        count_finish = 0
         count_jobs = 0
         for stage in self.task.pipeline.json_pl:
             for job in stage:
                 status = process_utils.get_process_status(json_filepath=job)
-                if status == 'finished':
+                if status == 'failed':
                     count_failed += 1
+                elif status == 'finished':
+                    count_finish += 1
                 count_jobs += 1
         
-        if count_failed > 0 and float(count_failed)/count_jobs <= 0.25:
+        #if count_failed > 0 and float(count_failed)/count_jobs <= 0.25:
+        if count_finish > 0:
             try: self.set_on_job_finish_custom()
             except AttributeError: pass
     
@@ -354,6 +419,7 @@ class ValidateWindow(window_utils.CCPEMTaskWindow):
 
         
     def set_on_job_finish_custom(self):
+        self.input_pdbs.disable()
         rv_index = os.path.join(
                 self.task.job_location,
                 'report/index.html')
@@ -521,6 +587,7 @@ class ValidateWindow(window_utils.CCPEMTaskWindow):
     #overide error log tabs 
     def check_error_logs(self):
         pass
+
 def main():
     '''
     Launch standalone task runner.
