@@ -45,7 +45,7 @@ class CCPEMTasks(object):
     '''
     Class of Tasks.  Relates program name to task function.
     '''
-    def __init__(self):
+    def __init__(self, alpha=False):
         '''
         Set up task dictionary
         '''
@@ -56,6 +56,18 @@ class CCPEMTasks(object):
         if ccp4_flag_errors is not None: 
             self.errors['CCP4 flags'] = ccp4_flag_errors
 
+        # AceDRG
+        self.acedrg = None
+        try:
+            from ccpem_core.tasks.acedrg import acedrg_task
+            from ccpem_gui.tasks.acedrg import acedrg_window
+            self.acedrg = CCPEMTaskWrapper(
+                name=acedrg_task.AcedrgTask.task_info.name,
+                task=acedrg_task.AcedrgTask,
+                window=acedrg_window.AcedrgWindow)
+        except Exception as e:
+            self.errors['AceDRG'] = e
+
         # Buccaneer
         self.buccaneer = None
         try:
@@ -93,6 +105,18 @@ class CCPEMTasks(object):
         except Exception as e:
             self.errors['Confidence Maps'] = e
 
+        # cryoEF
+        self.cryoEF = None
+        try:
+            from ccpem_core.tasks.cryoEF import cryoEF_task
+            from ccpem_gui.tasks.cryoEF import cryoEF_window
+            self.cryoEF = CCPEMTaskWrapper(
+                name=cryoEF_task.cryoEF.task_info.name,
+                task=cryoEF_task.cryoEF,
+                window=cryoEF_window.cryoEFWindow)
+        except Exception as e:
+            self.errors['cryoEF'] = e
+            
         # Dock-EM
         self.dock_em = None
         try:
@@ -118,6 +142,19 @@ class CCPEMTasks(object):
         except Exception as e:
             self.errors['Flex-EM'] = e
 
+        # Haruspex
+        if alpha:
+            self.haruspex = None
+            try:
+                from ccpem_core.tasks.haruspex import haruspex_task
+                from ccpem_gui.tasks.haruspex import haruspex_window
+                self.haruspex = CCPEMTaskWrapper(
+                    name=haruspex_task.Haruspex.task_info.name,
+                    task=haruspex_task.Haruspex,
+                    window=haruspex_window.HaruspexWindow)
+            except Exception as e:
+                self.errors['Haruspex'] = e
+
         # LAFTER
         self.lafter = None
         try:
@@ -141,7 +178,20 @@ class CCPEMTasks(object):
                 window=loc_scale_window.LocScaleWindow)
         except Exception as e:
             self.errors['LocScale'] = e
-
+        
+        #Model2Map
+        if alpha:
+            self.model2map = None
+            try:
+                from ccpem_core.tasks.model2map import model2map_task
+                from ccpem_gui.tasks.model2map import model2map_window
+                self.model2map = CCPEMTaskWrapper(
+                    name=model2map_task.Model2Map.task_info.name,
+                    task=model2map_task.Model2Map,
+                    window=model2map_window.Model2MapWindow)
+            except Exception as e:
+                self.errors['Model2Map'] = e
+            
         # MapProcess
         self.map_process = None
         try:
