@@ -21,7 +21,7 @@ from ccpem_core.test_data.tasks import map_process as test_data
 list_methods = ['shiftpeak_to_zero','pad','lowpass_filter','crop',
                 'threshold','mask','dust_filter','downsample','softmask',
                 'shift_origin']
-list_methods_noargs = ['shiftpeak_to_zero','softmask']
+list_methods_noargs = ['shiftpeak_to_zero','threshold','crop','dust_filter','softmask',]
 
 dict_tooltips = {
     'shiftpeak_to_zero': 'Move background peak to zero',
@@ -47,20 +47,95 @@ class MapProcessWindow(window_utils.CCPEMTaskWindow):
                  parent=None):
         super(MapProcessWindow, self).__init__(task=task,
                                                   parent=parent)
+    
+    def set_args(self):
+        '''
+        Set input arguments
+        '''
+        # Input map
+        self.map_input = window_utils.FileArgInput(
+            parent=self,
+            arg_name='map_path',
+            required=True,
+            args=self.args)
+        self.args_widget.args_layout.addWidget(self.map_input)
 
+        self.map_contour = window_utils.NumberArgInput(
+            parent=self,
+            arg_name='map_contour',
+            decimals=5,
+            args=self.args,
+            required=False)
+        self.args_widget.args_layout.addWidget(self.map_contour)
+        
+        # Create ListWidget
+        self.list_widget = QtGui.QListWidget()
+        total_height = 0
+        self.dict_method_buttons = {}
+        self.list_selected_methods = []
+        for i in range(len(list_methods)):
+            method_name = list_methods[i]
+            item = QtGui.QListWidgetItem() 
+            itemwidget = QtGui.QWidget()
+            # Method
+            if method_name in list_methods_noargs:
+                options_frame = QtGui.QVBoxLayout()
+                button = QtGui.QRadioButton(str(method_name))
+                button.setToolTip(dict_tooltips[method_name])
+                options_frame.addWidget(button)
+                self.dict_method_buttons[method_name] = button
+            else:
+                options_frame = window_utils.CCPEMExtensionFrame(
+                    button_name=str(method_name),
+                    button_tooltip=dict_tooltips[method_name])
+                self.dict_method_buttons[method_name] = options_frame.button
+                options_frame.setAlignment(QtCore.Qt.AlignTop)
+            
+            self.dict_method_buttons[method_name].clicked.connect(
+                                lambda checked: self.handle_button(
+                                                        checked))
+            #options_frame.setSizeConstraint(QtGui.QLayout.SetFixedSize)
+            #add method args
+            self.add_method_args(options_frame, method_name)
+            #if options_frame.button.isChecked():
+            itemwidget.setLayout(options_frame)
+            s = itemwidget.sizeHint()
+            if not method_name in list_methods_noargs:
+                item_height = s.height()+options_frame.frame.sizeHint().height()
+                s.setHeight(item_height)
+            total_height += s.height()
+            item.setSizeHint(s)
+            item.setWhatsThis(method_name)
+            self.list_widget.addItem(item)
+            self.list_widget.setItemWidget(item, itemwidget)
+        # Enable drag & drop ordering of items.
+        self.list_widget.setDragDropMode(QtGui.QAbstractItemView.InternalMove)
+        self.list_widget.model().layoutChanged.connect(self.check_method_order)
+        # Make sure list widget is big enough for all items
+        self.list_widget.setMinimumHeight(int(total_height*0.5))
+        #self.list_widget.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
+        self.list_widget.setSizePolicy(
+            QtGui.QSizePolicy.Expanding, 
+            QtGui.QSizePolicy.Expanding)
+        self.args_widget.args_layout.addWidget(self.list_widget)
+        #edit tooltip for setup window
+        self.setup_dock.setToolTip('Drag and drop to rearrange \n Select to choose operation')
+        #set initial states
+        if self.task.args.list_process.value != None:
+            for method_name in self.task.args.list_process.value:
+                if method_name in self.dict_method_buttons:
+                    method_button = self.dict_method_buttons[method_name]
+                    method_button.setChecked(True)
+        
     def add_method_args(self,options_frame, method_name):
+        self.contour_added = False
         if method_name == 'lowpass_filter':
             self.map_resolution = window_utils.NumberArgInput(
                 parent=self,
                 arg_name='map_resolution',
+                decimals=3,
                 args=self.args)
             options_frame.add_extension_widget(self.map_resolution)
-        elif method_name in ['threshold','crop','dust_filter']:
-            self.map_contour = window_utils.NumberArgInput(
-                parent=self,
-                arg_name='map_contour',
-                args=self.args)
-            options_frame.add_extension_widget(self.map_contour)
         elif method_name == 'mask':
             self.mask_path = window_utils.FileArgInput(
                 parent=self,
@@ -85,73 +160,34 @@ class MapProcessWindow(window_utils.CCPEMTaskWindow):
             self.map_newapix = window_utils.NumberArgInput(
                 parent=self,
                 arg_name='map_apix',
+                decimals=3,
                 args=self.args)
             options_frame.add_extension_widget(self.map_newapix)
-    
-    def set_args(self):
-        '''
-        Set input arguments
-        '''
-        # Input map
-        self.map_input = window_utils.FileArgInput(
-            parent=self,
-            arg_name='map_path',
-            required=True,
-            args=self.args)
-        self.args_widget.args_layout.addWidget(self.map_input)
 
-        # Create ListWidget
-        self.list_widget = QtGui.QListWidget()
-        total_height = 0
-        self.dict_method_buttons = {}
-        for i in range(len(list_methods)):
-            method_name = list_methods[i]
-            item = QtGui.QListWidgetItem() 
-            itemwidget = QtGui.QWidget()
-            # Method
-            if method_name in list_methods_noargs:
-                options_frame = QtGui.QVBoxLayout()
-                button = QtGui.QRadioButton(str(method_name))
-                button.setToolTip(dict_tooltips[method_name])
-                options_frame.addWidget(button)
-                self.dict_method_buttons[method_name] = button
-            else:
-                options_frame = window_utils.CCPEMExtensionFrame(
-                    button_name=str(method_name),
-                    button_tooltip=dict_tooltips[method_name])
-                self.dict_method_buttons[method_name] = options_frame.button
-                options_frame.setAlignment(QtCore.Qt.AlignTop)
-            #options_frame.setSizeConstraint(QtGui.QLayout.SetFixedSize)
-            #add method args
-            self.add_method_args(options_frame, method_name)
-            #if options_frame.button.isChecked():
-            itemwidget.setLayout(options_frame)
-            s = itemwidget.sizeHint()
-            if not method_name in list_methods_noargs:
-                item_height = s.height()+options_frame.frame.sizeHint().height()
-                s.setHeight(item_height)
-            total_height += s.height()
-            item.setSizeHint(s)
-            item.setWhatsThis(method_name)
-            self.list_widget.addItem(item)
-            self.list_widget.setItemWidget(item, itemwidget)
-        # Enable drag & drop ordering of items.
-        self.list_widget.setDragDropMode(QtGui.QAbstractItemView.InternalMove)
 
-        # Make sure list widget is big enough for all items
-        self.list_widget.setMinimumHeight(total_height)
-        self.list_widget.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
-        self.list_widget.setSizePolicy(
-            QtGui.QSizePolicy.Expanding, 
-            QtGui.QSizePolicy.Expanding)
-        self.args_widget.args_layout.addWidget(self.list_widget)
-    
-    def handle_toolbar_run(self):
-        self.check_method_order()
-        super(MapProcessWindow, self).handle_toolbar_run()
+    def handle_button(self,checked):
+        sender_button = self.sender()
+        method_name = str(sender_button.text())
+        if checked:
+            if not method_name in self.list_selected_methods:
+                self.list_selected_methods.append(method_name)
+            self.task.args.list_process.value = self.list_selected_methods
+            
+            if method_name in ['threshold','crop','dust_filter']:
+                self.map_contour.set_required(True)
+        else:
+            if method_name in self.list_selected_methods:
+                self.list_selected_methods.remove(method_name)
+            self.task.args.list_process.value = self.list_selected_methods
+            if method_name in ['threshold','crop','dust_filter']:
+                if 'threshold' not in self.list_selected_methods and \
+                'crop' not in self.list_selected_methods and \
+                'dust_filter' not in self.list_selected_methods:
+                    self.map_contour.set_required(False)
+            
         
+    
     def check_method_order(self):
-        self.list_selected_methods = []
         for i in range(self.list_widget.count()):
             list_item = self.list_widget.item(i)
             item_str = str(list_item.whatsThis())
@@ -160,7 +196,7 @@ class MapProcessWindow(window_utils.CCPEMTaskWindow):
                 if button.isChecked(): 
                     self.list_selected_methods.append(item_str)
             except: continue
-        self.args.list_process.value = self.list_selected_methods
+        self.task.args.list_process.value = self.list_selected_methods
         
     def set_on_job_finish_custom(self):
         if os.path.isfile(self.args.map_path.value):
