@@ -114,7 +114,7 @@ class ProShadeStructureInput ( QtGui.QWidget ):
         
         # Path viewer
         self.str_value                        = QtGui.QLineEdit ( )
-        self.str_value.editingFinished.connect ( self.edit_finised_str )
+        self.str_value.editingFinished.connect (self.edit_finished_str)
         self.str_value.setSizePolicy          ( QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding )
         str_box.addWidget                     ( self.str_value, stretch = True )
         str_box.addStretch                    ( 0 )
@@ -126,7 +126,7 @@ class ProShadeStructureInput ( QtGui.QWidget ):
         path = self.get_file_from_brower      ( )
         self.set_str_file                     ( path = path )
     
-    def edit_finised_str ( self ):
+    def edit_finished_str (self):
         path                                  = self.str_value.text ( )
         self.set_str_file                     ( path = path )
         self.set_shading                      ( self.str_label, shade = False )
@@ -620,15 +620,16 @@ class ProShadeWindow ( window_utils.CCPEMTaskWindow ):
             self.iqrsPerPeak_input.show       ( )
             self.peakAxisTolerance_input.hide ( )
 
-    def handle_toolbar_run(self):
+    def validate_input(self):
         '''
-        Over ride from baseclass to allow validation of multiple datasets
+        Override from baseclass to handle non-automatically updated values
         '''
         # Assign value for the non-automatically updated values
         self.structures_input_dists.set_args ( strs_arg = self.args.input_str )
-        
-        # Run the process
-        self.run_detached_process ( )
+        # TODO: should call super().validate_input() here to check other inputs
+        #  but currently this causes some running modes to fail to launch even
+        #  when the visible inputs are all correct
+        return True
 
     def run_coot_custom ( self ):
         # Initialise
