@@ -98,7 +98,7 @@ class PDBInput(QtGui.QWidget):
         pdb_select.clicked.connect(self.get_pdb_file)
         pdb_box.addWidget(pdb_select)
         self.pdb_value = QtGui.QLineEdit()
-        self.pdb_value.editingFinished.connect(self.edit_finised_pdb)
+        self.pdb_value.editingFinished.connect(self.edit_finished_pdb)
         pdb_box.addWidget(self.pdb_value)
 
         width_ar = 30
@@ -144,8 +144,8 @@ class PDBInput(QtGui.QWidget):
         if path is not None:
             self.set_pdb_file(path=path)
 
-    def edit_finised_pdb(self):
-        path = self.pdb_value.text()
+    def edit_finished_pdb(self):
+        path = str(self.pdb_value.text())
         self.set_pdb_file(path=path)
 
     def set_pdb_file(self, path):
@@ -463,25 +463,26 @@ class ProSMARTWindow(window_utils.CCPEMTaskWindow):
             self.results_dock.show()
             self.results_dock.raise_()
 
-    def handle_toolbar_run(self):
+    def validate_input(self):
         '''
-        Over ride from baseclass to allow validation of multiple datasets
+        Override from baseclass to allow validation of multiple datasets
         '''
         # Remove reference pdbs if not used
         if self.args.mode() != 'Reference model':
             self.args.ref_pdbs.value = None
-        if self.check_input():
-            # Valdited multiple datasets
+        ready = super(ProSMARTWindow, self).validate_input()
+        if ready:
+            # Validate inputs for multiple datasets
             if not self.target_pdbs.validate_datasets():
-                return
-            else :
+                ready = False
+            else:
                 self.target_pdbs.set_args()
             if self.ref_pdbs is not None:
                 if not self.ref_pdbs.validate_datasets():
-                    return
-                else :
+                    ready = False
+                else:
                     self.ref_pdbs.set_args()
-        self.run_detached_process()
+        return ready
 
     def set_on_job_running_custom(self):
         if hasattr(self, 'tb_chimera_button'):
