@@ -107,7 +107,7 @@ class RefmacDatasetInput(QtGui.QWidget):
         pdb_select.clicked.connect(self.get_pdb_file)
         pdb_box.addWidget(pdb_select)
         self.pdb_value = QtGui.QLineEdit()
-        self.pdb_value.editingFinished.connect(self.edit_finised_pdb)
+        self.pdb_value.editingFinished.connect(self.edit_finished_pdb)
         pdb_box.addWidget(self.pdb_value)
         self.layout.addLayout(pdb_box)
 
@@ -162,17 +162,17 @@ class RefmacDatasetInput(QtGui.QWidget):
         path = self.get_file_from_brower(file_types=ccpem_file_types.pdb_ext)
         self.set_pdb_file(path=path)
 
-    def edit_finised_pdb(self):
-        path = self.pdb_value.text()
+    def edit_finished_pdb(self):
+        path = str(self.pdb_value.text())
         self.set_pdb_file(path=path)
 
     def get_map_file(self):
         path = self.get_file_from_brower(file_types=ccpem_file_types.mrc_ext)
         self.set_map_file(path=path)
 
-    def edit_finised_map(self):
+    def edit_finished_map(self):
         path = self.map_value.text()
-        self.set_pdb_file(path=path)
+        self.set_map_file(path=path)
 
     def set_pdb_file(self, path):
         self.set_shading(self.pdb_label, shade=True)
@@ -980,6 +980,7 @@ class Refmac5Window(window_utils.CCPEMTaskWindow):
             self.numberCPUsOrNodes = window_utils.NumberArgInput(
                 parent=self,
                 arg_name='numberCPUsOrNodes',
+                minimum=1,
                 args=self.args)
             self.args_widget.args_layout.insertWidget(5, self.numberCPUsOrNodes)
 
@@ -1009,7 +1010,7 @@ class Refmac5Window(window_utils.CCPEMTaskWindow):
                 parent=self,
                 arg_name='start_pdb',
                 args=self.args,
-                label='Input PDB',
+                label='Input model',
                 file_types=ccpem_file_types.pdb_ext,
                 required=True)
             self.args_widget.args_layout.insertWidget(2, self.pdb_input)
@@ -1069,31 +1070,29 @@ class Refmac5Window(window_utils.CCPEMTaskWindow):
             self.results_dock.show()
             self.results_dock.raise_()
 
-    def handle_toolbar_run(self):
+    def validate_input(self):
         '''
-        Over ride from baseclass to allow validation of multiple datasets
+        Override from baseclass to allow validation of multiple datasets
         '''
-        if self.check_input():
-            if self.args.dc_mode():
-                # Valdited multiple datasets
-                if not self.datasets_input.validate_datasets():
-                    return
-                else :
-                    self.datasets_input.set_args(
-                        pdbs_arg=self.args.start_pdb,
-                        maps_arg=self.args.input_map,
-                        resolutions_arg=self.args.resolution)
-
-            if self.args.external_restraints_on.value:
-                if not self.restraints.validate_datasets():
-                    return
-                else:
-                    self.restraints.set_args(
-                      restraints_files_arg=self.args.restraints_files,
-                      external_weight_scales_arg=self.args.external_weight_scales,
-                      external_weight_gmwts_arg=self.args.external_weight_gmwts)
-
-            self.run_detached_process()
+        ready = super(Refmac5Window, self).validate_input()
+        if ready and self.args.dc_mode():
+            # Validate inputs for multiple datasets
+            if not self.datasets_input.validate_datasets():
+                ready = False
+            else:
+                self.datasets_input.set_args(
+                    pdbs_arg=self.args.start_pdb,
+                    maps_arg=self.args.input_map,
+                    resolutions_arg=self.args.resolution)
+        if ready and self.args.external_restraints_on.value:
+            if not self.restraints.validate_datasets():
+                ready = False
+            else:
+                self.restraints.set_args(
+                  restraints_files_arg=self.args.restraints_files,
+                  external_weight_scales_arg=self.args.external_weight_scales,
+                  external_weight_gmwts_arg=self.args.external_weight_gmwts)
+        return ready
 
 
     def set_on_job_running_custom(self):
@@ -1119,6 +1118,12 @@ class Refmac5Window(window_utils.CCPEMTaskWindow):
         '''
         # Refined model
         if hasattr(self.task, 'process_refine'):
+            if os.path.isfile(self.task.process_refine.cifout_path):
+                self.launcher.add_file(
+                    path=self.task.process_refine.cifout_path,
+                    file_type='pdb',
+                    description='Refined mmCIF file',
+                    selected=False)
             self.launcher.add_file(
                 path=self.task.process_refine.pdbout_path,
                 file_type='pdb',
