@@ -512,9 +512,12 @@ class TemPyMainWindow(window_utils.CCPEMTaskWindow):
                     str(self.args.atomic_model.value))
             
         arg_list_chimera = ["--script", "%s" % (chimera_command)]
-        self.chimera_bin_command = which('chimera')
         process = QtCore.QProcess()
-        process.startDetached(self.chimera_bin_command, arg_list_chimera)
+        chimera_bin = which('chimera')
+        if chimera_bin is None:
+            print 'Chimera executable not found (add executable to system PATH)'
+        else:
+            process.startDetached(chimera_bin, arg_list_chimera)
            
     def set_output_ui(self):
         '''
