@@ -67,12 +67,12 @@ class PDBInput(QtGui.QWidget):
         self.pdb_label = QtGui.QLabel(label)
         self.pdb_label.setFixedWidth(label_width)
         pdb_box.addWidget(self.pdb_label)
-        pdb_select = QtGui.QPushButton('Select')
-        pdb_select.setFixedWidth(button_width)
-        pdb_select.clicked.connect(self.get_pdb_file)
-        pdb_box.addWidget(pdb_select)
+        self.pdb_select = QtGui.QPushButton('Select')
+        self.pdb_select.setFixedWidth(button_width)
+        self.pdb_select.clicked.connect(self.get_pdb_file)
+        pdb_box.addWidget(self.pdb_select)
         self.pdb_value = QtGui.QLineEdit()
-        self.pdb_value.editingFinished.connect(self.edit_finised_pdb)
+        self.pdb_value.editingFinished.connect(self.edit_finished_pdb)
         pdb_box.addWidget(self.pdb_value)
 
         width_ar = 30
@@ -99,8 +99,8 @@ class PDBInput(QtGui.QWidget):
         if path is not None:
             self.set_pdb_file(path=path)
 
-    def edit_finised_pdb(self):
-        path = self.pdb_value.text()
+    def edit_finished_pdb(self):
+        path = str(self.pdb_value.text())
         self.set_pdb_file(path=path)
 
     def set_pdb_file(self, path):
@@ -151,7 +151,10 @@ class PDBInput(QtGui.QWidget):
     def add(self):
         if hasattr(self.parent, 'add_dataset'):
             self.parent.add_dataset()
-
+    def disable(self):
+        self.pdb_value.setReadOnly(True)
+        self.pdb_select.setEnabled(False)
+        
 class MultiPDBInput(QtGui.QWidget):
     def __init__(self,
                  group_label=None,
@@ -240,6 +243,15 @@ class MultiPDBInput(QtGui.QWidget):
         else:
             child = children[0]
             self.pdbs_arg.value = child.dataset.pdb_path
+    
+    def disable(self):
+        index = self.layout.count()
+        for l in xrange(index):
+            PDBWidget = self.layout.itemAt(l).widget()
+            try: 
+                PDBWidget.disable()
+            except AttributeError:
+                pass
 
 
 class GlobScoreWindow(window_utils.CCPEMTaskWindow):
@@ -305,7 +317,14 @@ class GlobScoreWindow(window_utils.CCPEMTaskWindow):
             chains_arg=self.args.input_pdb_chains,
             parent=self)
         self.args_widget.args_layout.addWidget(self.input_pdbs)
-        
+        # -> Use Refmac for model-map conversion
+        self.use_refmac = window_utils.CheckArgInput(
+            parent=self,
+            arg_name='use_refmac',
+            args=self.args,
+            label_width=245)
+        self.args_widget.args_layout.addWidget(self.use_refmac)
+        self.use_refmac.value_line.stateChanged.connect(self.set_modelmap)
         # Advanced options
         advanced_options_frame = window_utils.CCPEMExtensionFrame(
             button_name='Advanced options',
@@ -326,17 +345,21 @@ class GlobScoreWindow(window_utils.CCPEMTaskWindow):
         self.input_contour_level = window_utils.NumberArgInput(
             parent=self,
             arg_name='map_contour_level',
-            required=True,
+            required=False,
             args=self.args)
         advanced_options_frame.add_extension_widget(self.input_contour_level)
+        self.title_input.value_line.editingFinished.connect(
+            self.set_auto_contour)
         self.set_auto_contour()
 
     def set_auto_contour(self):
-        if self.args.auto_contour_level():
+        if self.args.auto_contour_level.value:
             self.input_contour_level.hide()
+            self.task.args.map_contour_level.value = self.args.map_contour_level.value = 0.0
         else:
             self.input_contour_level.show()
-    
+            self.task.args.map_contour_level.value = self.args.map_contour_level.value
+        
     def add_mapprocess_button(self):
         if os.path.isfile(self.args.map_path.value):
             self.mp_button.setEnabled(True)
@@ -370,6 +393,12 @@ class GlobScoreWindow(window_utils.CCPEMTaskWindow):
         window.show()
         self.task.mapprocess_task = task
     
+    def set_modelmap(self):
+        if self.use_refmac.value_line.isChecked():
+            self.task.args.use_refmac.value = True
+        else:
+            self.task.args.use_refmac.value = False
+
     def check_processed_map(self):
         processed_map_path = os.path.splitext(
                             os.path.abspath(self.args.map_path.value))[0] \
@@ -464,19 +493,21 @@ class GlobScoreWindow(window_utils.CCPEMTaskWindow):
             self.results_dock.show()
             self.results_dock.raise_()
 
-    def handle_toolbar_run(self):
+    def validate_input(self):
         '''
-        Over ride from baseclass to allow validation of multiple datasets
+        Override from baseclass to allow validation of multiple datasets
         '''
-        if self.check_input():
-            # Valdited multiple datasets
+        ready = super(GlobScoreWindow, self).validate_input()
+        if ready:
+            # Validate inputs for multiple datasets
             if not self.input_pdbs.validate_datasets():
-                return
-            else :
+                ready = False
+            else:
                 self.input_pdbs.set_args()
-        self.run_detached_process()
+        return ready
 
     def set_on_job_finish_custom(self):
+        self.input_pdbs.disable()
         # Add pdbs to launcher
         pdbs = self.task.args.input_pdbs()
         if not isinstance(pdbs, list):
@@ -495,7 +526,7 @@ class GlobScoreWindow(window_utils.CCPEMTaskWindow):
             selected=True)
         # Add raw data
         self.rawdata_path = os.path.join(self.task.job_location,
-                                         'global_scores.txt')
+                                         'global_scores.csv')
         if os.path.exists(self.rawdata_path):
             self.launcher.add_file(
                 description='Raw scores (CSV text)',
