@@ -67,12 +67,12 @@ class PDBInput(QtGui.QWidget):
         self.pdb_label = QtGui.QLabel(label)
         self.pdb_label.setFixedWidth(label_width)
         pdb_box.addWidget(self.pdb_label)
-        pdb_select = QtGui.QPushButton('Select')
-        pdb_select.setFixedWidth(button_width)
-        pdb_select.clicked.connect(self.get_pdb_file)
-        pdb_box.addWidget(pdb_select)
+        self.pdb_select = QtGui.QPushButton('Select')
+        self.pdb_select.setFixedWidth(button_width)
+        self.pdb_select.clicked.connect(self.get_pdb_file)
+        pdb_box.addWidget(self.pdb_select)
         self.pdb_value = QtGui.QLineEdit()
-        self.pdb_value.editingFinished.connect(self.edit_finised_pdb)
+        self.pdb_value.editingFinished.connect(self.edit_finished_pdb)
         pdb_box.addWidget(self.pdb_value)
 
         width_ar = 30
@@ -99,8 +99,8 @@ class PDBInput(QtGui.QWidget):
         if path is not None:
             self.set_pdb_file(path=path)
 
-    def edit_finised_pdb(self):
-        path = self.pdb_value.text()
+    def edit_finished_pdb(self):
+        path = str(self.pdb_value.text())
         self.set_pdb_file(path=path)
 
     def set_pdb_file(self, path):
@@ -151,7 +151,11 @@ class PDBInput(QtGui.QWidget):
     def add(self):
         if hasattr(self.parent, 'add_dataset'):
             self.parent.add_dataset()
-
+    
+    def disable(self):
+        self.pdb_value.setReadOnly(True)
+        self.pdb_select.setEnabled(False)
+        
 class MultiPDBInput(QtGui.QWidget):
     def __init__(self,
                  group_label=None,
@@ -188,7 +192,16 @@ class MultiPDBInput(QtGui.QWidget):
         else:
             self.add_dataset(
                 pdb_path=pdbs)
-
+            
+    def disable(self):
+        index = self.layout.count()
+        for l in xrange(index):
+            PDBWidget = self.layout.itemAt(l).widget()
+            try: 
+                PDBWidget.disable()
+            except AttributeError:
+                pass
+            
     def handle_add_button(self):
         '''
         Handle dataset button
@@ -246,7 +259,7 @@ class SMOCMapWindow(window_utils.CCPEMTaskWindow):
     '''
     SMOC window.
     '''
-    gui_test_args = get_test_data_path(test_data, 'unittest_args.json')
+    gui_test_args = get_test_data_path(test_data, 'unittest_args_smoc.json')
 
     def __init__(self,
                  task,
@@ -333,6 +346,7 @@ class SMOCMapWindow(window_utils.CCPEMTaskWindow):
         #add to layout
         method_options_frame.add_extension_widget(self.use_smoc_method)
         method_options_frame.add_extension_widget(self.use_sccc_method)
+        #set SMOC/SCCC based on resolution
         self.map_resolution.value_line.editingFinished.connect(self.select_metric)
 #         self.map_resolution.value_line.editingFinished.connect(self.check_processed_map)
         # Input PDBs
@@ -342,7 +356,22 @@ class SMOCMapWindow(window_utils.CCPEMTaskWindow):
             chains_arg=self.args.input_pdb_chains,
             parent=self)
         self.args_widget.args_layout.addWidget(self.input_pdbs)
-
+#                 # -> Use Refmac for model-map conversion
+#         self.use_refmac = window_utils.CheckArgInput(
+#             parent=self,
+#             arg_name='use_refmac',
+#             args=self.args,
+#             label_width=245)
+#         self.args_widget.args_layout.addWidget(self.use_refmac)
+#         self.use_refmac.value_line.stateChanged.connect(self.set_modelmap)
+#         # Ligand library input for Refmac5
+#         self.lib_input = window_utils.FileArgInput(
+#             parent=self,
+#             arg_name='lib_in',
+#             args=self.args,
+#             file_types=ccpem_file_types.lib_ext,
+#             required=False)
+#         self.args_widget.args_layout.addWidget(self.lib_input)
         # Input rigid body file
         self.input_rigid_body = window_utils.FileArgInput(
             parent=self,
@@ -424,7 +453,12 @@ class SMOCMapWindow(window_utils.CCPEMTaskWindow):
             #self.auto_distance.hide()
             self.input_fragament_length.hide()
             #self.local_distance.hide()
-        
+    
+    def set_modelmap(self):
+        if self.use_refmac.value_line.isChecked():
+            self.task.args.use_refmac.value = True
+        else:
+            self.task.args.use_refmac.value = False
 
     def set_fragment_length_auto(self):
         if self.args.auto_fragment_length():
@@ -473,7 +507,7 @@ class SMOCMapWindow(window_utils.CCPEMTaskWindow):
         #print self.task.processed_map_path
     def check_processed_map(self):
         processed_map_path = os.path.splitext(
-                            os.path.abspath(self.args.map_path.value))[0] \
+                            os.path.basename(self.args.map_path.value))[0] \
                             +'_processed.mrc'
         #print processed_map_path
         if os.path.isfile(processed_map_path) and \
@@ -512,7 +546,7 @@ class SMOCMapWindow(window_utils.CCPEMTaskWindow):
         list_pdbs = []
         for pdb in pdbs:
             scored_pdb = os.path.join(self.task.job_location,
-                        os.path.basename(str(pdb)).split('.')[0]+'_sc.pdb')
+                        os.path.basename(str(pdb)).split('.')[0]+'_smoc.pdb')
             if os.path.isfile(scored_pdb):
                 list_pdbs.append(scored_pdb)
                 
@@ -587,19 +621,21 @@ class SMOCMapWindow(window_utils.CCPEMTaskWindow):
             self.results_dock.show()
             self.results_dock.raise_()
 
-    def handle_toolbar_run(self):
+    def validate_input(self):
         '''
-        Over ride from baseclass to allow validation of multiple datasets
+        Override from baseclass to allow validation of multiple datasets
         '''
-        if self.check_input():
-            # Valdited multiple datasets
+        ready = super(SMOCMapWindow, self).validate_input()
+        if ready:
+            # Validate inputs for multiple datasets
             if not self.input_pdbs.validate_datasets():
-                return
-            else :
+                ready = False
+            else:
                 self.input_pdbs.set_args()
-        self.run_detached_process()
+        return ready
 
     def set_on_job_finish_custom(self):
+        self.input_pdbs.disable()
         # Add pdbs to launcher
         pdbs = self.task.args.input_pdbs()
         if not isinstance(pdbs, list):
@@ -619,7 +655,7 @@ class SMOCMapWindow(window_utils.CCPEMTaskWindow):
         
         for pdb in pdbs:
             scored_pdb = os.path.join(self.task.job_location,
-                                      os.path.basename(pdb).split('.')[0]+'_sc.pdb')
+                                      os.path.basename(pdb).split('.')[0]+'_smoc.pdb')
             if os.path.isfile(scored_pdb):
                 self.launcher.add_file(
                     path=scored_pdb,
@@ -628,14 +664,14 @@ class SMOCMapWindow(window_utils.CCPEMTaskWindow):
                     selected=True)
         # Add raw data
         self.rawdata_path = os.path.join(self.task.job_location,
-                                         'smoc_score.txt')
+                                         'smoc_score.csv')
         if os.path.exists(self.rawdata_path):
             self.launcher.add_file(
                 description='Raw scores (CSV text)',
                 path=self.rawdata_path)
         else:
             self.rawdata_path = os.path.join(self.task.job_location,
-                                         'sccc_score.txt')
+                                         'sccc_score.csv')
             if os.path.exists(self.rawdata_path):
                 self.launcher.add_file(
                     description='Raw scores (CSV text)',
