@@ -9,13 +9,9 @@
 
 import unittest
 import time
+
 from ccpem_gui.tasks import ccpem_tasks
 from ccpem_core.tasks.molrep import molrep_task
-try:
-    import modeller
-    modeller_available = True
-except ImportError:
-    modeller_available = False
 
 
 class Test(unittest.TestCase):
@@ -26,7 +22,7 @@ class Test(unittest.TestCase):
     def test_tasks(self):
         # Get task and assert returns correct class
         time_s = time.time()
-        tasks = ccpem_tasks.CCPEMTasks()
+        tasks = ccpem_tasks.CCPEMTasks(alpha=True)
 
         # Test task initialisation
         time_t = time.time() - time_s
@@ -36,11 +32,27 @@ class Test(unittest.TestCase):
         task = tasks.get_task_class(program='Molrep')()
         assert isinstance(task, molrep_task.MolRep)
 
-    @unittest.skipIf(not modeller_available,
-                     'Modeller not found: skipping test tasks for errors')
     def test_task_errors(self):
-        # Check no errors in task loading
-        tasks = ccpem_tasks.CCPEMTasks()
+        # Check only expected errors in task loading
+        tasks = ccpem_tasks.CCPEMTasks(alpha=True)
+
+        # Check for known errors if modeller is missing
+        try:
+            import modeller
+        except ImportError:
+            choyce_err = tasks.errors.pop('Choyce')
+            assert "modeller" in str(choyce_err)
+            flexem_err = tasks.errors.pop('Flex-EM')
+            assert "modeller" in str(flexem_err)
+
+        # Check for known errors if tensorflow is missing
+        try:
+            import tensorflow
+        except ImportError:
+            haruspex_err = tasks.errors.pop('Haruspex')
+            assert "tensorflow" in str(haruspex_err)
+
+        # Any remaining errors indicate an unexpected problem
         if tasks.errors:
             raise AssertionError(tasks.errors)
 
