@@ -195,7 +195,7 @@ class CCPEMTaskWindow(QtGui.QMainWindow):
         raise NotImplementedError('Subclasses should implement this')
 
     def create_child(self):
-        self.set_all_inputs()
+        self.finish_all_edits()
         child_args = self.args.deep_copy()
         child_task = self.task.__class__(args=child_args,
                                          database_path=self.database_path)
@@ -377,6 +377,14 @@ class CCPEMTaskWindow(QtGui.QMainWindow):
                 'PyMOL',
                 self.run_pymol)
         #
+        if settings.which(program='ChimeraX') is not None:
+            pymol_icon = os.path.join(icon_utils.get_other_icons_path(),
+                                      'ChimeraX_32x32.png')
+            self.tb_pymol_button = self.tool_bar.addAction(
+                QtGui.QIcon(pymol_icon),
+                'ChimeraX',
+                self.run_chimera_x)
+        #
         self.tool_bar.addSeparator()
         #
         terminal_icon = os.path.join(icon_utils.get_icons_path(),
@@ -488,6 +496,17 @@ class CCPEMTaskWindow(QtGui.QMainWindow):
     def run_chimera_custom(self):
         return NotImplemented
 
+    def run_chimera_x(self):
+        '''
+        Run ChimeraX viewer.
+        '''
+        if self.run_chimera_x_custom() == NotImplemented:
+            self.launcher.launch_mg_viewers(mg_viewer='ChimeraX',
+                                            launch_mg_without_file=True)
+
+    def run_chimera_x_custom(self):
+        return NotImplemented
+
     def run_ccp4mg(self):
         '''
         Run ccp4mg viewer.
@@ -543,23 +562,32 @@ class CCPEMTaskWindow(QtGui.QMainWindow):
 
     def handle_toolbar_run(self):
         '''
-        Run task.
+        Check inputs are valid and then run task.
+
+        Do not override this simply for input validation. Override validate_input()
+        instead.
         '''
-        self.set_all_inputs()
-        if self.check_input():
+        self.finish_all_edits()
+        if self.validate_input():
             self.run_detached_process()
 
-    def set_all_inputs(self):
+    def finish_all_edits(self):
         '''
-        Set all input values.  Done explicity when a task is run or cloned
-        as editFinished signal can be by-passed in some circumstances.
+        Ensure any partial edits in input widgets are committed.
+
+        This is done explicitly when a task is run or cloned as the editFinished
+        signal is not sent if an edit is in progress when the toolbar button is
+        pressed.
         '''
         for child in self.findChildren(CCPEMArgBaseWidget):
             child.on_edit_finished()
 
-    def check_input(self):
+    def validate_input(self):
         '''
-        Check required inputs are set, display warning message if not.
+        Check that required inputs are set.
+
+        If inputs are invalid, returns False and displays a warning message.
+        If inputs are all valid, returns True.
         '''
         base_classes = self.findChildren(CCPEMArgBaseWidget)
         ready = True
