@@ -22,6 +22,8 @@ class opt_md:
     def __init__(self, path, code, run_num, rand, em_map_file, input_pdb_file,
                  format, apix, res, x, y, z, rigid_filename,
                  num_of_iter, cap_shift=0.39, phi_psi_retraints=True,
+                 template_distance_restraint=True,max_distance=5.0,
+                 dict_add_restraints={},density_weight=0.9,
                  test_mode=False):
 
         env = environ(rand_seed=-rand)
@@ -70,10 +72,10 @@ class opt_md:
         mdl.res_num_from(mdl2,aln) #added
         ##check number of chains
         nochain = False
-        if len(mdl.chains) == 1:
-            for c in mdl.chains:
-                c.name = ' '
-            nochain = True
+#         if len(mdl.chains) == 1:
+#             for c in mdl.chains:
+#                 c.name = ' '
+#             nochain = True
         mdl.write(file=code+'_ini.pdb')
         '''
         # renumber chains
@@ -87,18 +89,56 @@ class opt_md:
                             restraint_type='stereo',
                             spline_on_site=False)
 
-        # XXX Add as option
+        at = mdl.atoms
+        # Add as option
         if phi_psi_retraints:
+            print '>>Adding template phi/psi restraints'
             mdl.restraints.make(sel_all,
                                 aln=aln,
                                 restraint_type='phi-psi_binormal',
                                 spline_on_site=True,
                                 residue_span_range=(0, 99999))
-
+        if template_distance_restraint:
+            print '>>Adding template distance restraints'
+            mdl.restraints.make_distance(sel_all,
+                                         sel_all,
+                                aln=aln,
+                                restraint_group=physical.xy_distance,#Ca-Ca distance
+                                maximal_distance=max_distance,
+                                spline_on_site=False,
+                                residue_span_range=(0, 99999))
+        for kw in dict_add_restraints:
+            if kw == 'helix':
+                for res_seg in dict_add_restraints[kw]:
+                    print '++Adding helix restraint', res_seg
+                    try:
+                        mdl.restraints.add(
+                            secondary_structure.alpha(
+                                mdl.residue_range(str(res_seg[0]), str(res_seg[1]))))
+                    except IOError: pass
+            elif kw == 'strand':
+                for res_seg in dict_add_restraints[kw]:
+                    print '++Adding strand restraint', res_seg
+                    try:
+                        mdl.restraints.add(
+                            secondary_structure.strand(
+                                mdl.residue_range(str(res_seg[0]), str(res_seg[1]))))
+                    except IOError: pass
+            elif kw == 'distance':
+                for res_seg in dict_add_restraints[kw]:
+                    print '++Adding distance restraint', res_seg
+                    try:
+                        mdl.restraints.add(
+                            forms.gaussian(
+                                group=physical.xy_distance,
+                               feature=features.distance(at['CA:'+str(res_seg[0])],
+                                                         at['CA:'+str(res_seg[1])]),
+                               mean=res_seg[2], stdev=0.1))
+                    except IOError: pass
         # define rigid bodies
         sel_rigid = []
         rand_rigid = []
-        load_rigid(path, mdl, sel_rigid, rand_rigid, rigid_filename,nochain)
+        load_rigid('', mdl, sel_rigid, rand_rigid, rigid_filename,nochain)
         for n in sel_rigid:
             mdl.restraints.rigid_bodies.append(rigid_body(n))
 
@@ -124,7 +164,7 @@ class opt_md:
 
         # start simulated annealing molecular dynamics
         print "MD annealing"
-        scal = physical.values(default=1.0, em_density=10000)
+        scal = physical.values(default=1.0, em_density=int(10000*density_weight))
 
         cap = cap_shift
         timestep = 5.0
@@ -176,7 +216,7 @@ class opt_md:
                     scal = physical.values(default=0.0, em_density=1.0)
                     (molpdf, terms) = sel_all.energy(schedule_scale=scal)
                     print "iteration number= %s  step= %d %d  temp= %d  EM score= %.3f" %(a,icount,its,int(temp),-molpdf)
-                    scal = physical.values(default=1.0, em_density=10000)
+                    scal = physical.values(default=1.0, em_density=int(10000*density_weight))
                     icount = icount+equil_its
                 init_vel = False
 
@@ -204,19 +244,19 @@ class opt_md:
 
                     scal = physical.values(default=0.0, em_density=1.0)
                     (molpdf, terms) = sel_all.energy(schedule_scale=scal)
-                    print "step= %d %d   temp= %d    EM score= %.3f" %(icount,its,int(temp),-molpdf)
-                    scal = physical.values(default=1.0, em_density=10000)
+                    print "iteration number= %s step= %d %d   temp= %d    EM score= %.3f" %(a,icount,its,int(temp),-molpdf)
+                    scal = physical.values(default=1.0, em_density=int(10000*density_weight))
                     icount = icount+equil_its
 
-            filename = 'md'+run_num+'_'+str(a)+'.pdb'
-            sel_all.write(file=filename)
-
             #CG refinement
             scal = physical.values(default=1.0, em_density=100.0)
             CG.optimize(sel_all, output='REPORT', 
-                        max_iterations=50,
+                        max_iterations=100,
                         schedule_scale=scal)
 
+            filename = 'md'+run_num+'_'+str(a)+'.pdb'
+            sel_all.write(file=filename)
+
             # Get iteration energies
             scal = physical.values(default=0.0, em_density=1.0)
             (molpdf, terms) = sel_all.energy(schedule_scale=scal)
@@ -242,7 +282,7 @@ class opt_md:
         if test_mode:
             max_it = 5
         else:
-            max_it = 200
+            max_it = 100
 
         scal = physical.values(default=1.0, em_density=100.0)
         CG.optimize(sel_all,
