@@ -1,3 +1,25 @@
+  470  ls
+  471  pwd
+  GNU nano 2.0.9                                          File: ShapeM-XtBv1.3.f90                                                                                           
+
+!=******************************************************************************
+!=* ShapeM - Programs                                     AUTHOR: A.ROSEMAN    *
+!=*                                                                            *
+!=*                                                                            *
+!=* ShapeM is a set of software programs to match 3D density object shapes.    *
+!=* Specifically, it has been designed to locate known molecular shapes within *
+!=* molecular densities of larger complexes determined by cryoEM. It was       *
+  470  ls
+  471  pwd
+  GNU nano 2.0.9                                          File: ShapeM-XtBv1.3.f90                                                                                           
+
+!=******************************************************************************
+!=* ShapeM - Programs                                     AUTHOR: A.ROSEMAN    *
+!=*                                                                            *
+!=*                                                                            *
+!=* ShapeM is a set of software programs to match 3D density object shapes.    *
+!=* Specifically, it has been designed to locate known molecular shapes within *
+!=* molecular densities of larger complexes determined by cryoEM. It was       *
 !=******************************************************************************
 !=* ShapeM - Programs              		          AUTHOR: A.ROSEMAN    *
 !=*                                    		                               *
@@ -45,6 +67,9 @@
 ! mod to dock densities, therefore not inverse transformation
 !mod so no wrapparounds in density rotn
 
+!v1.2 origin to match V1.2 search density prog.
+!v1.3 extracted densities will match the sobj
+
 !C input: 
 !C       mrc density map file - fit object
 !c       mask file
@@ -52,12 +77,24 @@
 !C       run version key.
 !C       sampling of the map used for the search
 !C	 range of solutions to generate
+
 !C       size of extracted objects.  Keep at the target map size for convenience.
+! v1.2 no, now match extracgted objects to size of the seachobject/mask, by default.
 
 
-!C output: pdb files corresponding to searchdoc keys.
+
+!C output: mrc files corresponding to searchdoc keys.
 !C  
 
+!V1.2
+! window out boxes at centres given in searchdock files.
+! apply reverse rotatoins about the centres of these boxes.
+! these are the hits then aligned to the original search file.
+! note: make the masked ones and not masked ones. Will be useful. 
+
+!v1.3 make extracted object that are exactly aligned to the sobj.
+
+
 
 	program ShapeMXtB
 
@@ -67,7 +104,7 @@
         
         real sampling
         real psi,theta,phi
-        character*40 pdbfile,outfile2,filename,filename2,maskfilename
+        character*40 pdbfile,outfile2,filename,filename2,maskfilename,sobjfilename
         character*24 space1
  	character*6 head
 	character*10 space2
@@ -84,7 +121,7 @@
       	
 !C sampling for position search, in voxels      		
         integer  counter,counter2,zx,zy,zz,r1,r2,N
-	integer nx3,ny3,nz3
+	integer nx3,ny3,nz3,lx,ly,lz
 
 	integer size,sizeo2,sizem1,sx,fx,sy,fy,sz,fz
 	real, dimension (:,:,:), allocatable :: map, obj, rotobj,mask       
@@ -97,22 +134,29 @@
 	
 
       
-      write(6,*) 'ShapeM-XtB'
-      write(6,*) '=========='
+      write(6,*) 'ShapeM-XtB, v1.3d'
+      write(6,*) '=================='
     
       write(6,*)
 
+	print*,'Enter the EM map filename.'
+	read*,filename
+	print*, filename
+
+	print*,'Enter the sampling of the map (A/pixel)'
+	read*, sampling
+
 !C 1. read coords
       write (6,*) 'Enter the filename of the search object map    '
-      read (5,10) filename
+      read (5,10) sobjfilename
 10    format(A40)
-      print*, filename
+      print*, sobjfilename
 
-	print*,"Enter the mask map filename. (This is mask just for presentation, it won't affect the origin of rotation.)"
-	read*,maskfilename
+!	print*,"Enter the mask map filename. (This is mask just for presentation, it won't affect the origin of rotation.)";
+!	read*,maskfilename
 	
-	print*,'Enter a threshold for the mask map.'
-	read*,maskth
+!	print*,'Enter a threshold for the mask map.'
+!	read*,maskth
 	
 
 
@@ -124,8 +168,8 @@
        pdbfile = 'com'//num(runcode)//'.pdb'       
 ! pdb file com used to record origin of rotations.
 
-      write (6,*) 'Enter the sampling in Angstoms.'
-      read*,sampling
+!      write (6,*) 'Enter the sampling in Angstoms.'
+!      read*,sampling
 
 !      write (6,*) 'Enter the filename of the doc file    '
 !      read (5,10) filename2
@@ -140,10 +184,10 @@
       print *, r1,r2
 
 
-      write (6,*) 'Enter a size (in pixels) for the extracted objects.'
-      read *, size
-      print*,size
-    
+ !     write (6,*) 'Enter a size (in pixels) for the extracted objects.'
+ !     read *, size
+ !     print*,size
+ ! v1.3 .. make match the sobj in size.   
 
  	print*, pdbfile
 
@@ -167,8 +211,23 @@
 	comy=comy/sampling
 	comz=comz/sampling
 
-
         PRINT*,'COM of sobj at ',comx,comy,comz,' pixels.'
+
+! calc nearest integer posn of com
+! don't really need this. by definition the com will be the centre of the boxes windowed.
+! actually this is all not neccessary. wan't objects to line up with the sobj, therefore the centre must be centre of sobj box, not the com,
+! this will now be V1.3
+! give a warning that sobj must be rotatable within own box, i.e. write out when corners zeroed, to show what could be cut off. 
+! apply the corners cuttoff in the search. 
+! or..write out a version of the sobj centered by com.
+
+
+!v1.3 make nearest integer   
+
+	comx=int((comx/1)+.5)
+	comy=int((comy/1)+.5)
+	comz=int((comz/1)+.5)
+	print*,comx,comy,comz
    
 !C ***************************************************
 
@@ -182,6 +241,18 @@
 
 !C ***********************************************
 !C open input from MRC format file
+
+        
+        CALL IMOPEN(2,sobjfilename,'RO')
+        CALL IRDHDR(2,NXYZ,MXYZ,MODE,DMIN,DMAX,DMEAN)
+
+	lx=nx
+	ly=ny
+	lz=nz
+
+        if ((lx.ne.ly).or.(lx.ne.lz)) then stop "Search object map must be cubic."
+!        CALL IMCLOSE(2)
+
         
         CALL IMOPEN(1,filename,'RO')
         CALL IRDHDR(1,NXYZ,MXYZ,MODE,DMIN,DMAX,DMEAN)
@@ -202,11 +273,14 @@
 
 
 !C allocate  image_files 
-        allocate (map(0:nxp1,0:nym1,0:nzm1),mask(0:nxp1,0:nym1,0:nzm1) )    
-       
+        allocate (map(0:nxp1+lx,0:nym1+ly,0:nzm1+lz),mask(0:nxp1,0:nym1,0:nzm1) )    
+        map=0
        
+        bnx=nx+lx
+        bny=ny+ly
+	bnz=nz+lz
 
-        CALL READMRCMAP(1,filename,map,nx,ny,nz,T,F,T,err) !read, is open, dont close, add 2 bytes
+        CALL READMRCMAP3(1,filename,map,bnx,bny,bnz,T,F,T,err,nx,ny,nz) !read, is open, dont close, add 2 bytes
         
 !C ***************************************************
 
@@ -217,28 +291,28 @@
 	newxyz(1:3)=size
 	
 
-        CALL IMOPEN(3,maskfilename,'RO')
-        CALL IRDHDR(3,NXYZ,MXYZ,MODE,DMIN,DMAX,DMEAN)
+!        CALL IMOPEN(3,maskfilename,'RO')
+!        CALL IRDHDR(3,NXYZ,MXYZ,MODE,DMIN,DMAX,DMEAN)
 
-        CALL READMRCMAP(3,maskfilename,mask,nx,ny,nz,T,F,T,err) !read, not open,  close, add 2 bytes  
+!        CALL READMRCMAP(3,maskfilename,mask,nx,ny,nz,T,F,T,err) !read, not open,  close, add 2 bytes  
      
 !make mask binary
 
 ! vs maskth
-	where (mask.lt.maskth) 
-			  mask=0.0
-	elsewhere
-			  mask=1.0
-	end where
+!	where (mask.lt.maskth) 
+!			  mask=0.0
+!	elsewhere
+!			  mask=1.0
+!	end where
 
-        print*,"Sum in mask = ", sum(mask)
+ !       print*,"Sum in mask = ", sum(mask)
 
-        map=map*mask
+ !       map=map*mask
 
-        print*,"Sum in masked map = ", sum(map)
+ !       print*,"Sum in masked map = ", sum(map)
 
  
-         allocate (obj(0:nxp1,0:nym1,0:nzm1),rotobj(0:nxp1,0:nym1,0:nzm1))      
+         allocate (obj(0:lx+1,0:ly-1,0:lz-1),rotobj(0:lx+1,0:ly-1,0:lz-1))      
          rotobj=1
      
               
@@ -246,9 +320,9 @@
 !C write top set of hits here
 	count=0
 	
-                comx=comx+cx
-	        comy=comy+cy
-                comz=comz+cz
+        !        comx=comx+cx
+	 !       comy=comy+cy
+          !      comz=comz+cz
 
 
 	do N=1,r2
@@ -257,24 +331,33 @@
 		read (4,510) ji1,x,y,z,angnum,b,psi,theta,phi,j2,ccc
 		 print*,35
 		 
-		shx=-x
-		shy=-y
-		shz=-z
+		shx=x-comx+cx
+		shy=y-comy+cy
+		shz=z-comz+cz
 
 
 		if (ji1.ge.r1) then
                	       
-			obj=map
-			call rotobjNI(obj,rotobj,psi,theta,phi,nx,ny,nz,comx,comy,comz)
-		
-                        rotobj= eoshift(rotobj,shift=shx,boundary=0.0,dim=1)
-                        rotobj= eoshift(rotobj,shift=shy,boundary=0.0,dim=2)
-                        rotobj= eoshift(rotobj,shift=shz,boundary=0.0,dim=3)
+			sx=int(lx/2.)+1
+			sy=int(ly/2.)+1
+			sz=int(lz/2.)+1
+			fx=int(lx/2.)
+			fy=int(ly/2.)
+			fz=int(lz/2.)
+
+
+			obj=map(shx-sx:shx+fx,shy-sy:shy+fy,shz-sz:shz+fz)
+			call rotobjI(obj,rotobj,psi,theta,phi,lx,ly,lz,comx,comy,comz)
+		! do inverse rotn
+
+!                        rotobj= eoshift(rotobj,shift=shx,boundary=0.0,dim=1)
+ !                       rotobj= eoshift(rotobj,shift=shy,boundary=0.0,dim=2)
+  !                      rotobj= eoshift(rotobj,shift=shz,boundary=0.0,dim=3)
 
 				         
 			CALL IMOPEN(9,'maphit-run'//num(runcode)//'-'//num(ji1)//'.mrc','UNKNOWN')
-        		CALL ITRHDR(9,1)
-       			CALL WRITEMRCMAP(9,rotobj,nx,ny,nz,sampling,err) !open already, close it, subtract 2 from nx
+        		CALL ITRHDR(9,2)
+       			CALL WRITEMRCMAP(9,rotobj,lx,ly,lz,sampling,err) !open already, close it, subtract 2 from nx
 ! call ialsiz ? 
 		endif
 
@@ -313,12 +396,13 @@
 
 	contains         
                 
-        SUBROUTINE rotobjNI(obj,therotobj,psi,theta,phi,nx,ny,nz,cx,cy,cz)
+        SUBROUTINE rotobjI(obj,therotobj,psi,theta,phi,nx,ny,nz,cx,cy,cz)
            
 
 !c sub to apply rotation about centre of map to map object 
 !c convert rotations to matrix form
 !C spider convention for euler angles to mat
+! invert
 
 ! the object maps are nxp1,ny,nz in size
 !rotn centre is nx/2,ny/2,nz/2
@@ -378,6 +462,8 @@
         call matmult(mata,mat,matd)
 
 
+        mate=transpose(matd) !inverse of rotation is done
+
         do iz=0,nzm1
                 do iy=0,nym1
                         do ix=0,nxm1
@@ -388,7 +474,7 @@
 
                                 val2=obj(ix,iy,iz)
                                                              
-				 call matmult2(x,y,z,matd,cx,cy,cz)      !transform coords by rotn
+				 call matmult2(x,y,z,mate,cx,cy,cz)      !transform coords by rotn
 			
 
                                  call interpo3(therotobj,nx,ny,nz,x,y,z,val2)            !interpo density at newposn to new map.
@@ -400,7 +486,7 @@
        
         return
 
-        END SUBROUTINE rotobjNI
+        END SUBROUTINE rotobjI
 
 
         subroutine interpo3(map,nx,ny,nz,sx,sy,sz,val)
@@ -566,3 +652,6 @@
          
  
 
+! to do still : generalise windowing to avoid edge problems, use one of the window subroutines.
+
+!put in invert rotn
