@@ -12,7 +12,7 @@ Tests to make sure the Python environment (i.e. ccpem-python) is working as expe
 
 import unittest
 
-class Test(unittest.TestCase):
+class CCPEMPythonTest(unittest.TestCase):
     '''
     Tests for ccpem-python environment
     '''
@@ -47,3 +47,14 @@ class Test(unittest.TestCase):
     def test_mrcfile_import(self):
         import mrcfile
         assert mrcfile.__version__
+
+    def test_fdrcontrol_import(self):
+        import FDRcontrol
+        import confidenceMapUtil
+        assert hasattr(FDRcontrol, 'main')
+        assert hasattr(confidenceMapUtil, 'confidenceMapMain')
+
+    def test_gemmi(self):
+        # Example from Gemmi docs
+        import gemmi
+        assert str(gemmi.Op('x-y,x,z+1/6') * '-x,-y,z+1/2') == '<gemmi.Op("-x+y,-x,z+2/3")>'
