@@ -8,7 +8,7 @@ doc=storred_doc
 rm -fr ${rep}
 rm -f ${doc}
 
-open_after 0.05 &
+open_after 0.5 &
 
 p0=0.0
 p1=0.0
@@ -47,12 +47,16 @@ sleep ${ti}
 sleep ${ti}
 ./python_link demo.py -d ${doc} -p ${p1} -v -e Examples/7_aimless.log -s -n
 sleep ${ti}
-./python_link demo.py -d ${doc} -p ${p0} -v -e Examples/cofe_acorn.log -s
-sleep ${ti}
-junk() {
-./python_link demo.py -d ${doc} -p ${p0} -v -e Examples/18.11.27_stdout.log
-sleep ${ti}
+if test -f Examples/cofe_acorn.log; then
+  ./python_link demo.py -d ${doc} -p ${p0} -v -e Examples/cofe_acorn.log -s
+  sleep ${ti}
+fi
+skip() {
+  ./python_link demo.py -d ${doc} -p ${p0} -v -e Examples/18.11.27_stdout.log
+  sleep ${ti}
 }
+./python_link demo.py -d ${doc} -p ${p1} -v Examples/19.11.29_phaser_stdout.log
+sleep ${ti}
 ./python_link demo.py /dev/null
 
 rm -f ./python_link
