@@ -40,11 +40,23 @@ class LogDataParser(object):
     patches = None,
     stop_on_error = True,
   ):
-    line = istream.readline()
+    line = istream.readline().decode('utf-8')
     line_cou = 1
     while line:
+      if line.endswith('\r\n'):
+        line = line[:-2] + '\n'
+
       if ostream:
-        ostream.write(line)
+        try:
+          ostream.write(line)
+
+        except UnicodeEncodeError:
+          # requires to deal with andgstrom sign in phaser
+          ostream.write(line.encode('utf-8'))
+
+        except:
+          pass
+
         ostream.flush()
 
       if patches:
@@ -70,7 +82,7 @@ class LogDataParser(object):
 
       except StopIteration:
         if self.recovery_list:
-          if verbose:
+          if verbose and ostream:
             ostream.write('|++++++++++++++++++\n')
             ostream.write('|WARNING, giving up on line %d' %line_cou + '\n')
             ostream.write('|' + line.rstrip() + '\n')
@@ -89,7 +101,7 @@ class LogDataParser(object):
         if stop_on_error:
           raise
 
-        elif verbose:
+        elif verbose and ostream:
           ostream.write('|++++++++++++++++++\n')
           ostream.write('|WARNING, giving up on line %d' %line_cou + '\n')
           ostream.write('|' + line.rstrip() + '\n')
@@ -103,6 +115,6 @@ class LogDataParser(object):
 
           ostream.write('|++++++++++++++++++\n')
 
-      line = istream.readline()
+      line = istream.readline().decode('utf-8')
       line_cou += 1
 
