@@ -22,10 +22,10 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'CONFINCLUDEDIR': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/include',
  'CONFINCLUDEPY': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/include/python2.7',
  'COREPYTHONPATH': ':plat-linux2:lib-tk:lib-old',
- 'COVERAGE_INFO': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/builddir/Python-2.7.15.tgz/coverage.info',
- 'COVERAGE_REPORT': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/builddir/Python-2.7.15.tgz/lcov-report',
+ 'COVERAGE_INFO': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/builddir/Python-2.7.18.tgz/coverage.info',
+ 'COVERAGE_REPORT': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/builddir/Python-2.7.18.tgz/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov report"',
- 'CPPFLAGS': '-I. -IInclude -I/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.15/Include',
+ 'CPPFLAGS': '-I. -IInclude -I/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.18/Include',
  'CXX': 'g++ -pthread',
  'C_THREADS': 0,
  'DESTDIRS': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install /home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib /home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib/python2.7 /home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib/python2.7/lib-dynload',
@@ -405,7 +405,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'MAINCC': 'gcc -pthread',
  'MAJOR_IN_MKDEV': 0,
  'MAJOR_IN_SYSMACROS': 0,
- 'MAKESETUP': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.15/Modules/makesetup',
+ 'MAKESETUP': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.18/Modules/makesetup',
  'MANDIR': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/share/man',
  'MEMTESTOPTS': '-l -x test_subprocess test_io test_lib2to3 \\ -x test_dl test___all__ test_fork1 \\',
  'MKDIR_P': '/bin/mkdir -p',
@@ -453,7 +453,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'PYTHON_FOR_REGEN': 'python2.7',
  'PYTHON_HEADERS': '\\',
  'PYTHON_OBJS': '\\',
- 'PY_CFLAGS': '-fno-strict-aliasing -O3 -fno-strict-aliasing -pipe -march=core2 -mtune=generic -DNDEBUG -g -fwrapv -O3 -Wall -Wstrict-prototypes -I. -IInclude -I/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.15/Include -fPIC -DPy_BUILD_CORE',
+ 'PY_CFLAGS': '-fno-strict-aliasing -O3 -fno-strict-aliasing -pipe -march=core2 -mtune=generic -DNDEBUG -g -fwrapv -O3 -Wall -Wstrict-prototypes -I. -IInclude -I/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.18/Include -fPIC -DPy_BUILD_CORE',
  'PY_FORMAT_LONG_LONG': '"ll"',
  'PY_FORMAT_SIZE_T': '"z"',
  'PY_UNICODE_TYPE': 0,
@@ -465,7 +465,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'RANLIB': 'ranlib',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/builddir/Python-2.7.15.tgz:/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib:/opt/software/cuda/8.0/lib64:/opt/rh/devtoolset-3/root/usr/lib64:/opt/rh/devtoolset-3/root/usr/lib',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/builddir/Python-2.7.18.tgz:/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib:/opt/software/cuda/8.0/lib64:/opt/rh/devtoolset-3/root/usr/lib64:/opt/rh/devtoolset-3/root/usr/lib',
  'SCRIPTDIR': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib',
  'SETPGRP_HAVE_ARG': 0,
  'SGI_ABI': '',
@@ -493,8 +493,8 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'SIZEOF_WCHAR_T': 4,
  'SIZEOF__BOOL': 1,
  'SO': '.so',
- 'SRCDIRS': 'Parser Objects Python Modules',
- 'SRC_GDB_HOOKS': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.15/Tools/gdb/libpython.py',
+ 'SRCDIRS': 'Parser Objects Python Modules Modules/_io',
+ 'SRC_GDB_HOOKS': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.18/Tools/gdb/libpython.py',
  'STDC_HEADERS': 1,
  'STRICT_SYSV_CURSES': "/* Don't use ncurses extensions */",
  'STRINGLIB_HEADERS': '\\',
@@ -507,8 +507,8 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'TCLTK_LIBS': '',
  'TESTOPTS': '-l',
  'TESTPATH': '',
- 'TESTPROG': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.15/Lib/test/regrtest.py',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/builddir/Python-2.7.15.tgz:/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib:/opt/software/cuda/8.0/lib64:/opt/rh/devtoolset-3/root/usr/lib64:/opt/rh/devtoolset-3/root/usr/lib ./python -Wd -3 -E -tt',
+ 'TESTPROG': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.18/Lib/test/regrtest.py',
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/builddir/Python-2.7.18.tgz:/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib:/opt/software/cuda/8.0/lib64:/opt/rh/devtoolset-3/root/usr/lib64:/opt/rh/devtoolset-3/root/usr/lib ./python -Wd -3 -E -tt',
  'TESTPYTHONOPTS': '',
  'THREADOBJ': 'Python/thread.o',
  'TIME_WITH_SYS_TIME': 1,
@@ -520,7 +520,7 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'USE_TOOLBOX_OBJECT_GLUE': 0,
  'VA_LIST_IS_ARRAY': 1,
  'VERSION': '2.7',
- 'VPATH': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.15',
+ 'VPATH': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.18',
  'WANT_SIGFPE_HANDLER': 0,
  'WANT_WCTYPE_FUNCTIONS': 0,
  'WINDOW_HAS_FLAGS': 1,
@@ -534,11 +534,11 @@ build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/builddir/Python-2.7.15.tgz',
- 'abs_srcdir': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.15',
+ 'abs_builddir': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/builddir/Python-2.7.18.tgz',
+ 'abs_srcdir': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.18',
  'build': 'x86_64-pc-linux-gnu',
  'datarootdir': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/share',
  'exec_prefix': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install',
  'host': 'x86_64-pc-linux-gnu',
  'prefix': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install',
- 'srcdir': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.15'}
+ 'srcdir': '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/checkout/Python-2.7.18'}
