@@ -1,4 +1,4 @@
-# Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2018 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
 # This file is part of PyQt4.
 # 
@@ -25,13 +25,13 @@ import sipconfig
 # These are installation specific values created when PyQt4 was configured.
 _pkg_config = {
     'pyqt_bin_dir':      '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/devbin',
-    'pyqt_config_args':  '--verbose --confirm-license --no-designer-plugin --no-qsci-api -e QtCore -e QtGui -e QtNetwork -e QtScript -e QtXml -e QtOpenGL -e QtSql -e QtSvg -e QtTest -e QtWebKit -e QtXmlPatterns --bindir=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/devbin --destdir=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib/python2.7/site-packages --sipdir=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/share/sip LFLAGS+=-L/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib -s -Wl,--hash-style=both -Wl,--build-id=sha1',
+    'pyqt_config_args':  '--confirm-license --no-designer-plugin --no-qsci-api -e QtCore -e QtGui -e QtNetwork -e QtScript -e QtXml -e QtOpenGL -e QtSql -e QtSvg -e QtTest -e QtWebKit -e QtXmlPatterns --bindir=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/devbin --sipdir=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/share/sip LFLAGS+=-L/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib -s -Wl,--hash-style=both -Wl,--build-id=sha1 --destdir=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib/python2.7/site-packages',
     'pyqt_mod_dir':      '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib/python2.7/site-packages/PyQt4',
-    'pyqt_modules':      'QtCore QtGui QtNetwork QtScript QtXml QtOpenGL QtSql QtSvg QtTest QtWebKit QtXmlPatterns',
+    'pyqt_modules':      'QtCore QtGui QtNetwork QtScript QtOpenGL QtSql QtSvg QtTest QtWebKit QtXml QtXmlPatterns',
     'pyqt_sip_dir':      '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/share/sip',
-    'pyqt_sip_flags':    '-x VendorID -t WS_X11 -x PyQt_SessionManager -x PyQt_NoPrintRangeBug -t Qt_4_8_6 -x Py_v3 -g',
-    'pyqt_version':      0x040b04,
-    'pyqt_version_str':  '4.11.4',
+    'pyqt_sip_flags':    '-n PyQt4.sip -x VendorID -t WS_X11 -x PyQt_SessionManager -x PyQt_NoPrintRangeBug -t Qt_4_8_6 -x Py_v3 -g',
+    'pyqt_version':      0x040c03,
+    'pyqt_version_str':  '4.12.3',
     'qt_archdata_dir':   '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install',
     'qt_data_dir':       '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install',
     'qt_dir':            '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install',
