@@ -23,6 +23,8 @@ import sys
 import os
 import scipy.cluster.hierarchy as hier
 try:
+    import matplotlib
+    #matplotlib.use('Agg')
     import matplotlib.gridspec as gridspec
     import matplotlib.cm as cm
     import matplotlib.gridspec as gridspec
@@ -432,7 +434,7 @@ class Plot:
         ###colorlist = ['darkcyan','brown','gray','darkgreen','darkkhaki','coral','indigo','maroon']#,'darkkhaki'
         colormap = plt.cm.Spectral#Set1,Spectral#YlOrRd,Spectral,BuGn,Set1,Accent,spring
         if len(dict_points) < 4: colormap = plt.cm.gist_earth
-        plt.gca().set_color_cycle([colormap(i) for i in nplinspace(0, 1, len(dict_points)+1)])
+        plt.gca().set_prop_cycle('color', [colormap(i) for i in nplinspace(0, 1, len(dict_points)+1)])
         
         if ylim is not None: plt.gca().set_ylim(ylim)
         #,'legend.markerscale': 5}
