@@ -186,9 +186,7 @@ def fit_protein_stepped_refine_function(res_spec, imol_map, use_rama = False):
     chain_id = res_spec[1]
     res_no   = res_spec[2]
     ins_code = res_spec[3]
-    current_steps_per_frame = dragged_refinement_steps_per_frame()
     current_rama_state = refine_ramachandran_angles_state()
-    set_dragged_refinement_steps_per_frame(400)
     
     for alt_conf in residue_alt_confs(imol, chain_id, res_no, ins_code):
         if use_rama:
@@ -206,7 +204,6 @@ def fit_protein_stepped_refine_function(res_spec, imol_map, use_rama = False):
             rotate_y_scene(10, 0.3)    
 
     set_refine_ramachandran_angles(current_rama_state)
-    set_dragged_refinement_steps_per_frame(current_steps_per_frame)
 
     
 def fit_protein_rama_fit_function(res_spec, imol_map):
@@ -506,18 +503,15 @@ def stepped_refine_protein_for_rama(imol):
     if (not valid_map_molecule_qm(imol_map)):
         info_dialog("Oops, must set map to refine to")
     else:
-        current_steps_frame = dragged_refinement_steps_per_frame()
         current_rama_state  = refine_ramachandran_angles_state()
         def refine_func(chain_id, res_no):
             set_go_to_atom_chain_residue_atom_name(chain_id, res_no, "CA")
             refine_auto_range(imol, chain_id, res_no, "")
             accept_regularizement()
 
-        set_dragged_refinement_steps_per_frame(400)
         set_refine_ramachandran_angles(1)
         stepped_refine_protein_with_refine_func(imol, refine_func, 1)
         set_refine_ramachandran_angles(current_rama_state)
-        set_dragged_refinement_steps_per_frame(current_steps_frame)
 
 #
 def stepped_refine_protein_with_refine_func(imol, refine_func, res_step):
@@ -823,11 +817,14 @@ def pepflip_active_residue():
        print "No active atom"
     else:
        imol       = active_atom[0]
-       chain_id   = active_atom[1]
-       res_no     = active_atom[2]
-       ins_code   = active_atom[3]
-       atom_name  = active_atom[4]
-       alt_conf   = active_atom[5]
+
+       ca = closest_atom_raw() # don't map to CA
+
+       chain_id   = ca[1]
+       res_no     = ca[2]
+       ins_code   = ca[3]
+       atom_name  = ca[4]
+       alt_conf   = ca[5]
 
        if (atom_name == " N  "): # PDBv3 fixme
 	   res_no -= 1;
