@@ -1,8 +1,8 @@
 import os
 # unset so to trigger exceptions and track use: RDBaseDir=os.environ['RDBASE']
-RDCodeDir=os.path.join(r'/home/jenkins/workspace/ccpem/sl6_devtoolset/devtools/install/lib/python2.7/site-packages','rdkit')
+RDCodeDir=os.path.join(r'/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib/python2.7/site-packages','rdkit')
 # not really hard-coded alternative RDCodeDir=os.path.dirname(__file__)
-_share = os.path.join(r'/home/jenkins/workspace/ccpem/sl6_devtoolset/devtools/install', r'share/RDKit')
+_share = os.path.join(r'/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install', r'share/RDKit')
 RDDataDir=os.path.join(_share,'Data')
 RDDocsDir=os.path.join(_share,'Docs')
 RDProjDir=os.path.join(_share,'Projects')
