@@ -35,7 +35,7 @@ _pkg_config = {
     'py_conf_inc_dir':    '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/include/python2.7',
     'py_inc_dir':         '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/include/python2.7',
     'py_lib_dir':         '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib/python2.7/config',
-    'py_version':         0x02070f,
+    'py_version':         0x020712,
     'qt_framework':       0,
     'sip_bin':            '/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/devbin/sip',
     'sip_config_args':    '--sip-module PyQt4.sip --bindir=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/devbin --destdir=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib/python2.7/site-packages --incdir=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/include --sipdir=/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/share/sip LFLAGS+=-L/home/jenkins/workspace/CCP-EM/sl6_devtoolset/devtools/install/lib -s -Wl,--hash-style=both -Wl,--build-id=sha1',
@@ -2147,7 +2147,7 @@ def _quote(s):
     if sys.platform == "win32":
         s = s.replace("/", "\\")
 
-    if s.find(" ") >= 0:
+    if " " in s or ":" in s or sys.platform == "win32":
         s = '"' + s + '"'
 
     return s
