@@ -55,15 +55,25 @@
        (lambda ()
 	 (water-coordination-gui)))
 
-		(add-simple-coot-menu-menuitem
-		 menu "Validation Outliers"
-		 (lambda ()
-			(using-active-atom
-			 (let ((imol-map (imol-refinement-map)))
-				(if (not (valid-map-molecule? imol-map))
-					 (add-status-bar-text "Refinement Map is currently not set")
-					 (validation-outliers-dialog aa-imol imol-map))))))))
+      (add-simple-coot-menu-menuitem
+       menu "Atom Overlaps"
+       (lambda ()
+         (using-active-atom
+          (molecule-atom-overlaps-gui aa-imol))))
 
+      (add-simple-coot-menu-menuitem
+       menu "Pepflips from Difference Map..."
+       (lambda ()
+         (pepflips-by-difference-map-gui)))
+
+      (add-simple-coot-menu-menuitem
+       menu "Validation Outliers"
+       (lambda ()
+           (using-active-atom
+           (let ((imol-map (imol-refinement-map)))
+              (if (not (valid-map-molecule? imol-map))
+                 (add-status-bar-text "Refinement Map is currently not set")
+                 (validation-outliers-dialog aa-imol imol-map))))))))
 
 ;(define (add-module-user-defined-restraints)
 ;  (if (defined? 'coot-main-menubar)
@@ -479,6 +489,12 @@
 	;;
 	;; ---------------------------------------------------------------------
 
+	(add-simple-coot-menu-menuitem
+	 submenu-models "Add Hydrogens"
+	 (lambda ()
+	   (using-active-atom
+	    (coot-reduce aa-imol))))
+
 	(add-simple-coot-menu-menuitem
 	 submenu-models "Add Hydrogens using Refmac"
 	 (lambda ()
@@ -497,14 +513,12 @@
 				 (lambda (imol)
 				   (move-waters-to-around-protein imol)))))
 
-
       (add-simple-coot-menu-menuitem 
        submenu-models "Assign (force) HETATMs for this Residue"
        (lambda ()
 	 (using-active-atom
 	  (hetify-residue aa-imol aa-chain-id aa-res-no aa-ins-code))))
 
-
 	(add-simple-coot-menu-menuitem
 	 submenu-models "Assign HETATM to molecule..."
 	 (lambda() 
@@ -519,7 +533,6 @@
 				 (lambda (imol)
 				   (copy-molecule imol)))))
 
-
 	(add-simple-coot-menu-menuitem
 	 submenu-models "Copy Fragment..."
 	 (lambda ()
