@@ -2,7 +2,7 @@
 /*
  *  =================================================================
  *
- *    18.12.17   <--  Date of Last Modification.
+ *    16.04.18   <--  Date of Last Modification.
  *                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  *  -----------------------------------------------------------------
  *
@@ -13,7 +13,7 @@
  *  **** Content :  Common GUI widgets
  *       ~~~~~~~~~
  *
- *  (C) E. Krissinel, A. Lebedev 2016-2017
+ *  (C) E. Krissinel, A. Lebedev 2016-2018
  *
  *  =================================================================
  *
@@ -83,6 +83,7 @@ Widget.prototype.setTooltip = function ( text )  {
   return this;
 }
 
+
 Widget.prototype.setWidth = function ( width )  {
   this.element.style.width = width;
   return this;
@@ -308,6 +309,11 @@ Widget.prototype.addOnChangeListener = function ( listener_func )  {
   return this;
 }
 
+Widget.prototype.addOnInputListener = function ( listener_func )  {
+  this.element.addEventListener('input',listener_func );
+  return this;
+}
+
 Widget.prototype.emitSignal = function ( signal,data )  {
   var event = new CustomEvent ( signal,{
     'detail' : data
@@ -405,6 +411,17 @@ Grid.prototype.getNRows = function()  {
   return this.element.rows.length;
 }
 
+Grid.prototype.truncateRows = function ( row )  {
+  while (this.element.rows.length>row+1)
+    this.element.deleteRow ( -1 );  // deletes last row
+  return this.element.rows.length;
+}
+
+Grid.prototype.deleteRow = function ( row )  {
+  this.element.deleteRow ( row );
+  return this.element.rows.length;
+}
+
 Grid.prototype.getNCols = function()  {
   var ncols = 0;
   for (var i=0;i<this.element.rows.length;i++)
@@ -678,6 +695,10 @@ IconLabel.prototype.setIconLabel = function ( text,icon_uri )  {
   }
 }
 
+IconLabel.prototype.setBackground = function ( icon_uri )  {
+  if (icon_uri.length>0)
+    $(this.element).css({'background-image':'url("'+icon_uri+'")'});
+}
 
 
 // -------------------------------------------------------------------------
@@ -702,6 +723,7 @@ InputText.prototype.setStyle = function ( type,pattern,placeholder,tooltip )  {
     else if ((pattern=='real') || (pattern=='real_'))
       this.element.setAttribute ( 'pattern','^[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?$' );
     else
+      //this.element.pattern = pattern;
       this.element.setAttribute ( 'pattern',pattern );
   }
   return this;
@@ -732,10 +754,12 @@ InputText.prototype.setValue = function ( text )  {
   return this.element.value = text;
 }
 
+/*
 InputText.prototype.addOnInputListener = function ( listener_func )  {
   this.element.addEventListener('input',listener_func );
   return this;
 }
+*/
 
 InputText.prototype.setOnEnterListener = function ( socket_function )  {
   (function(self){
@@ -836,7 +860,6 @@ Button.prototype.setEnabled = function ( enabled_bool )  {
   return this;
 }
 
-
 Button.prototype.isDisabled = function()  {
   return $(this.element).button ( 'option','disabled' );
 }
@@ -909,6 +932,7 @@ RadioSet.prototype = Object.create ( Widget.prototype );
 RadioSet.prototype.constructor = RadioSet;
 
 RadioSet.prototype.addButton = function ( text,btnId,tooltip,checked_bool )  {
+
   var _id = this.element.id + '_' + btnId;
 
   var label = new Widget ( 'label' );
@@ -1110,6 +1134,7 @@ SelectFile.prototype.getFiles = function()  {
   return this.element.files;
 }
 
+
 // -------------------------------------------------------------------------
 // ToolBar class
 
@@ -1125,16 +1150,51 @@ ToolBar.prototype.constructor = ToolBar;
 // -------------------------------------------------------------------------
 // IFrame class
 
+/*
+function IFrame ( uri )  {
+  Widget.call ( this,'object' );
+  if (uri.length>0)
+    this.element.setAttribute ( 'data',uri );
+  $(this.element).css ( {'border':'none'} );
+}
+
+IFrame.prototype = Object.create ( Widget.prototype );
+IFrame.prototype.constructor = IFrame;
+
+IFrame.prototype.loadPage = function ( uri )  {
+  //this.setVisible ( false );
+  this.element.data = uri;
+}
+
+IFrame.prototype.setHTML = function ( html )  {
+  //this.setVisible ( false );
+  this.element.data = 'data:text/html;charset=utf-8,' + encodeURI(html);
+}
+
+IFrame.prototype.getURL = function()  {
+  return this.element.data;
+}
+
+IFrame.prototype.clear = function()  {
+  //this.setVisible ( false );
+  this.element.data = 'about:blank';
+}
+
+IFrame.prototype.reload = function()  {
+  //this.setVisible ( false );
+  this.element.data = this.element.src;
+}
+*/
+
 function IFrame ( uri )  {
   Widget.call ( this,'iframe' );
   if (uri.length>0)
     this.element.setAttribute ( 'src',uri );
+  //this.element.setAttribute ( 'sandbox','allow-same-origin' );
   $(this.element).css ( {'border':'none'} );
-  /*
-  (function(iframe){
-    iframe.element.onload = function(){ iframe.setVisible(true); };
-  }(this))
-  */
+  //(function(iframe){
+  //  iframe.element.onload = function(){ iframe.setVisible(true); };
+  //}(this))
 }
 
 IFrame.prototype = Object.create ( Widget.prototype );
